package org.eclipse.jet.compiled;

import org.eclipse.jet.JET2Context;
import org.eclipse.jet.JET2Template;
import org.eclipse.jet.JET2Writer;
import org.eclipse.bpel.model.*;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.partnerlinktype.*;
import org.spagic.bpel.ode.utils.*;
import org.spagic.bpel.ode.DeployActivator;

public class _jet_deploy implements JET2Template {

    public _jet_deploy() {
        super();
    }

    private static final String NL = System.getProperty("line.separator"); //$NON-NLS-1$

    public void generate(final JET2Context context, final JET2Writer __out) {
        JET2Writer out = __out;
 	
	Process process = (Process)context.getSource();
	String processName = process.getName();
	String targetNamespace = process.getTargetNamespace();
	

        out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("<deploy xmlns=\"");  //$NON-NLS-1$        
        out.write(IConstants.XMLNS_ODE_DEPLOY);
        out.write("\">");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("  ");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("\t<process xmlns:pns=\"");  //$NON-NLS-1$        
        out.write(targetNamespace);
        out.write("\" name=\"pns:");  //$NON-NLS-1$        
        out.write(processName);
        out.write("\" >");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("\t\t<in-memory>true</in-memory>");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("    \t<active>true</active>    ");  //$NON-NLS-1$        
        out.write(NL);         

	/****** for every PartnerLink if myRole -> <provide>, partnerRole  -> <invoke> *****/
	PartnerLinks partnerLinks = process.getPartnerLinks();
	WsdlParserUtil wsdlParser = (WsdlParserUtil)context.getVariable("wsdlParser");
	for (PartnerLink partnerLink : partnerLinks.getChildren()) {
		Role myRole = partnerLink.getMyRole();
		Role partnerRole = partnerLink.getPartnerRole();
		String plnkName = partnerLink.getName();
		String roleName = "";
		if ( myRole != null ) {
			/************ 
			 simplification: if MyRole is not null soap:binding and wsdl:service are relative 
			 to bpel call (receive) generated automatically by plugin. 
			 ODE don't use the provide port; if port="" ODE correctly works, 
			 we set the port with the process Name 
			*************/

        out.write("    \t<provide partnerLink=\"");  //$NON-NLS-1$        
        out.write(plnkName);
        out.write("\">");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("\t\t\t<service xmlns:plnk=\"");  //$NON-NLS-1$        
        out.write(targetNamespace);
        out.write("\" name=\"plnk:");  //$NON-NLS-1$        
        out.write(processName+IConstants.SUFFIX_SERVICE);
        out.write("\" port=\"");  //$NON-NLS-1$        
        out.write(processName);
        out.write("\" />");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("    \t</provide>");  //$NON-NLS-1$        
        out.write(NL);         

	}
	if ( partnerRole != null ) { 
			/************ 
			simplification: if partnerRole is not null the port and service are relative 
			to web services invoked by bpel (in the invoke task). 
			In this release, I omit the asynchronous case. 
			***************/ 
		roleName = partnerRole.getName(); 	
		WSBindingInfo wsBindingInfo = wsdlParser.getWSBindingInfo(plnkName, roleName);
		if (wsBindingInfo == null)
			throw new RuntimeException();

        out.write("    \t<invoke partnerLink=\"");  //$NON-NLS-1$        
        out.write(plnkName);
        out.write("\">");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("\t\t\t<service xmlns:plnk=\"");  //$NON-NLS-1$        
        out.write(wsBindingInfo.getTargetNamespace());
        out.write("\" name=\"plnk:");  //$NON-NLS-1$        
        out.write(wsBindingInfo.getService());
        out.write("\" port=\"");  //$NON-NLS-1$        
        out.write(wsBindingInfo.getPort());
        out.write("\" />");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("    \t</invoke>");  //$NON-NLS-1$        
        out.write(NL);         

	}
	}//end for

        out.write("\t</process>");  //$NON-NLS-1$        
        out.write(NL);         
        out.write("</deploy>\t");  //$NON-NLS-1$        
        out.write(NL);         
    }
}
