/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.bpel.ode.log;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Status;
import org.spagic.bpel.ode.DeployActivator;



public class SpagicLogger {		
	//
	// Pass pluginID
	public static void info(String msg) {
	      info(DeployActivator.PLUGIN_ID, msg);
	}
	
	public static void warning(String msg) {
	    warning(DeployActivator.PLUGIN_ID, msg);
	}
	
	public static void warning(String msg, Throwable e) {
		warning(DeployActivator.PLUGIN_ID, msg, e); 
	}
		
	public static void error(String msg) {
		error(DeployActivator.PLUGIN_ID, msg);
	}

	public static void error(String msg, Throwable e) {
		error(DeployActivator.PLUGIN_ID, msg, e);
	}	
	
	// Pass pluginID
	public static  void info(String pluginID, String msg) {
		DeployActivator.getDefault().getLog().log(new Status(Status.INFO, pluginID, Status.INFO, msg, null));
	}
	
	public static  void warning(String pluginID, String msg) {
	    warning(pluginID, msg, null);
	}
	
	public static  void warning(String pluginID, String msg, Throwable e) {
		DeployActivator.getDefault().getLog().log(new Status(Status.WARNING, pluginID, Status.WARNING, msg, getCause(e)));
	}
	
	public static  void error(String pluginID, String msg) {
	      error(pluginID, msg, null);
	}
	
	public static void error(String pluginID, String msg, Throwable e) {
		DeployActivator.getDefault().getLog().log(new Status(Status.ERROR, pluginID, Status.ERROR, msg, getCause(e)));
	}

    /**
     * This class must not be called outside this class
     * Utility method to get the cause of an exception.
     * @param exception : the exception to analyse
     * @return Throwable
     */
    public static Throwable getCause(final Throwable exception) {
        // Figure out which exception should
    	//actually be logged -- if the given exception is
        // a wrapper, unwrap it
        Throwable cause = null;
        if (exception != null) {
            if (exception instanceof CoreException) {
                // Workaround: CoreException contains
            	//a cause, but does not actually implement getCause().
                // If we get a CoreException, we need to
            	//manually unpack the cause. Otherwise, use
                // the general-purpose mechanism.
            	//Remove this branch if CoreException ever implements
                // a correct getCause() method.
                CoreException ce = (CoreException) exception;
                cause = ce.getStatus().getException();
            } else {
            	// use reflect instead of a direct call
            	//to getCause(), to allow compilation against
            	//JCL Foundation (bug 80053)
            	try {
            		Method causeMethod =
            			exception.getClass().getMethod(
            					"getCause", new Class[0]);
            		Object o = causeMethod.invoke(
            				exception, new Object[0]);
            		if (o instanceof Throwable) {
            			cause = (Throwable) o;
            		}
            	} catch (NoSuchMethodException e) {
            		//ignore
					cause = null;
            	} catch (IllegalArgumentException e) {
            		// ignore
					cause = null;
				} catch (IllegalAccessException e) {
            		// ignore
					cause = null;
				} catch (InvocationTargetException e) {
            		// ignore
					cause = null;
				}
            }
            if (cause == null) {
                cause = exception;
            }
        }
        return cause;
    }
}