/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.bpel.ode.popup.actions;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.resource.BPELReader;
import org.eclipse.bpel.model.resource.BPELResource;
import org.eclipse.bpel.model.resource.BPELResourceFactoryImpl;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jet.JET2Platform;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.spagic.bpel.ode.log.SpagicLogger;
import org.spagic.bpel.ode.smx.utils.OdeSmxUtil;
import org.spagic.bpel.ode.smx.utils.ZipUtils;
import org.spagic.bpel.ode.utils.WsdlParserUtil;

public class DeployAction implements IObjectActionDelegate {

	private IFile bpelFile                 = null;
	static final String locationSuggestion = "http://host:port/processName";
	/**
	 * Constructor for Action1.
	 */
	public DeployAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		InputDialog inputDialog = new InputDialog(
			shell,
			"Ode Plug-in",
			"Insert process uri", locationSuggestion, null);
		
		inputDialog.open();
		String processUri = inputDialog.getValue();
		if (!processUri.equals(locationSuggestion))
			processUri = inputDialog.getValue();
		else{
			MessageDialog.openInformation(
					shell,
					"Ode Plug-in",
					"To deploy in ServiceMix you have to insert the process uri");
			return;
		}
			
		
		//Creation BPELResource
		ResourceSet resourceSet = new ResourceSetImpl();
		URI bpelUri = URI.createFileURI(bpelFile.getFullPath().toString());		
		InputStream inputStream = null;
		try {
			inputStream = resourceSet.getURIConverter().createInputStream(bpelUri);
		} catch(IOException ioe) {
			SpagicLogger.error("DeployAction: " +ioe);
			MessageDialog.openError(shell, "Ode Plug-in", "Errors during deploy....");
			return;
		}
		BPELResource bpelRes = (BPELResource)new BPELResourceFactoryImpl().createResource(bpelUri);
		BPELReader reader = new BPELReader();
		reader.read(bpelRes, inputStream);
		Process process = (Process)bpelRes.getContents().get(0);
		//Creation variables for the firts JET Transformation
		Map<String, String> varFirstTransformation = new HashMap<String, String>();
		varFirstTransformation.put("org.spagic.bpel.ode.directoryPrj", bpelFile.getProject().getName());
		IPath container = bpelFile.getParent().getLocation();
        String absPathBpel = container.toOSString();
        varFirstTransformation.put("org.spagic.bpel.ode.absPathBpel", absPathBpel);
        varFirstTransformation.put("org.spagic.bpel.ode.processUri", processUri);
		//call JET Transformation
		try {
			//first transformation generate deploy.xml and *Wrapped.wsdl
			JET2Platform.runTransformOnObject("org.spagic.bpel.ode", 
					process, varFirstTransformation , new NullProgressMonitor() );
			//second transformation generate zip for smx
			WsdlParserUtil wsdlParser = null;
			List<String> absoluteWSDLList = null;
			try {
				wsdlParser = new WsdlParserUtil(absPathBpel, process);
				absoluteWSDLList = wsdlParser.getWsdlWsInvoked();
			} catch (Exception ex){
				SpagicLogger.error("DeployAction: " +ex);
				throw new RuntimeException();
			}
			Map<String, String> varSecondTransformation = new HashMap<String, String>();	
			varSecondTransformation.put("processName",process.getName() );
			varSecondTransformation.put("absoluteWSDLList", OdeSmxUtil.getOneStringFromStringArray(absoluteWSDLList));
			varSecondTransformation.put("org.eclipse.jet.resource.project.name",bpelFile.getProject().getName());
			
			JET2Platform.runTransformOnObject("org.spagic.bpel.ode.smx", 
					null, varSecondTransformation, new NullProgressMonitor() );
			IFolder iFolder = bpelFile.getProject().getFolder("Bpel/"+process.getName() + "-sa");
			ZipUtils.getDefault().zipFolderAndSubFolder(iFolder);
		} catch (RuntimeException re) {
			SpagicLogger.error("DeployAction: " +re);
			MessageDialog.openError(
					shell,
					"Ode Plug-in",
					"Errors during deploy....");
			return;
		}
		
		
		
		MessageDialog.openInformation(
				shell,
				"Ode Plug-in",
				"Deploy in ServiceMix completed.");
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		setBpelFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}

	public IFile getBpelFile() {
		return bpelFile;
	}

	public void setBpelFile(IFile bpelFile) {
		this.bpelFile = bpelFile;
	}

}
