/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.bpel.ode.utils;

import org.dom4j.io.SAXReader;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.xpath.DefaultXPath;
import org.dom4j.XPath;
import org.dom4j.Node;
import org.eclipse.bpel.model.Import;
import org.eclipse.bpel.model.PartnerLink;
import org.eclipse.bpel.model.PartnerLinks;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.adapters.INamespaceMap;
import org.eclipse.bpel.model.partnerlinktype.Role;
import org.eclipse.bpel.model.util.BPELUtils;
import org.spagic.bpel.ode.log.SpagicLogger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

public class WsdlParserUtil {
	private Process process = null;
	private String bpelLocation = null;
	private Document bpelDocument = null;
	private HashMap<String, Document> wsdlDocumentMap = new HashMap<String, Document>();

	private HashMap<String, String> namespaces = new HashMap<String, String>();
	private INamespaceMap<String, String> namespacesBpel = null;
	
	public WsdlParserUtil(String absPathBpel, Process process) 
		throws Exception{
		this.process = process;
		this.bpelLocation = absPathBpel + "\\" + process.getName() + ".bpel";;
		SAXReader reader = new SAXReader();
		initNamespaces();
		initNamespacesBpel(process);
		try {
			bpelDocument = reader.read(new File(bpelLocation));
			String locationWsdl = "";
			Document wsdlDocument = null;
			/******* save into wsdlDocumentMap all document relative wsdl imported by bpel *****/
			for (Import processImport : process.getImports()) {
				if (processImport.getImportType().equals(IConstants.XMLNS_WSDL)) {
					locationWsdl = absPathBpel + "\\" + processImport.getLocation();
					wsdlDocument = reader.read(new File(locationWsdl));
					wsdlDocumentMap.put(locationWsdl, wsdlDocument);
				}
			}
		} catch(DocumentException de){
			SpagicLogger.error("WsdlParserUtil-dbg2", de);
			throw de;
		}
	}

	private void initNamespaces() {
		namespaces.put("bpws", IConstants.XMLNS_BPEL20_EXECUTABLE);
		namespaces.put("wsdl", IConstants.XMLNS_WSDL);
		namespaces.put("plnk", IConstants.XMLNS_PLNK);
	}
	
	private void initNamespacesBpel(Process process) {
		namespacesBpel = BPELUtils.getNamespaceMap(process);
	}
	
	public String getPortTypeByMyRole(String partnerLink, String role) {
		return getPortTypeByRole(partnerLink, role, true);
	}
	
	public String getPortTypeByPartnerRole(String partnerLink, String role) {
		return getPortTypeByRole(partnerLink, role, false);
	}
	
	//from *.bpel with partnerLink name and the role -> partnerLinkType
	//from *.wsdl with partnelLinkType and role -> portType
	private String getPortTypeByRole(String partnerLink, String role, boolean providerRole){
		Node plnkWithSpecificRole = getPlnkWithSpecificRole(partnerLink, role, providerRole);
		String partnerLinkType = plnkWithSpecificRole.valueOf("@partnerLinkType");
		int indexSeparator = partnerLinkType.indexOf(":");
		String prefix = partnerLinkType.substring(0, indexSeparator);
		String targetNamespace = namespacesBpel.get(prefix);
		partnerLinkType = partnerLinkType.substring(indexSeparator + 1);
		return getPortType(targetNamespace, partnerLinkType, role);
	}
	
	private Node getPlnkWithSpecificRole(String partnerLink, String role, boolean providerRole) {
		XPath xPathExpression = null;
		if (providerRole)
			xPathExpression = new DefaultXPath("/bpws:process/bpws:partnerLinks/bpws:partnerLink[@name='" + partnerLink + "' and @myRole='" + role +"']");
		else
			xPathExpression = new DefaultXPath("/bpws:process/bpws:partnerLinks/bpws:partnerLink[@name='" + partnerLink + "' and @partnerRole='" + role +"']");
		xPathExpression.setNamespaceURIs(namespaces);
		Node plnkWithSpecificRole = xPathExpression.selectSingleNode(bpelDocument);
		return plnkWithSpecificRole;
	}

	//retrieve the port type from a .wsdl relative to a bpel process
	public String getPortType(String targetNamespace, String plnkName, String roleName) {
		Set<String> wsdlLocationSet = wsdlDocumentMap.keySet();
		Iterator<String> it = wsdlLocationSet.iterator();
		Document document = null;
		String wsdlLocation = null;
		XPath xpRole = new DefaultXPath("/wsdl:definitions[@targetNamespace='" + targetNamespace + "']" +
										"/plnk:partnerLinkType[@name='" + plnkName+ "']" +
										"/plnk:role[@name='" + roleName + "']");
		xpRole.setNamespaceURIs(namespaces);
		while (it.hasNext()) {
			wsdlLocation = it.next();
			document = wsdlDocumentMap.get(wsdlLocation);
			Node roleNode = xpRole.selectSingleNode(document);
			if (roleNode != null)
				return roleNode.valueOf("@portType");
			else 
				continue;
		}
		System.out.println("WsdlParserUtil::getPortType: doesn't exist role node.");
		return null;
	}
	
	
	public WSBindingInfo getWSBindingInfo(String partnerLinkName, String role){
		String portType = getPortTypeByPartnerRole(partnerLinkName, role);
		if (portType == null)
			return null;
		int indexSeparator = portType.indexOf(":");
		String prefix = portType.substring(0, indexSeparator);
		String targetNamespace = getNamespaceFromArtifacts(prefix);
		portType = portType.substring(indexSeparator + 1);
		String[] serviceAndPort = getServiceAndPort(targetNamespace, portType);
		return new WSBindingInfo(targetNamespace, serviceAndPort[0], serviceAndPort[1]);
	}
	
	private String getNamespaceFromArtifacts(String prefix){
		//Impossibile recuperare dal file *Artifacts.wsdl i namespace a partire dal prefisso
		//Funzione importante perch leggo i port type ma devo sapere in quali namespace sono stati definiti
		//Da migliorare!!!!
		String wsdlArtifact = bpelLocation.replace(".bpel", "Artifacts.wsdl");
		String namespaceReturn; 
		BufferedReader input = null;
	    try {
	    	input = new BufferedReader( new FileReader(new File(wsdlArtifact)) );
	    	String line = null; //not declared within while loop
	    	while (( line = input.readLine()) != null) {
	    		if (line.indexOf("xmlns:"+prefix) != -1) {
		    		int indexFound = line.indexOf("xmlns:"+prefix);
		    		int indexStartDoubleQuots = line.indexOf("\"", indexFound);
		        	int indexEndDoubleQuots = line.indexOf("\"", indexStartDoubleQuots+1);
		        	namespaceReturn = line.substring(indexStartDoubleQuots+1, indexEndDoubleQuots);
		        	return namespaceReturn;
	    		}
	    	}
	    } catch (FileNotFoundException ex) {
	    	ex.printStackTrace();
	    } catch (IOException ex){
	    	ex.printStackTrace();
	    } finally {
	    	try {
	    		if (input!= null) 
	    			input.close();
	    	} catch (IOException ex) {
	    		ex.printStackTrace();
	    	}
	    }
	    return null;
	}
	
	private String[] getServiceAndPort(String targetNamespace, String portType){
		String[] serviceAndPort = new String[2];
		Set<String> wsdlLocationSet = wsdlDocumentMap.keySet();
		Iterator<String> it = wsdlLocationSet.iterator();
		Document document = null;
		String wsdlLocation = null;
		XPath xpPortType = new DefaultXPath("/wsdl:definitions[@targetNamespace='" + targetNamespace + "']" +
										"/wsdl:portType[@name='" + portType + "']");
		xpPortType.setNamespaceURIs(namespaces);
		while (it.hasNext()) {
			wsdlLocation = it.next();
			document = wsdlDocumentMap.get(wsdlLocation);
			Node portTypeNode = xpPortType.selectSingleNode(document);
			if (portTypeNode != null) { //found .wsdl 
				XPath xpTmp = new DefaultXPath("/wsdl:definitions/wsdl:service");
				serviceAndPort[0] = xpTmp.selectSingleNode(document).valueOf("@name");
				xpTmp = new DefaultXPath("/wsdl:definitions/wsdl:service/wsdl:port");
				serviceAndPort[1] = xpTmp.selectSingleNode(document).valueOf("@name");
				return serviceAndPort;
			}
			else 
				continue;
		}
		return null;
	}
	
	public static String getPartialWsdlWrappedContent(String wsdlLocation) {
		StringBuffer contents = new StringBuffer();
		File wsdlFile = new File(wsdlLocation);
		BufferedReader input = null;
		try {
			input = new BufferedReader( new FileReader(wsdlFile) );
			String line = null; 
			boolean foundDefinition = false;
			//generation definition
			while (( line = input.readLine()) != null ) {
				if (line.indexOf("<definitions") != -1)
					foundDefinition = true;
				if(foundDefinition) {
					int indexDefClose = line.indexOf(">");
					if (indexDefClose != -1) {
						String lineBeforeDefClose = line.substring(0, indexDefClose);
						String lineAfterDefClose = line.substring(indexDefClose + 1);
						contents.append(lineBeforeDefClose);
						contents.append(System.getProperty("line.separator"));
						contents.append("xmlns:wsdl=\"" + IConstants.XMLNS_WSDL +"\"");
						contents.append(System.getProperty("line.separator"));
						contents.append("xmlns:soap=\"" + IConstants.XMLNS_SOAP +"\"");
						contents.append(System.getProperty("line.separator"));
						contents.append("xmlns:smix=\"" + IConstants.XMLNS_SMIX +"\"");
						contents.append(">");
						contents.append(lineAfterDefClose);
						break;
					}
				}
				contents.append(line);
				contents.append(System.getProperty("line.separator"));
			}//end generation definition
			
			while(( line = input.readLine()) != null ) {
				if (line.indexOf("</definitions") == -1) {
					contents.append(line);
					contents.append(System.getProperty("line.separator"));
				}
				else
					contents.append(line.substring(0, line.indexOf("</definitions"))); 
			}
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException ex){
			ex.printStackTrace();
		} finally {
			try {
				if (input!= null) 
					input.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		return contents.toString();
	}

	public ArrayList<String> getWsdlWsInvoked() {
		ArrayList<String> wsdlListReturn = new ArrayList<String>();
		PartnerLinks partnerLinks = process.getPartnerLinks();
		//per ogni partnerlink avente providerRole != null (ovvero relativi agli invoke)
		//recupero partnerlinktype -> da cui ricavo portType
		//recupero dai wsdl importati da bpel solo quelli contenti definizione di port type
		for (PartnerLink partnerLink : partnerLinks.getChildren()) {
			Role partnerRole = partnerLink.getPartnerRole();
			String plnkName = partnerLink.getName();
			if (partnerRole != null) {
				String portType = getPortTypeByPartnerRole(plnkName, partnerRole.getName());	
				
				int indexSeparator = portType.indexOf(":");
				String prefixPortType = portType.substring(0, indexSeparator);
				String targetNamespace = getNamespaceFromArtifacts(prefixPortType);
				portType = portType.substring(indexSeparator + 1);
				
				Set<String> wsdlLocationSet = wsdlDocumentMap.keySet();
				Iterator<String> it = wsdlLocationSet.iterator();
				Document document = null;
				String wsdlLocation = null;
				XPath xpPortType = new DefaultXPath("/wsdl:definitions[@targetNamespace='" + targetNamespace + "']" +
												"/wsdl:portType[@name='" + portType + "']");
				xpPortType.setNamespaceURIs(namespaces);
				while (it.hasNext()) {
					wsdlLocation = it.next();
					document = wsdlDocumentMap.get(wsdlLocation);
					Node portTypeNode = xpPortType.selectSingleNode(document);
					if (portTypeNode != null)//found .wsdl containing the port type
						wsdlListReturn.add(wsdlLocation);
					else 
						continue;
				}
			}
		}
		return wsdlListReturn;
	}
}
