
package org.spagic.bpmn.diagram.palette;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Collections;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gmf.runtime.diagram.ui.services.palette.PaletteService;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.stp.bpmn.diagram.part.BpmnDiagramEditorPlugin;
import org.eclipse.stp.bpmn.provider.BpmnEditPlugin;
import org.eclipse.stp.bpmn.tools.MultipleShapesMoveTool;
import org.eclipse.stp.bpmn.tools.SelectionToolEx;
import org.osgi.framework.Bundle;
import org.spagic.bpmn.diagram.SpagicBpmnDiagramActivator;
import org.spagic.bpmn.diagram.SpagicBpmnDiagramEditor;
import org.spagic.bpmn.diagram.palette.toolentries.AbstractToolEntry;
import org.spagic.bpmn.diagram.palette.toolentries.IToolEntry;
import org.spagic.bpmn.diagram.palette.toolentries.NodeToolEntry;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @generated
 */
@SuppressWarnings("unchecked")
public class XMLBpmnPaletteFactory {

	private static Node selectSingleNode(Node document, String xPath) throws XPathExpressionException {
		XPath xpath = XPathFactory.newInstance().newXPath();
        return (Node)xpath.evaluate(xPath, document, XPathConstants.NODE);
	}
	
	private static NodeList selectNodes(Node document, String xPath) throws XPathExpressionException {
		XPath xpath = XPathFactory.newInstance().newXPath();
        return (NodeList)xpath.evaluate(xPath, document, XPathConstants.NODESET);
	}
	
	private static String getAttributeValue(Node n, String attributeName){
		try{
			Node attributeNode = n.getAttributes().getNamedItem(attributeName); 
			return (attributeNode != null ? attributeNode.getNodeValue() : "");
		}catch (Exception e){
			e.printStackTrace();
			return null;
		}
	}
	public static File getFileFromPlugin(String filePath){
		Bundle bundle = SpagicBpmnDiagramActivator.getDefault().getBundle();
		Path path = new Path(filePath); 
		URL url = FileLocator.find(bundle, path, Collections.EMPTY_MAP);
		URL fileUrl = null;
		try {
			fileUrl = FileLocator.toFileURL(url);
		}catch (IOException e) {
			// Will happen if the file cannot be read for some reason
			e.printStackTrace();
		}
		return new File(fileUrl.getPath());
	}
	
	
	public static String getStateLocationDir(){
		IPath path = SpagicBpmnDiagramActivator.getDefault().getStateLocation();
		String cfgDirPath = path.toOSString();
		return cfgDirPath;
	}
	
	
	public static String getPaletteFileNameInStateLocation(){
		return getStateLocationDir() + File.separator + "palette.xml";
	}
	
	public static File getPaletteFile(){
		
		File f = new File(getPaletteFileNameInStateLocation());
		
		if (f.exists()){
			return f;
		}else{
			copyInPluginStateLocation(getFileFromPlugin("palette.xml"), f);
		}
		
		return new File(getPaletteFileNameInStateLocation());
	}
	
	public static void copyInPluginStateLocation(File source, File destination){
		BufferedReader sourceReader = null;
		BufferedWriter bw = null;
		try{
		 sourceReader = new BufferedReader (new InputStreamReader(new FileInputStream(source)));
		 bw = new BufferedWriter(new FileWriter(destination));
		 String line = null;
		 while ( (line = sourceReader.readLine()) != null){
			 bw.write(line);
			 bw.write("\n");
			 bw.flush();
		 }
		}catch (Exception e) {
			e.printStackTrace();
		}finally{
			try{
				sourceReader.close();
				bw.close();
			}catch (Exception e) {
				e.printStackTrace();
			}
			
		}
	}
	
	
	
	public static IToolEntry getToolEntryForClass(String className){
		IToolEntry entry = null;
		try{
			entry = (IToolEntry)Class.forName(className).newInstance();
		}catch (Throwable e) {
			BpmnDiagramEditorPlugin.getInstance().logError(e.getMessage(), e);
		}finally{
			
		}
		return entry;
	}
    /**
     * 
     * 
     * @param paletteRoot
     * @notgenenated customizes the default entry to use the customized
     * select-tool. TODO: find a better place to do this so we don't need to
     * use introspection.
     */
    public void fillPalette(PaletteRoot paletteRoot) {
        PaletteEntry pe = null;
        for (Object paletteEntry : paletteRoot.getChildren()) {
            pe = (PaletteEntry)paletteEntry;
            if (pe.getId().equals(PaletteService.GROUP_STANDARD)) {
                break;
            }
            pe = null;
        }
        if (pe != null) {
            setupStandardPaletteEntries(pe);
        }
        try{
        	DocumentBuilder builder =
        		DocumentBuilderFactory.newInstance().newDocumentBuilder();
        	//Document document = builder.parse(getFileFromPlugin("palette.xml"));
        	Document document = builder.parse(getPaletteFile());
        	String xPath ="/bpmn-palette/group";
    		NodeList nodes = selectNodes(document, xPath); 
    		
    		Node n = null;
    		String groupName = null;
    		String groupDescription = null;
    		for (int i=0; i<nodes.getLength(); i++) {
    			n = nodes.item(i);
    			groupName = getAttributeValue(n,"name");
    			groupDescription = getAttributeValue(n,"label");
    			PaletteContainer paletteContainer = new PaletteDrawer(groupDescription);
    	        paletteContainer.setDescription(groupDescription);
    	        
    	        String xPathEntry ="/bpmn-palette/group[@name='"+groupName+"']/entry";
    	        NodeList entries = selectNodes(document, xPathEntry);
    	        Node entryNode = null;
    	        String entryClass = null;
    	        IToolEntry entry = null;
    	        for (int j = 0; j < entries.getLength(); j++){
    	        	entryNode = entries.item(j);
    	        	entryClass = getAttributeValue(entryNode,"class");
    	        	entry = getToolEntryForClass(entryClass);
    	        	if (entry != null){
    	        		paletteContainer.add(entry.createTool());
    	        	}
    	        }
    			paletteRoot.add(paletteContainer);
    		} // --
    		
        	
        }catch (Exception e) {
        	BpmnDiagramEditorPlugin.getInstance().logError(e.getMessage(), e);
		}
        
        ToolEntry toolEntry = paletteRoot.getDefaultEntry();
        SelectionToolEx.setToolClass(toolEntry, SelectionToolEx.class);
    }
    /**
     * @see PaletteService
     * @param pe The palette entry that contains the default selection tool,
     * the zoom and the note attachment tool. We subsitute them by BPMN tools.
     */
    protected void setupStandardPaletteEntries(PaletteEntry pe) {
        PaletteContainer standard = (PaletteContainer) pe;
        ImageDescriptor smallImage = AbstractToolEntry.findSmallToolImgDescr(IToolEntry.MOVE_HALF_EAST);
        ImageDescriptor largeImage = AbstractToolEntry.findLargeToolImgDescr(IToolEntry.MOVE_HALF_EAST);
        ToolEntry result = new NodeToolEntry("Insert space", 
                "Moves all shapes on the East of the cursor horizontally.", smallImage, 
                largeImage, MultipleShapesMoveTool.class);
        standard.add(result);
    }
    
    public static final URL getSVGURL(String fileName) {
        Bundle bundle = Platform.getBundle(BpmnEditPlugin.INSTANCE.getSymbolicName());
        return FileLocator.find(bundle, new Path("icons/full/objsvg/" + fileName + ".svg"), null);
    }
    
    
   
}
