/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.bpel.popup.actions;

import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.spagic.im.out.bpel.util.BpelTemplate;
import org.spagic.im.out.bpel.util.IConstants;
import org.spagic.utils.SpagicLogger;
import org.spagic.utils.SpagicUtils;

public class Im2BpelAction implements IObjectActionDelegate {
	
	private Shell shell                 = null;
	private IFile intermediateModelFile = null;
	
	/**
	 * Constructor for Action1.
	 */
	public Im2BpelAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		shell = new Shell();
		MessageDialog.openInformation(
			shell, "org.spagic.im.out.bpel",
			"BPEL Generation starting.");
		try{
			ResourceSet resourceSet = new ResourceSetImpl();
			URI uri = URI.createFileURI(intermediateModelFile.getFullPath().toString());
			Resource resourceIM = resourceSet.getResource(uri, true);
			StpIntermediateModel intermediateModel = (StpIntermediateModel)resourceIM.getContents().get(0);
			ProcessCollection processCollection = intermediateModel.getProcessCollection();
			List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
			
			for (org.eclipse.stp.im.Process intermediateModelProcess : processList) {
				if(SpagicUtils.isProcessDeployableForRuntime(intermediateModelProcess, IConstants.bpelRuntime))
					createBpelFromIntermediateModel(intermediateModelProcess);
			}
			MessageDialog.openInformation(
					shell, "org.spagic.im.out.bpel",
					"BPEL Generation executed.");
		} catch (Exception e) {
			SpagicLogger.error("Im2bpelAction: run ", e);
			MessageDialog.openInformation(
					shell, "org.spagic.im.out.bpel",
					"Error during BPEL Generation.");
		}
	}
	
	public void createBpelFromIntermediateModel(org.eclipse.stp.im.Process intermediateModelProcess)
		throws Exception {
		ImPackage imPackage = ImPackage.eINSTANCE;
		String namespace = SpagicUtils.getNameSpaceForProcess(intermediateModelProcess);
		String processName = intermediateModelProcess.getName();
		//generation process folder
		IContainer bpelFolder = intermediateModelFile.getParent().getParent().getFolder(new Path("Bpel"));
    	IFolder bpelProcessFolder = bpelFolder.getFolder(new Path(processName));
    	if(!bpelProcessFolder.exists())
    		bpelProcessFolder.create(true, true, null);
    	
		BpelTemplate bpelTemplate = new BpelTemplate(namespace, processName); 
		try {
			bpelTemplate.createWsdlFile(bpelProcessFolder);
        } catch(Exception ex) {
        	SpagicLogger.error("Im2BpelAction:: createBpelFromIntermediateModel: after createWsdlFile", ex);
			MessageDialog.openError(
				shell, "org.spagic.im.out.bpel",
				"Error occurred during generation .wsdl file");
			throw ex;
        }
        try {			
        	bpelTemplate.creatBpelFile(bpelProcessFolder, intermediateModelProcess);
        } catch(Exception ex) {
        	SpagicLogger.error("Im2BpelAction:: createBpelFromIntermediateModel: after creatBpelFile", ex);
			MessageDialog.openError(
			shell, "org.spagic.im.out.bpel",
			"Error occurred during generation .bpel file");
			throw ex;
        }
	}
	
	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		setIntermediateModelFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}

	public IFile getIntermediateModel() {
		return intermediateModelFile;
	}

	public void setIntermediateModelFile(IFile intermediateModelFile) {
		this.intermediateModelFile = intermediateModelFile;
	}

}
