/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.bpel.util;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.bpel.model.resource.BPELReader;
import org.eclipse.bpel.model.resource.BPELResource;
import org.eclipse.bpel.model.resource.BPELResourceFactoryImpl;
import org.eclipse.bpel.ui.BPELUIPlugin;
import org.eclipse.bpel.ui.Templates.Template;
import org.eclipse.bpel.ui.Templates.TemplateResource;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.spagic.utils.SpagicLogger;

public class BpelTemplate {
    private Map<String,Object> args;
    public final String SYNCHRONOUS_TEMPLATE = "Synchronous BPEL Process";
    private Template template;
    private TemplateResource wsdlResource = null;
    private TemplateResource bpelResource = null;
    
    public BpelTemplate(String namespace, String processName) {
		super();
		args = new HashMap<String, Object>();
		args.put("processName", processName);
		args.put("namespace", namespace);
		template = BPELUIPlugin.INSTANCE.getTemplates().getTemplateByName(SYNCHRONOUS_TEMPLATE);
		loadResources();
	}
    
	public BpelTemplate(Map<String, Object> args, Template template) {
		super();
		this.args = args;
		this.template = template;
		loadResources();
	}
	
	private void loadResources() {
        List<TemplateResource> resources = template.getResources();
        Iterator<TemplateResource> it = resources.iterator();
        TemplateResource resource = null;
        while ( it.hasNext()) {
        	resource = (TemplateResource)it.next();
        	if (resource.getName(args).indexOf(".bpel") != -1)
        		bpelResource = resource;
        	else if (resource.getName(args).indexOf(".wsdl") != -1)
        		wsdlResource = resource;
        }
	}

	public void setArgs(Map<String, Object> args) {
		this.args = args;
	}
	
	public void setTemplate(Template template) {
		this.template = template;
	}
	
    public void createWsdlFile(IContainer container) 	
    			throws Exception {
    	String wsdlString = wsdlResource.process(args);
    	String wsdlFileName = wsdlResource.getName(args);
		IFile file = container.getFile(new Path(wsdlFileName));
		InputStream stream = new ByteArrayInputStream( wsdlString.getBytes() );
		try {            
			if (file.exists()) 
				file.setContents(stream, true, true, null);
			else 
				file.create(stream, true, null);
		} catch(Exception ex){
			SpagicLogger.error("BpelTemplate: createWsdlFile", ex);
			throw ex;
		} finally {
			try {
				stream.close();
			} catch (Exception ex) {
				// ignore
			}
		}
    } 
    
    public void creatBpelFile(IContainer container, org.eclipse.stp.im.Process intermediateModelProcess) throws Exception {
    	String bpelString = bpelResource.process(args);
    	String bpelFileName = bpelResource.getName(args);
    	InputStream stream = new ByteArrayInputStream(bpelString.getBytes());
		IFile bpelFile = container.getFile(new Path(bpelFileName));		  
		URI bpelFileURI = URI.createFileURI(bpelFile.getLocation().toString());
		BPELResource bpelRes = (BPELResource)new BPELResourceFactoryImpl().createResource(bpelFileURI);
		BPELReader reader = new BPELReader();
		reader.read(bpelRes, stream);
		BPELGenerator bpelGenerator = new BPELGenerator(intermediateModelProcess, bpelRes);
		bpelGenerator.generate();
		bpelRes.save(args);
    	
    }
}
