/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.bpel.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.im.*;
import org.eclipse.stp.im.constants.ImConstants;

public class BpelUtil {

	public static String getRuntimeId(){
		return "runtime.bpel";
	}
	
	public static ServiceBinding getServiceBinding(Step step){
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding serviceBinding : step.getStepbindings()){
			runtimPropertyForServiceBinding = serviceBinding.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null){
				if (runtimPropertyForServiceBinding.getValue().equals(getRuntimeId())){
					return serviceBinding;
				}
			}else{
				System.out.println("ServiceBinding["+serviceBinding.getName()+"] -> for Step["+step.getName()+"] does not have a property called "+ ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		System.out.println("Not found for Step["+step.getName()+"] a binding for runtime "+ ImConstants.IM_POOL_RUNTIME_ID);
		return null;
	}
	
	public static Step getStartStep(EList<Step> steps) {
		for (Step step : steps) {
			if (step.getTargetTransitions().isEmpty())
				return step;
		}
		return null;
	}
	
	public static Step getNextStep(Step currentStep) {
		EList<Transition> transitionList = currentStep.getSourceTransitions();
		if(!transitionList.isEmpty())
			return ((Transition)transitionList.get(0)).getTarget();
		return null;
	}
	
	public static List<Step> getNextSteps(Step currentStep) {
		List<Transition> transitionList = currentStep.getSourceTransitions();
		List<Step> stepList = new ArrayList<Step>();
		if(!transitionList.isEmpty()){
			for (Transition transition : transitionList)
				stepList.add(transition.getTarget());
		}
		return stepList;
	}
	
	public static String getCondition(Step sourceStep, Step targetStep) {
		List<Transition> transitionList = targetStep.getTargetTransitions();
		if(!transitionList.isEmpty()){
			for (Transition transition : transitionList) {
				if (transition.getSource().equals(sourceStep)) 
					if (transition instanceof TransitionUnderCondition) {
						Condition condition = ((TransitionUnderCondition) transition).getCondition();
						return ((ExpressionCondition)condition).getExpression();
					} else 
						return null;
			}
		}
		return null;
	}
		
	public static boolean isEndStep(Step currentStep) {
		if (currentStep.getSourceTransitions().isEmpty())
				return true;
		return false;
	}
	
	public static boolean isEndSequenceStep(Step currentStep) {
		Property startSequence = currentStep.getProperties().get("startSequence");
		if (startSequence == null || startSequence.getValue().equals("true"))
			return false;
		return true;
	}
	
	public static boolean isEndIfControlStep(Step currentStep) {
		Property startSequence = currentStep.getProperties().get("startIfControl");
		if (startSequence == null || startSequence.getValue().equals("true"))
			return false;
		return true;
	}
	
	public static boolean isEndFlowStep(Step currentStep) {
		Property startFlow = currentStep.getProperties().get("startFlow");
		if (startFlow == null || startFlow.getValue().equals("true"))
			return false;
		return true;
	}
	
	public static boolean isEndLoopStep(Step currentStep, String nameLoop) {
		if (currentStep.getName().equalsIgnoreCase("end."+nameLoop))
			return true;
		return false;
	}
	
	public static boolean executeAtLeastOne(Step currentStep) {
		Property executeAtLeastOnce = currentStep.getProperties().get("ExecuteAtLeastOnce");
		if (executeAtLeastOnce == null || executeAtLeastOnce.getValue().equals("true"))
			return true;
		return false;
	}
}
