package org.eclipse.jet.compiled;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jet.JET2TemplateLoader;
import org.eclipse.jet.JET2TemplateLoaderExtension;
import org.eclipse.jet.JET2Template;

public class _jet_transformation implements JET2TemplateLoader, JET2TemplateLoaderExtension {

    private static final class TemplateFactoryMethod {

        private final String templatePath;
        private final String templateClass;

        public TemplateFactoryMethod(String templatePath, String templateClass) {
            this.templatePath = templatePath;
            this.templateClass = templateClass;
            
        }
        
        public String getTemplateClass() {
            return this.templateClass;
        }

        public String getTemplatePath() {
            return this.templatePath;
        }

        public JET2Template newTemplate() {
            JET2Template template = null;
            try {
                template = (JET2Template) Class.forName(this.templateClass).newInstance();
            } catch (InstantiationException e) {
                // ignore
            } catch (IllegalAccessException e) {
                // ignore
            } catch (ClassNotFoundException e) {
                // ignore
            }
            return template;
        }
        
        
    }

    private Map pathToFactoryMap = null;
    
    private JET2TemplateLoader delegate = null;

    public _jet_transformation() {
        super();
    }

    public JET2Template getTemplate(String templatePath) {
        if(this.pathToFactoryMap == null) {
            initPathToFactoryMap();
        }
        TemplateFactoryMethod factory = (TemplateFactoryMethod)this.pathToFactoryMap.get(templatePath);
        return factory == null ? this.delegate == null ? null : this.delegate.getTemplate(templatePath) : factory.newTemplate();
    }

    private void addTemplate(String templatePath, String templateClass) {
        this.pathToFactoryMap.put(templatePath,
            new TemplateFactoryMethod(templatePath, templateClass));
    }
    
    private void initPathToFactoryMap() {
        this.pathToFactoryMap = new HashMap(20);
            addTemplate("templates/dump.jet", "org.eclipse.jet.compiled._jet_dump" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/jbixml.jet", "org.eclipse.jet.compiled._jet_jbixml" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/lwsu.jet", "org.eclipse.jet.compiled._jet_lwsu" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/main.jet", "org.eclipse.jet.compiled._jet_main" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-JDBCPollerBindingComponent.jet", "org.eclipse.jet.compiled._jet_JBIJDBCPollerBindingComponent" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-Lightweight.jet", "org.eclipse.jet.compiled._jet_JBILightweight" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardDroolsValidator.jet", "org.eclipse.jet.compiled._jet_JBIStandardDroolsValidator" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-MessageFilter.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPMessageFilter" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-Pipeline.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPPipeline" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-RecipientList.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPRecipientList" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-RecipientListAggregator.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPRecipientListAggregator" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-SplitterAggregator.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPSplitterAggregator" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardEIP-XPathSplitter.jet", "org.eclipse.jet.compiled._jet_JBIStandardEIPXPathSplitter" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardFile.jet", "org.eclipse.jet.compiled._jet_JBIStandardFile" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardHTTP.jet", "org.eclipse.jet.compiled._jet_JBIStandardHTTP" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardJMS.jet", "org.eclipse.jet.compiled._jet_JBIStandardJMS" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardRouter.jet", "org.eclipse.jet.compiled._jet_JBIStandardRouter" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/JBI-StandardTCP.jet", "org.eclipse.jet.compiled._jet_JBIStandardTCP" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/servicebindings/RoutingRules.jet", "org.eclipse.jet.compiled._jet_RoutingRules" ); //$NON-NLS-1$  //$NON-NLS-2$
            addTemplate("templates/stdsu.jet", "org.eclipse.jet.compiled._jet_stdsu" ); //$NON-NLS-1$  //$NON-NLS-2$
    }

    public void setDelegateLoader(JET2TemplateLoader loader) {
        this.delegate = loader;
    }
    
    public JET2TemplateLoader getDelegateLoader() {
        return this.delegate;
    }
}

