/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.jbi.smx.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.eclipse.ui.PlatformUI;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.SpagicUtils;
import org.spagic.utils.SpagicUtilsConstants;
import org.w3c.dom.Element;

public class JBIJetUtils {
	
	public static String MEP_IN_ONLY="http://www.w3.org/2004/08/wsdl/in-only";
	public static String MEP_IN_OUT="http://www.w3.org/2004/08/wsdl/in-out";
	private static List<String> folders = new ArrayList<String>();
	
	private static IRuntime runtime = null;
	private static Document jbiRuntimeSpecificDocument = null;	
	
	public static Map<String, String> serviceBindingsTemplateMap = new HashMap<String, String>();
	
	public static String SERVICE_UNIT_PROPERTY_NAME = "ServiceUnit";
	public static String LW_SERVICE_UNIT ="LwContainerServiceUnit";
	public static String REQUIRE_JNDI_DATASOURCE_PROPERTY_NAME ="RequireJndiDataSource";
	public static String JNDI_DATASOURCE_PROPERTY_NAME ="JndiDataSource";
	
	
	public static String getRuntimeId(){
		return "runtime.jbi.smx";
	}
	
	
	public static void initJbiConfiguration() throws RuntimeException{
		try{
			ImPackage imPackage = ImPackage.eINSTANCE;
			runtime = ImRuntimeActivator.getRuntime(getRuntimeId());
			String specificRuntimeXMLString = runtime.getRuntimeSpecific();
			
			jbiRuntimeSpecificDocument = DocumentHelper.parseText(specificRuntimeXMLString);
			folders.clear();
		}catch (Exception e) {
			throw new RuntimeException(e);
		}finally{
			
		}
	}
	
	public static String getServiceUnitNameForServiceBinding(ServiceBinding serviceBinding){
		
		String xPath = "/runtime-specific/service-binding-su-map/service-binding-su-map-entry[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		
		String suName = null;
		
		if (node != null){
			suName = node.valueOf("@serviceUnitName");
		}
		
		return suName;
		
	}
	public static Map<String, List<Step>> getServiceUnitListForProcessModels(org.eclipse.stp.im.Process aProcess){
		
		// 
		// To get the service unit for the step
		// me must get the right binding for the step and then see in the runtime specific configuration
		// which is the service unit for a particular binding
		//
		
		Map<String, List<Step>> suMap = new HashMap<String, List<Step>>();
		List<String> usedList = new ArrayList<String>();
		
		Property serviceUnitProperty = null;
		ServiceBinding serviceBindingForStepWithThisRuntime = null;
		String serviceUnitNameForBinding = null;
		
		for (Step step : aProcess.getSteps()){
			
			serviceBindingForStepWithThisRuntime = getRightStepBinding(step);
			
			if (serviceBindingForStepWithThisRuntime == null){
				throw new RuntimeException(" Not found a servicebing for step ["+step.getName()+"] for runtime ["+getRuntimeId()+"]");
			}
			
			serviceUnitNameForBinding = getServiceUnitNameForServiceBinding(serviceBindingForStepWithThisRuntime);
			
			
			if (serviceUnitNameForBinding == null){
				throw new RuntimeException(" Not found a service unit for service binding ["+serviceBindingForStepWithThisRuntime+"] for runtime ["+getRuntimeId()+"]");
			}
			
			if (suMap.containsKey(serviceUnitNameForBinding)){
					// Service Unit is already on su map
					// Add the step to su
					((List<Step>)suMap.get(serviceUnitNameForBinding)).add(step);
			}else{
					List<Step> stepsForSu = new ArrayList<Step>();
					stepsForSu.add(step);
					suMap.put(serviceUnitNameForBinding, stepsForSu);
			}
		}
		
		return suMap;
	}
	
	
	
	public static String getComponentNameForServiceUnit(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		return node.valueOf("@componentName");
	}
	
	public static String getVersionAsStringForProcess(Process aProcess){
		Property processVersionProperty = aProcess.getProperties().get(ImConstants.PROCESS_VERSION);
		String version = null;
		if (processVersionProperty != null)
			version = (String)processVersionProperty.getValue();
		else
			version = "0";
		return version;
	}
	
	public static String getNamespaceURI(Process aProcess){
		return SpagicUtils.getNameSpaceForProcess(aProcess);
	}
	
	
	public static String getNamespacePrefix(Process aProcess){
		
		return SpagicUtils.getNameSpacePrefixForProcess(aProcess);
	}
	
	
	
	public static boolean isStandard(String suName){
	
		return !suName.equalsIgnoreCase("LwContainerServiceUnit");
	
	}
	
	
	
	public static String getRuntimeNameSpacePrefix() {
		
		return runtime.getNsprefix();
	
	}

	public static String getRuntimeNameSpaceURI() {
	
		return runtime.getNsuri();
	
	}
	
	public static String getJetTemplateForServiceBinding(ServiceBinding serviceBinding){

		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		if (node == null)
			throw new RuntimeException(" Not found code-generator section for service binding ["+ serviceBinding.getName() + "]");
		return node.valueOf("@jetTemplate");
	
	}
	
	public static String getLightweightClassForServiceBinding(ServiceBinding serviceBinding, Step s){
		if (serviceBinding.getName().equalsIgnoreCase("JBI-AdvancedJDBCService")){
			boolean isStored = Boolean.valueOf(JBIJetUtils.getRequiredPropertyValue(s, "FlagIsStoredProcdure"));
			return ( isStored ? "org.spagic.smx.components.jdbcstore.JDBCStoreProcedureComponent" : "org.spagic.smx.components.jdbcquery.JDBCAdvancedQueryComponent");
		}else{
			String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
			Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
			return node.valueOf("@lwClassName");
		}
	}
	
	public static String getServiceUnitBriefName(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		return node.valueOf("@briefName");
		
	}
	public static String getServiceUnitNameSpaceURI(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		return node.valueOf("@nsuri");
	}
	
	public static String getServiceUnitNameSpacePrefix(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = jbiRuntimeSpecificDocument.selectSingleNode(xPath);
		return node.valueOf("@nsprefix");
		
	}
	
	public static SimplePropertyMap parseSimplePropertyMap(Node n){
		String decorator = null;
		IDecorator decoratorInst = null;
		String produceConfiguration = null;

		String flagIsRef = null;
		String flagIsConst = null;
		String constValue = null;
		SimplePropertyMap aPropertyMap = new SimplePropertyMap(n.valueOf("@im-model-name"), n
				.valueOf("@runtime-name"), Boolean.valueOf(n
				.valueOf("@required")));
		decorator = n.valueOf("@decorator");
		if (isValid(decorator)) {
			try {
				decoratorInst = (IDecorator) Class.forName(decorator)
						.newInstance();
			} catch (Exception e) {
				e.printStackTrace();
				LogUtils.error(e);
			}
			aPropertyMap.setDecorator(decoratorInst);
		}

		String condition = n.valueOf("@condition");
		if (isValid(condition)) {
			aPropertyMap.setCondition(condition);
		}
		produceConfiguration = n.valueOf("@produceConfiguration");
		if (isValid(produceConfiguration)) {
			aPropertyMap.setRequired(Boolean.valueOf(produceConfiguration));
		}

		flagIsRef = n.valueOf("@isRef");
		if (isValid(flagIsRef)) {
			((SimplePropertyMap)aPropertyMap).setFlagRef(Boolean.valueOf(flagIsRef));
		}
		
		flagIsConst = n.valueOf("@is-const");
		if (isValid(flagIsConst)) {
			((SimplePropertyMap)aPropertyMap).setFlagConst(Boolean.valueOf(flagIsConst));
		}
		
		constValue = n.valueOf("@const-value");
		if (isValid(constValue)) {
			((SimplePropertyMap)aPropertyMap).setConstValue(constValue);
		}
		
		return aPropertyMap;
	}
	
	public static IPropertyMap parseMapPropertyMap(Node n){
		MapPropertyMap aPropertyMap = new MapPropertyMap(n.valueOf("@im-model-name"), n
			.valueOf("@runtime-name"), Boolean.valueOf(n
			.valueOf("@required")));
		
		
		String mapEntryKey = n.valueOf("@map-entry-key");
		String valueBeanClass = n.valueOf("@value-bean-class");
		aPropertyMap.setMapEntryKey(mapEntryKey);
		if (isValid(valueBeanClass)){
			aPropertyMap.setValueBeanClass(valueBeanClass);
		}
		List<Node> beanMappingsProperties = n.selectNodes("value-bean-property");
		List<SimplePropertyMap>  beanMappingPropertyMap = new ArrayList<SimplePropertyMap>();
		for (Node beanMappingNode : beanMappingsProperties){
			beanMappingPropertyMap.add(parseSimplePropertyMap(beanMappingNode));
		}
		aPropertyMap.setBeanMappingProperty(beanMappingPropertyMap);
		
		return aPropertyMap;
	}
	
	public static IPropertyMap parseListPropertyMap(Node n){
		ListPropertyMap aPropertyMap = new ListPropertyMap(n.valueOf("@im-model-name"), n
			.valueOf("@runtime-name"), Boolean.valueOf(n
			.valueOf("@required")));
		
	
		String valueBeanClass = n.valueOf("@value-bean-class");
		
		if (isValid(valueBeanClass)){
			aPropertyMap.setValueBeanClass(valueBeanClass);
		}
		List<Node> beanMappingsProperties = n.selectNodes("value-bean-property");
		List<SimplePropertyMap>  beanMappingPropertyMap = new ArrayList<SimplePropertyMap>();
		for (Node beanMappingNode : beanMappingsProperties){
			beanMappingPropertyMap.add(parseSimplePropertyMap(beanMappingNode));
		}
		aPropertyMap.setBeanMappingProperty(beanMappingPropertyMap);
		return aPropertyMap;
	}
	
	public static IPropertyMap getPropertyMapFromNode(Node n) {
		IPropertyMap aPropertyMap = null;
		
		if (n.getName().equalsIgnoreCase("property-map")) {
			return parseSimplePropertyMap(n);
		}else if (n.getName().equalsIgnoreCase("map-property-map")) {
			return parseMapPropertyMap(n);
		}else if (n.getName().equalsIgnoreCase("list-property-map")) {
			return parseListPropertyMap(n);
		}else if (n.getName().equalsIgnoreCase("bean-property-map")) {
			aPropertyMap = new BeanPropertyMap(n.valueOf("@im-model-name"), n
					.valueOf("@runtime-name"), Boolean.valueOf(n
					.valueOf("@required")));
			String beanClass = n.valueOf("@bean-class");
			((BeanPropertyMap)aPropertyMap).setBeanClassName(beanClass);
			List<Node> children = new ArrayList<Node>();
			List<IPropertyMap> childrenPropertyMap = new ArrayList<IPropertyMap>();
			children.addAll(n.selectNodes("property-map"));
			children.addAll(n.selectNodes("map-property-map"));
			children.addAll(n.selectNodes("bean-property-map"));
			children.addAll(n.selectNodes("list-property-map"));
			for (Node n1 : children){
				childrenPropertyMap.add(getPropertyMapFromNode(n1));
			}
			((BeanPropertyMap)aPropertyMap).setChildrenProperties(childrenPropertyMap);
		}
		return aPropertyMap;
	}
	public static List<IPropertyMap> getCodeGeneratorPropertyMap(ServiceBinding serviceBinding){
		String xPath1 ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/property-map";
		String xPath2 ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/map-property-map";
		String xPath3 ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/bean-property-map";
		String xPath4 ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/list-property-map";
		List<Node> nodes = new ArrayList<Node>();
		nodes.addAll(jbiRuntimeSpecificDocument.selectNodes(xPath1));
		nodes.addAll(jbiRuntimeSpecificDocument.selectNodes(xPath2));
		nodes.addAll(jbiRuntimeSpecificDocument.selectNodes(xPath3));
		nodes.addAll(jbiRuntimeSpecificDocument.selectNodes(xPath4));
		
	
		List<IPropertyMap>  propertiesMap = new ArrayList<IPropertyMap>();
		
	
		for (Node n : nodes ){
			propertiesMap.add(getPropertyMapFromNode(n));
		}
		
		return propertiesMap;
	}
	
	public static List<CopyResource> getCodeGeneratorCopyResources(ServiceBinding serviceBinding){
		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/copy-resource";
	
		
		List<Node> nodes = jbiRuntimeSpecificDocument.selectNodes(xPath);
		
		CopyResource aCopyResource = null;
		List<CopyResource>  copyResources = new ArrayList<CopyResource>();
		
		String identifyProperty = null;
		String produceConfiguration = null;
		for (Node n : nodes ){
			identifyProperty = n.valueOf("@identifyingProperty");
			aCopyResource = new CopyResource(identifyProperty);
			
			copyResources.add(aCopyResource);
		}
		
		return copyResources;
	}

	public static boolean isValid(String s) {
		return (s != null && s.trim().length() > 0);
	}

	public static String getRequiredPropertyValue(Step step, String imName) {
	
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : null;
		
		if (!JBIJetUtils.isValid(valueInIm)){
			throw new RuntimeException(" Property ["+imName+"] --> is required to generate code for Step ["+step.getName()+"] with Binding ["+JBIJetUtils.getRightStepBinding(step).getName()+"]");
		}
		return valueInIm;
	}

	public static String getOptionalPropertyValue(Step step,
			String imName, String defaultValue) {
		
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : defaultValue;
		
		return valueInIm;
	}

	
	
	/**
	 * 
	 * @param s
	 * @return the right service binding for the runtime we're using during the transformation
	 */
	public static ServiceBinding getRightStepBinding(Step s){
		
		String runtimeID = getRuntimeId();
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding sb : s.getStepbindings()){
			runtimPropertyForServiceBinding = sb.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null){
				if (runtimPropertyForServiceBinding.getValue().equals(runtimeID)){
					return sb;
				}
			}else{
				LogUtils.warn("ServiceBinding["+sb.getName()+"] -> for Step["+s.getName()+"] does not have a property called "+ ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		LogUtils.warn("Not found for Step["+s.getName()+"] a binding for runtime "+ ImConstants.IM_POOL_RUNTIME_ID);
		return null;
		
	}
	
	public static boolean isRequiredNamespaceContext(String suName){
		if (suName.equalsIgnoreCase("DroolsServiceUnit") ||suName.equalsIgnoreCase("EIPServiceUnit")){
			return true;
		}
		return false;
	}
	
	public static void addFolder(String folderName){
		folders.add(folderName);
	}
	


	public static List<String> getFolders() {
		return folders;
	}


	public static void setFolders(List<String> folders) {
		JBIJetUtils.folders = folders;
	}
	
	public static String getRelativeFileName(String original){
		String relPath = original;
		if (original.indexOf("/") > 0)
			relPath = relPath.substring(original.lastIndexOf("/") + 1);
		if (original.indexOf("\\") > 0)
			relPath = relPath.substring(original.lastIndexOf("\\") + 1);
		return relPath;
	}
	
	public static List<String> getJndiDatasourceNames(List<Step> stepList){
		List<String> datasources = new ArrayList<String>();
		String flgRequireDataSource = null;
		String dsName = null;
		for (Step aStep : stepList){
			flgRequireDataSource = getOptionalPropertyValue(aStep, REQUIRE_JNDI_DATASOURCE_PROPERTY_NAME, "false");
			if (flgRequireDataSource.equalsIgnoreCase("true")){
				dsName = getRequiredPropertyValue(aStep, JNDI_DATASOURCE_PROPERTY_NAME);
				if (!datasources.contains(dsName))
					datasources.add(dsName);
			}
		}
		return datasources;
	}
	
	public static Map<String, String> getNamespaceMap(){
		return MetaDBHelper.getNameSpaceMap();
		
	}
	public static void jbiIntermediateModelPreprocessor(
			StpIntermediateModel aStpIntermediateModel) {

		ImFactory stpModelfactory = ImFactory.eINSTANCE;

		ServiceBinding sb = null;
		Step additionalStep = null;
		Step destinationStep = null;
		Transition sourceTransition = null;
		Transition newTransition = null;
		List<Step> stepToAddToProcess = new ArrayList<Step>();

		ServiceBinding syncSB = stpModelfactory.createServiceBinding();
		syncSB.setName("JBI-Sync");
		BasicProperty runtimeIdProperty = stpModelfactory.createBasicProperty();
		runtimeIdProperty.setKey(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
		runtimeIdProperty.setValue(getRuntimeId());
		syncSB.getProperties().put(runtimeIdProperty.getKey(),
				runtimeIdProperty);
		aStpIntermediateModel.getServicebindings().add(syncSB);
		//
		// Update to add sync component
		//
		for (Process aProcess : aStpIntermediateModel.getProcessCollection().getProcesses()) {
			if (!SpagicUtils.isProcessDeployableForRuntime(aProcess, getRuntimeId()))
				continue;
			stepToAddToProcess.clear();
			boolean syncAdded = false;
			
			List<Step> replyStepsToRemove = new ArrayList<Step>();
			List<Transition> transitionsToRemove = new ArrayList<Transition>();
			
			replyStepsToRemove.clear();
			transitionsToRemove.clear();
			
			for (Step aStep : aProcess.getSteps()) {
				sb = getRightStepBinding(aStep);
				if ((sb != null)
						&& (sb.getName().equalsIgnoreCase(
								"JBI-HTTPInputOutputBindingComponent") || (sb
								.getName()
								.equalsIgnoreCase("JBI-TcpIp-InputOutputBindingComponent")))) {
					
					// We must insert a sync component
				
					if (aProcess.getSteps().size() > 3){
						
						// Add the sync and relocate the transitions
						sourceTransition = aStep.getSourceTransitions().get(0);
						
						
						destinationStep = sourceTransition.getTarget();
						aProcess.getTransitions().remove(sourceTransition);
						aStep.getSourceTransitions().remove(sourceTransition);
						
						additionalStep = stpModelfactory.createStep();
						additionalStep.setName(aStep.getName()
							+ SpagicUtilsConstants.JBI_SYNC_SUFFIX);
						additionalStep.getStepbindings().add(syncSB);
						stepToAddToProcess.add(additionalStep);

						newTransition = stpModelfactory.createTransition();
						newTransition.setSource(aStep);
						newTransition.setTarget(additionalStep);
						aProcess.getTransitions().add(newTransition);

						newTransition = stpModelfactory.createTransition();
						newTransition.setSource(additionalStep);
						newTransition.setTarget(destinationStep);
						aProcess.getTransitions().add(newTransition);
						
						newTransition = stpModelfactory.createTransition();
						newTransition.setSource(additionalStep);
						newTransition.setTarget(aStep);
						aProcess.getTransitions().add(newTransition);
						
						syncAdded = true;
						
					}else{
						// Add the sync and relocate the transitions
						sourceTransition = aStep.getSourceTransitions().get(0);
						destinationStep = sourceTransition.getTarget();
						
						newTransition = stpModelfactory.createTransition();
						newTransition.setSource(destinationStep);
						newTransition.setTarget(aStep);
						aProcess.getTransitions().add(newTransition);
						syncAdded = false;
						
					}
				}
			}// for Step Loop 
			
			// Change Transitions
			
		
			
				
				// Change the transitions of the steps that has target reply to sync
			for (Step aStep : aProcess.getSteps()) {
				List<Transition> trToRemoveFromStepSourceTransition = new ArrayList<Transition>();
				for (Transition tr : aStep.getSourceTransitions()) {
					sb = getRightStepBinding(tr.getTarget());
					if (sb.getName().equalsIgnoreCase("JBI-Reply")){
						replyStepsToRemove.add(tr.getTarget());
						if (syncAdded)
							tr.setTarget(additionalStep);
						else{
							aProcess.getTransitions().remove(tr);
							trToRemoveFromStepSourceTransition.add(tr);
						}
					}
				}//for (Transition tr :
				aStep.getSourceTransitions().removeAll(trToRemoveFromStepSourceTransition);
			}//for (Step aStep :
		
			// Modification for Pipeline
			ServiceBinding httpIoSb = null;
			ServiceBinding tcpIoSb = null;
			for (ServiceBinding aServiceBinding : aStpIntermediateModel
					.getServicebindings()) {
				if (aServiceBinding.getName().equalsIgnoreCase(
						"JBI-HTTPInputOutputBindingComponent"))
					httpIoSb = aServiceBinding;
				if (aServiceBinding.getName().equalsIgnoreCase(
						"JBI-TcpIp-InputOutputBindingComponent"))
					tcpIoSb = aServiceBinding;
			}
			BasicProperty newProperty = null;
			for (Step aStep : aProcess.getSteps()) {
				sb = getRightStepBinding(aStep);

				if ((sb != null) && (sb.getName().equalsIgnoreCase("JBI-WebServicePipeline")
						|| (sb.getName().equalsIgnoreCase("JBI-TCPIP-Pipeline")))) {
					additionalStep = stpModelfactory.createStep();
					additionalStep.setName(aStep.getName()
							+ SpagicUtilsConstants.JBI_TRANSFORMER_SUFFIX);

					if (sb.getName().equalsIgnoreCase("JBI-WebServicePipeline"))
						additionalStep.getStepbindings().add(httpIoSb);

					if (sb.getName().equalsIgnoreCase("JBI-TCPIP-Pipeline"))
						additionalStep.getStepbindings().add(tcpIoSb);

					EMap<String, Property> propsOfPipeline = aStep
							.getProperties();

					for (String keyProp : propsOfPipeline.keySet()) {

						newProperty = stpModelfactory.createBasicProperty();
						newProperty.setKey(keyProp);
						newProperty.setValue((String) propsOfPipeline.get(
								keyProp).getValue());
						additionalStep.getProperties().put(newProperty.getKey(), newProperty);
					}

					// me must know which is the target step
					newProperty = stpModelfactory.createBasicProperty();
					newProperty.setKey("transformerStepName");
					newProperty.setValue(additionalStep.getName());
					aStep.getProperties().put("transformerStepName",
							newProperty);
					
					newProperty = stpModelfactory.createBasicProperty();
					newProperty.setKey("IsTransformerPipeline");
					newProperty.setValue("true");
					additionalStep.getProperties().put("IsTransformerPipeline",
							newProperty);
					
					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(aStep);
					newTransition.setTarget(additionalStep);
					aProcess.getTransitions().add(newTransition);

					newTransition = stpModelfactory.createTransition();
					newTransition.setSource(additionalStep);
					newTransition.setTarget(aStep);
					aProcess.getTransitions().add(newTransition);
					stepToAddToProcess.add(additionalStep);
				}
			}// end
			
			aProcess.getSteps().removeAll(replyStepsToRemove);		
			aProcess.getSteps().addAll(stepToAddToProcess);

		} // for Processes
	}
	
	public static boolean existFileInProject(String projectName, String wsdlFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject proj = root.getProject(projectName);
		IFile file = proj.getFile(wsdlFile);
		return file.exists();
	}
}
