/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.jbi.smx.utils;

import java.util.Map;

import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.Step;


public class SimplePropertyMap extends AbstractPropertyMap {
	
	protected boolean flagRef = false;
	protected boolean flagConst = false;
	protected String constValue = null;
	
	
	public SimplePropertyMap(String imName, String runtimeName, boolean required) {
		super();
		this.imName = imName;
		this.runtimeName = runtimeName;
		this.required = required;
	}
	
	
	protected String getPropertyValueFromStep(Step s){
		Property aProperty = s.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : null;
		return valueInIm;
	}
	
	/**
	 * @see org.spagic.im.out.jbi.smx.utils.IPropertyMap#eval(org.eclipse.stp.im.Step)
	 */
	public String eval(Step s){
		if (isUnderCondition() && !evaluateGenerationCondition(s)){
			return "";
		}
		String value = null;
		if (flagConst)
			value = constValue;
		else
			value = getPropertyValueFromStep(s);
		
		
		
		if (isRequired() && !JBIJetUtils.isValid(value)){
			throw new RuntimeException(" Property ["+imName+"] --> ["+runtimeName+"] is required to generate code for Step ["+s.getName()+"] with Binding ["+JBIJetUtils.getRightStepBinding(s).getName()+"]");
		}
		if (!isRequired() && value == null){
			value = "";
		}
		if (!(produceConfiguration)){
			return "";
		}else{
			String finalValue =  decorator != null ? decorator.decorate(value) : value;
			if (isFlagRef())
				return "<property name=\""+runtimeName+"\"  ref=\""+finalValue+"\"/>\n";
			else
				return "<property name=\""+runtimeName+"\"  value=\""+finalValue+"\"/>\n";
		}
	
	}
	
	public String evalOnMap(Map<String, String> detailProperties){
		if (isUnderCondition() && !evaluateGenerationConditionOnMap(detailProperties)){
			return "";
		}
		
		String valueInIm = detailProperties.get(imName);		
		if (isRequired() && !JBIJetUtils.isValid(valueInIm)){
			throw new RuntimeException(" Property ["+imName+"] --> ["+runtimeName+"] is required to generate code for Step ");
		}
		if (!isRequired() && valueInIm == null){
			valueInIm = "";
		}
		if (!(produceConfiguration)){
			return "";
		}else{
			String finalValue =  decorator != null ? decorator.decorate(valueInIm) : valueInIm;
			if (isFlagRef())
				return "<property name=\""+runtimeName+"\"  ref=\""+finalValue+"\"/>\n";
			else
				return "<property name=\""+runtimeName+"\"  value=\""+finalValue+"\"/>\n";
		}
	
	}
	
	public boolean isFlagRef() {
		return flagRef;
	}
	public void setFlagRef(boolean flagRef) {
		this.flagRef = flagRef;
	}
	
	public boolean evaluateGenerationConditionOnMap(Map<String, String> detailProperties){
		
		if (condition.startsWith("if-is-true")){
			String propName = condition.substring(condition.indexOf("(") + 1, condition.indexOf(")"));
			
			String value = detailProperties.get(propName);
			return (value != null && Boolean.valueOf(value));
		}
		if (condition.startsWith("if-is-false")){
				String propName = condition.substring(condition.indexOf("(") + 1, condition.indexOf(")"));
				
				String value = detailProperties.get(propName);
				return (value != null && !Boolean.valueOf(value));
		}
		if (condition.startsWith("if-has-value")){
			String propName = condition.substring(condition.indexOf("(") + 1, condition.indexOf(","));
			String valueToEnableVisibility=condition.substring(condition.indexOf(",") + 1, condition.indexOf(")"));
			String value = detailProperties.get(propName);
			return value.equalsIgnoreCase(valueToEnableVisibility);
		}
		return false;
	}




	public String getConstValue() {
		return constValue;
	}




	public boolean isFlagConst() {
		return flagConst;
	}


	public void setFlagConst(boolean flagConst) {
		this.flagConst = flagConst;
	}


	public void setConstValue(String constValue) {
		this.constValue = constValue;
	}
	
	
	
	
	
}
