/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.metadb.dialogs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class ChooseRuntimeDialog extends TitleAreaDialog {

    private Composite aComposite = null;
     
    private CLabel chooseRuntimeLabel = null;
    private CCombo chooseRuntimeCombo = null;
        
    private Map<String, String> runtimesList = null;
    private Map<String, String> mapNameToId  = null;
    private String choosedRuntimeId = null; 
    
    public ChooseRuntimeDialog( Shell parentShell, Map<String, String> runtimesList) {
    	
		super(parentShell);
		this.runtimesList = runtimesList;
		mapNameToId = new HashMap<String, String>();
		for ( String key : runtimesList.keySet()){
			mapNameToId.put(runtimesList.get(key), key);
		}
    }			

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
	    setTitle(" Spagic ");
	    setMessage(" Choose Runtime for Metadb Publishing ");
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
		aComposite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 4;
    	aComposite.setLayout(layout);
    	
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	

    
    	//Choose Runtime
    	chooseRuntimeLabel = new CLabel(aComposite, SWT.RIGHT);
    	chooseRuntimeLabel.setText(" Choose Runtime ");
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	chooseRuntimeCombo = new CCombo(aComposite, SWT.BORDER | SWT.FLAT );
    	chooseRuntimeCombo.setLayoutData(data);
    	String firstRuntimeName = null;
    	for ( String key : this.mapNameToId.keySet()){
    		if (firstRuntimeName == null)
    			firstRuntimeName = key;
    		chooseRuntimeCombo.add(key);
    	}
    	if (firstRuntimeName != null)
    		chooseRuntimeCombo.setText(firstRuntimeName);   
        return parent; 
	
	}
	
	
	protected void createButtonsForButtonBar(Composite parent) {
		
		// -- Create OK and Cancel buttons by default
		createButton(parent, IDialogConstants.OK_ID, "Proceed...",
				true);
		
	}
	
	@Override
	protected void okPressed() {
		String runtimeName = chooseRuntimeCombo.getText();
		this.choosedRuntimeId = mapNameToId.get(runtimeName);
	}

	public String getChoosedRuntimeId() {
		return choosedRuntimeId;
	}

	public void setChoosedRuntimeId(String choosedRuntimeId) {
		this.choosedRuntimeId = choosedRuntimeId;
	}
	
	
	
}
