/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.metadb.graph;

import org.spagic.metadb.model.Property;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.Step;
import org.spagic.metadb.model.Transition;

import java.io.ByteArrayOutputStream;
import java.util.Set;

import org.eclipse.stp.im.constants.ImConstants;
import org.spagic.graph.util.GraphEndNode;
import org.spagic.graph.util.GraphNode;
import org.spagic.graph.util.GraphStartNode;
import org.spagic.graph.util.JoinNode;
import org.spagic.graph.util.RouterNode;
import org.spagic.graph.util.SplitNode;
import org.spagic.im.out.metadb.IM2MetaDBActivator;
import org.spagic.utils.SpagicLogger;

public class GraphGenerator {

	/**
	 * Generates the SVG graph for the given process
	 * @param process Process to render as a graph
	 * @param graphVizExeLocation Location of Graphviz executable, retrieved from preferences
	 * @param graphVizTempFolder Temporary Location for intermediate files, retrieved from preferences
	 * @return SVG graph (XML string)
	
	public static String getGraphForProcess(org.spagic.metadb.model.Process process, String graphVizExeLocation, String graphVizTempFolder) {
		try{
			org.spagic.graph.util.Graph gv = new org.spagic.graph.util.Graph("graph"+System.currentTimeMillis());
			gv.setDotExeLocation(graphVizExeLocation);
			gv.setTempDirLocation(graphVizTempFolder);
			gv.startGraph();
			Set<Step> steps = process.getSteps();
			for (Step step : steps){
				if (step.isIsStart()){
					gv.addNode(new GraphStartNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
				}else if (step.isIsEnd()){
					gv.addNode(new GraphEndNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
				}else{
					String serviceEntityType = getStepPropertyValue(step, ImConstants.SERVICE_ENTITY_PROPERTY_NAME);
					System.out.println(" ServiceEntityType --> " + serviceEntityType);
					if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_SPLIT_CONTROL))
						gv.addNode(new SplitNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_JOIN_CONTROL)) 
						gv.addNode(new JoinNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ROUTER_CONTROL)) 
						gv.addNode(new RouterNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ITERATION_CONTROL)) 
						gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
				}
			}
			Set<Transition> transitions = process.getTransitions();
			for (Transition transition : transitions) {
				gv.addFlow(new org.spagic.graph.util.Transition(String.valueOf(transition.getId().getIdStepSource()),
							 String.valueOf(transition.getId().getIdStepTarget())));
			}
			gv.endGraph();
		
			byte[] img = gv.getGraph(gv.getDotSource());
			String svg = null;
		
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			bos.write(img);
			bos.flush();
			svg = bos.toString();
			bos.close();
		
		
			// Remove the DTD SVG Declartion
			String svgPart = svg.substring(svg.indexOf("<svg"));
			String svgWithoutDTD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + svgPart;
			//System.out.println("------------------------------------------------\n");
			//System.out.println(svgWithoutDTD);
			//System.out.println("------------------------------------------------\n");
			return svgWithoutDTD;
		}catch(Exception ex){
			SpagicLogger.error(IM2MetaDBActivator.PLUGIN_ID,"Exception in getting SVG For Process " + process.getName(), ex);
			return null;
		}
	}
	 */
	
	public static String getGraphForProcess(org.spagic.metadb.model.Process process, String graphVizExeLocation, String graphVizTempFolder) {
		try{
			org.spagic.graph.util.Graph gv = new org.spagic.graph.util.Graph("graph"+System.currentTimeMillis());
			gv.setDotExeLocation(graphVizExeLocation);
			gv.setTempDirLocation(graphVizTempFolder);
			gv.startGraph();
			Set<Step> steps = process.getSteps();
			

			for (Step step : steps){
				if (step.isIsStart())
					gv.addNode(new GraphStartNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
			}
			
		
			for (Step step : steps){
				if (!(step.isIsEnd() || step.isIsStart())){
					String serviceEntityType = getStepPropertyValue(step, ImConstants.SERVICE_ENTITY_PROPERTY_NAME);
					System.out.println(" ServiceEntityType --> " + serviceEntityType);
					if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_SPLIT_CONTROL))
						gv.addNode(new SplitNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_JOIN_CONTROL)) 
						gv.addNode(new JoinNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ROUTER_CONTROL)) 
						gv.addNode(new RouterNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else if (serviceEntityType.equalsIgnoreCase(ImConstants.SERVICE_ITERATION_CONTROL)) 
						gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
					else gv.addNode(new GraphNode(String.valueOf(step.getIdStep()),String.valueOf(step.getIdStep())));
				}
			}
			
			for (Step step : steps){
				if (step.isIsEnd())
					gv.addNode(new GraphEndNode(String.valueOf(step.getIdStep()), String.valueOf(step.getIdStep())));
			}
			
			Set<Transition> transitions = process.getTransitions();
			for (Transition transition : transitions) {
				gv.addFlow(new org.spagic.graph.util.Transition(String.valueOf(transition.getId().getIdStepSource()),
							 String.valueOf(transition.getId().getIdStepTarget())));
			}
			gv.endGraph();
		
			byte[] img = gv.getGraph(gv.getDotSource());
			String svg = null;
		
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			bos.write(img);
			bos.flush();
			svg = bos.toString();
			bos.close();
		
		
			// Remove the DTD SVG Declartion
			String svgPart = svg.substring(svg.indexOf("<svg"));
			String svgWithoutDTD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + svgPart;
			//System.out.println("------------------------------------------------\n");
			//System.out.println(svgWithoutDTD);
			//System.out.println("------------------------------------------------\n");
			return svgWithoutDTD;
		}catch(Exception ex){
			SpagicLogger.error(IM2MetaDBActivator.PLUGIN_ID,"Exception in getting SVG For Process " + process.getName(), ex);
			return null;
		}
	}
	
	private static String getStepPropertyValue(Step aStep, String propertyName){
		String value = null;
		Property aProperty = null;
		for (PropertyStep pStep : aStep.getPropertySteps()){
			aProperty = pStep.getProperty();
			if (aProperty.getCode().equalsIgnoreCase(propertyName)){
				value = aProperty.getValue();
				break;
			}
		}// for
		return value;
	}	

}
