/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.popup.actions;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.stp.im.ExtractDataRule;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.IterationControl;
import org.eclipse.stp.im.JoinControl;
import org.eclipse.stp.im.MapProperty;
import org.eclipse.stp.im.ObservableAttribute;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.RouterControl;
import org.eclipse.stp.im.Service;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.SplitControl;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.spagic.im.out.metadb.dialogs.ChooseRuntimeDialog;
import org.spagic.im.out.metadb.graph.GraphGenerator;
import org.spagic.im.out.metadb.processor.IProcessor;
import org.spagic.im.out.metadb.processor.RuntimeProcessor;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.ProcessDiagram;
import org.spagic.metadb.model.PropertyProcess;
import org.spagic.metadb.model.PropertyProcessId;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.PropertyStepId;
import org.spagic.metadb.model.Rule;
import org.spagic.metadb.model.TransitionId;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;
import org.spagic.utils.SpagicUtils;
import org.spagic.utils.SpagicUtilsActivator;
import org.spagic.utils.preference.SpagicPreferenceConstants;

public class IM2MetaDBAction implements IObjectActionDelegate {

	public static final Integer USAGE_EXTRACT_DATA = 0;
	public static final Integer USAGE_ITER_TYPE_DETERMINATION = 1;
	
	public static final String RULE_DESCRIPTION = "RuleForIterTypeDetermination:";
	public static final String ITER_DESCRIPTION = "No description available";

	public static final String START_SERVICE_NAME = "StartService";
	public static final String END_SERVICE_NAME = "EndService";
	
	
	private IFile imFile = null;
	
	public IFile getImFile() {
		return imFile;
	}

	public void setImFile(IFile imFile) {
		this.imFile = imFile;
	}

	/**
	 * Constructor for Action1.
	 */
	public IM2MetaDBAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * Open the Session Factory retrieving the parameters from Spagic Preference page
	 * @return A new Session Factory
	 */
	public SessionFactory getSessionFactory(){
		Configuration cfg = new Configuration().configure();

		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		
		String dbUrl = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_URL);
		String dbDriver = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER);
		String dbUser = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_USER);
		String dbPassword = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD);
		String dbDialect = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_HIB_DIALECT);
		String dbShowSql = prefs.getString(SpagicPreferenceConstants.P_SPAGIC_DB_SHOW_SQL);
		
        cfg.setProperty("hibernate.connection.driver_class", dbDriver);
        cfg.setProperty("hibernate.connection.url",dbUrl);
        cfg.setProperty("hibernate.connection.username",dbUser);
        cfg.setProperty("hibernate.connection.password",dbPassword);
        
        cfg.setProperty("hibernate.dialect",dbDialect);
        cfg.setProperty("hibernate.show_sql",dbShowSql);
        
		return cfg.buildSessionFactory();
	}
	
	/**
	 * @return The location of the Graphviz executable.
	 */
	public String getGraphVizExeLocation() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE);
	}
	
	/**
	 * @return The location of the temporary folder used for generating graphs.
	 */
	public String getGraphVizTempFolder() {
		org.eclipse.core.runtime.Preferences prefs = SpagicUtilsActivator.getDefault().getPluginPreferences();
		return prefs.getString(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER);
	}	

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		Session aSession = null;
		Transaction tx = null;
		
		try{
			// Retrieve the Intermediate Model that the user selected
			StpIntermediateModel imModel = retrieveIMModel();
			
			// Begin Hibernate transaction
			aSession = getSessionFactory().openSession();
			tx = aSession.beginTransaction();
			
			// Save all processes in process collection
			ProcessCollection processCollection = imModel.getProcessCollection();
			List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
			List<org.spagic.metadb.model.Process> metaDBProcessList = new ArrayList<org.spagic.metadb.model.Process>(processList.size());
			for (org.eclipse.stp.im.Process process : processList) {
				
				// Save the process and all sub objects
				org.spagic.metadb.model.Process metaDBProcess = saveEntireProcess(aSession, process, imModel);
				metaDBProcessList.add(metaDBProcess);

			}
			
			// Save iter
			saveIter(aSession, imModel, metaDBProcessList);
			
			// Force writing data on database
			aSession.flush();
			tx.commit();
			
			// Clean the session to force reload of the objects
			aSession.clear();
			tx = aSession.beginTransaction();
			
			// Produce all graphs
			for (org.spagic.metadb.model.Process metaDBProcess : metaDBProcessList) {
				
				// Reload the process
				metaDBProcess = AuditDBUtils.getProcessById(aSession, metaDBProcess.getIdProcess());				

				// Create the graph and save it on the metadb
				saveProcessGraph(aSession, metaDBProcess);
			}
			
			// Force writing data on database
			aSession.flush();
			tx.commit();
			
			MessageDialog.openInformation(
					shell,
					"IntermediateModel 2 MetaDB Plug-in",
					"Export Intermediate Model to MetaDB was executed.");
			
		} catch (IllegalArgumentException iaex) {
			if (tx != null) {
				tx.rollback();
			}
			SpagicLogger.error("IntermediateModel 2 MetaDB Plug-in", iaex);
			MessageDialog.openError(
					shell,
					"IntermediateModel 2 MetaDB Plug-in",
					iaex.getMessage());
		} catch (Exception e) {
			if (tx != null) {
				tx.rollback();
			}
			SpagicLogger.error("IntermediateModel 2 MetaDB Plug-in", e);
			MessageDialog.openError(
					shell,
					"IntermediateModel 2 MetaDB Plug-in",
					"Error occurred during export from intermediate model to MetaDB: " + e);
		} finally {
			if (aSession != null) {
				aSession.close();
			}
		}
		
		
	}
	
	/**
	 * Save a process on meta-db and all related sub-objects (steps, transitions, etc).
	 * @param aSession Hibernate session
	 * @param process Intermediate Model process to save
	 * @return The saved process
	 */
	private org.spagic.metadb.model.Process saveEntireProcess(Session aSession, org.eclipse.stp.im.Process process, StpIntermediateModel stpIntermediateModel) {
		// Retrieve the processor to apply to this process before saving on the database
		// TODO Dialog richiesta scelta binding
		String runtimeBinding = retrieveBindingForProcess(process);
		IProcessor processor = RuntimeProcessor.getInstance().getProcessor(runtimeBinding);
		
		// Apply the processor to the process: this can change the process, but if
		// we dont save the process, the changes are not persisted on the IM (that is
		// the desired behaviour)
		process = processor.preProcess(process, stpIntermediateModel);
		
		
		// Save the process
		org.spagic.metadb.model.Process metaDBProcess = saveProcess(aSession, process, processor);
		
		// Retrieve the transitions
		List<Transition> transitionList = process.getTransitions();
		
		// Save the steps
		List<Step> stepList = process.getSteps();
		Map<String, org.spagic.metadb.model.Step> stepMap = new HashMap<String, org.spagic.metadb.model.Step>();
		Service serviceForStep = null;
		for (Step step : stepList) {
			// Save the step
			/*boolean isStartStep = isStartStep(transitionList, step);
			boolean isEndStep = isEndStep(transitionList, step);
			*/
			serviceForStep = step.getServiceModel();
			boolean isStartStep = (serviceForStep != null ? serviceForStep.getServiceName().equalsIgnoreCase(START_SERVICE_NAME) : false);
			boolean isEndStep = (serviceForStep != null ?  serviceForStep.getServiceName().equalsIgnoreCase(END_SERVICE_NAME) : false);
			
			org.spagic.metadb.model.Step metaDBStep = saveStep(aSession, step, isStartStep, isEndStep, process, metaDBProcess, runtimeBinding);
			stepMap.put(new QName(metaDBStep.getNamespace(), metaDBStep.getName()).toString(), metaDBStep);
		}
		
		aSession.flush();
		
		// Saves the transitions
		for (Transition transition : transitionList) {
			saveTransition(aSession, transition, process, metaDBProcess, stepMap);
		}
		
		return metaDBProcess;
	}
	
	/**
	 * Retrieve the Intermediate Model that the user selected.
	 * @return The Intermediate Model that the user selected.
	 */
	private StpIntermediateModel retrieveIMModel() {
		// Read the bpmn emf model
		ResourceSet resourceSetIM = new ResourceSetImpl();
		URI uri = URI.createFileURI(imFile.getFullPath().toString());

		// Force Package Registration
		@SuppressWarnings("unused")
		ImPackage imPackage = ImPackage.eINSTANCE;

		// load on demand = true
		Resource resourceIM = resourceSetIM.getResource(uri, true);	
								
		// Get The intermediate model
		StpIntermediateModel imModel= (StpIntermediateModel)resourceIM.getContents().get(0);
		return imModel;
	}
	
	/**
	 * Retrieve the binding used by the process
	 * @param process Process to store on database
	 * @return The binding used in the process
	 */
	private String retrieveBindingForProcess(org.eclipse.stp.im.Process process) {
		
			Map<String, String> usedRuntimes = SpagicUtils.getRuntimeUsedByProcess(process);
			String choosedRuntimeId = null;
			
			if (usedRuntimes.values().size() == 1){
				
				choosedRuntimeId = usedRuntimes.keySet().iterator().next();
				MessageDialog.openInformation(new Shell(), "Spagic", "Process ["+process.getName()+"] Only ["+usedRuntimes.values().iterator().next()+"] Runtime Found");
				
			}else{
				
				ChooseRuntimeDialog aChooseRuntimeDialog = new ChooseRuntimeDialog(new Shell(),usedRuntimes);
				aChooseRuntimeDialog.open();
				choosedRuntimeId = aChooseRuntimeDialog.getChoosedRuntimeId();
			}
			
			return choosedRuntimeId;
	
	}
	
	/**
	 * Determine if a step is a starting step.
	 * @param transitionList List of all process transitions
	 * @param currentStep Step to examine
	 * @return True if the step is a starting step in the process
	 */
	/*private boolean isStartStep(List<Transition> transitionList, Step currentStep) {
		for (Transition transition : transitionList) {
			Step targetStep = transition.getTarget();
			if (targetStep.equals(currentStep)) {
				return false;
			}
		}
		return true;		
	}*/
	
	/**
	 * Determine if a step is an ending step.
	 * @param transitionList List of all process transitions
	 * @param currentStep Step to examine
	 * @return True if the step is an ending step in the process
	 */
	/*private boolean isEndStep(List<Transition> transitionList, Step currentStep) {
		for (Transition transition : transitionList) {
			Step sourceStep = transition.getSource();
			if (sourceStep.equals(currentStep)) {
				return false;
			}
		}
		return true;		
	}*/
		
	/**
	 * Saves on the meta-db without sub-objects.
	 * @param aSession Hibernate session
	 * @param process Process to save
	 * @return The saved process
	 */
	private org.spagic.metadb.model.Process saveProcess(Session aSession, org.eclipse.stp.im.Process process, IProcessor processor) {
		String processName = process.getName();
		String processNamespace = null;
		String processVersion = null;
		
		// Retrieve process name and version to check that the same process doesn't
		// exist already in the database
		{
			Property propertyVersion = process.getProperties().get(ImConstants.PROCESS_VERSION);
			processVersion = (String)propertyVersion.getValue();
			processNamespace = SpagicUtils.getNameSpaceForProcess(process);

			org.spagic.metadb.model.Process existingProcess = AuditDBUtils.getProcessByQNameAndVersion(aSession, new QName(processNamespace, processName), processVersion);
			if (existingProcess != null) {
				throw new IllegalArgumentException("Process identified by namespace " + processNamespace + ", name " + processName + ", and version" + processVersion + " already exists in database");
			}
		}
		
		// Create a new process
		org.spagic.metadb.model.Process metaDBProcess = new org.spagic.metadb.model.Process();
		
		metaDBProcess.setName(processName);
		metaDBProcess.setProcessVersion(processVersion);
		metaDBProcess.setNamespace(processNamespace);
		metaDBProcess.setDescription(process.getDescritpion());
		metaDBProcess.setTypeService(AuditDBUtils.getTypeServiceWithValue(aSession, processor.getTechnology() + "-PROCESS"));
		// TODO
		//metaDBProcess.setUddiName(uddiName);
		//metaDBProcess.setUddiOrg(uddiOrg);
		metaDBProcess.setStartdate(new Date());
		metaDBProcess.setEnddate(null);
		metaDBProcess.setState(ProcessInstanceStateConstants.PROCESS_ACTIVE);
		aSession.save(metaDBProcess);
		
		
		// Store all other properties
		for (Map.Entry<String, Property> entry : process.getProperties()) {
			Property property = entry.getValue();
			
			org.spagic.metadb.model.Property metaDBProperty = new org.spagic.metadb.model.Property();
			
			// TODO
			// Verify if it can be useful decouple the STP IM property names and the MetaDB property names
			metaDBProperty.setCode(property.getKey());
			metaDBProperty.setValue((String)property.getValue());
			aSession.save(metaDBProperty);
			
			// Associate Property and Process
			PropertyProcessId propertyProcessId = new PropertyProcessId(metaDBProperty.getIdProperty(), metaDBProcess.getIdProcess());
			PropertyProcess metaDBPropertyProcess = new PropertyProcess(propertyProcessId, metaDBProcess, metaDBProperty);
			aSession.save(metaDBPropertyProcess);
		}
		
		return metaDBProcess;
	}
	
	private void createMetaDBPropertyForStep(Session aSession, org.spagic.metadb.model.Step aMetaDbStep, String propertyKey, String propertyValue){
		org.spagic.metadb.model.Property metaDBProperty = null;
		
		metaDBProperty = new org.spagic.metadb.model.Property();
		
		// TODO
		// Verify if it can be useful decouple the STP IM property names and the MetaDB property names
		metaDBProperty.setCode(propertyKey);
		metaDBProperty.setValue(propertyValue);
		aSession.save(metaDBProperty);
		
		// Associate Property and Step
		PropertyStepId propertyStepId = new PropertyStepId(aMetaDbStep.getIdStep(), metaDBProperty.getIdProperty());
		PropertyStep metaDBPropertyStep = new PropertyStep(propertyStepId, aMetaDbStep, metaDBProperty);
		aSession.save(metaDBPropertyStep);
	}
	/**
	 * Saves a single process step
	 * @param aSession Hibernate session
	 * @param step Step to save
	 * @param isStartStep True if the step is a starting step
	 * @param isEndStep True if the step is an ending step
	 * @param process The step owner process
	 * @param metaDBProcess The already saved owner process
	 * @return The savedn step
	 */
	private org.spagic.metadb.model.Step saveStep(Session aSession,
			org.eclipse.stp.im.Step step, boolean isStartStep,
			boolean isEndStep,
			org.eclipse.stp.im.Process process,
			org.spagic.metadb.model.Process metaDBProcess, String runtimeBinding) {
		
		org.spagic.metadb.model.Step metaDBStep = new org.spagic.metadb.model.Step();

		// Compute step QName
		QName stepQName = SpagicUtils.getQNameForStep(process, step);

		metaDBStep.setDescription(step.getDescription());
		metaDBStep.setName(stepQName.getLocalPart());
		metaDBStep.setNamespace(stepQName.getNamespaceURI());
		metaDBStep.setProcess(metaDBProcess);
		metaDBStep.setProcessVersion(metaDBProcess.getProcessVersion());
		metaDBStep.setIsStart(isStartStep);
		metaDBStep.setIsEnd(isEndStep);
		aSession.save(metaDBStep);
		
		
	
		for (Map.Entry<String, Property> entry : step.getProperties()) {
			Property property = entry.getValue();
			createMetaDBPropertyForStep(aSession, metaDBStep, property.getKey(), (String)property.getValue());
			
		}
		
		// AZ - BEGIN - 
		// Save Three Additional Property
		// Service Name
		// Service Binding Name
		// Service Type
		Service stepServiceModel = step.getServiceModel();
		ServiceBinding rightServiceBinding = SpagicUtils.getRightStepBinding(step, runtimeBinding);
		
		String serviceName = "GenericService";
		String serviceEntity = ImConstants.SERVICE_GENERIC;
		String rightServiceBindingName = "AutoGeneratedBinding";
		
		if (stepServiceModel != null){
			serviceName = stepServiceModel.getServiceName();
				
			if (stepServiceModel instanceof JoinControl) {
				serviceEntity = ImConstants.SERVICE_JOIN_CONTROL;	
			}else if (stepServiceModel instanceof RouterControl) {
				serviceEntity = ImConstants.SERVICE_ROUTER_CONTROL;
			}else if (stepServiceModel instanceof SplitControl) {
				serviceEntity = ImConstants.SERVICE_SPLIT_CONTROL;
			}else if (stepServiceModel instanceof IterationControl) {
				serviceEntity = ImConstants.SERVICE_ITERATION_CONTROL;
			}
		}
		if (rightServiceBinding != null)
			rightServiceBindingName = rightServiceBinding.getName();
		
		
		createMetaDBPropertyForStep(aSession, 
				metaDBStep, 
				ImConstants.IM_SERVICE_NAME, 
				serviceName);
		
		
		
		createMetaDBPropertyForStep(aSession, 
				metaDBStep, 
				ImConstants.IM_SERVICE_BINDING_NAME, 
				rightServiceBindingName);
		
		
		createMetaDBPropertyForStep(aSession, 
									metaDBStep, 
									ImConstants.SERVICE_ENTITY_PROPERTY_NAME, 
									serviceEntity);
		
		// AZ - END
		
		// Store the relevant data associations
		List<ObservableAttribute> observableAttributes = step.getObservableAttributes();
		int ruleCounter = 0;
		for (ObservableAttribute observableAttribute : observableAttributes) {
			ExtractDataRule dataRule = observableAttribute.getObservableAttributeExtractRule();
			
			// Retrieve the rule by name (should be unique)
			String ruleName = dataRule.getName();
			Rule rule = AuditDBUtils.getRuleByName(aSession, ruleName);
			
			org.spagic.metadb.model.StepRuleId metaDBStepRuleId = new org.spagic.metadb.model.StepRuleId(metaDBStep.getIdStep(), rule.getIdRule());
			org.spagic.metadb.model.StepRule metaDBStepRule = new org.spagic.metadb.model.StepRule();
			metaDBStepRule.setId(metaDBStepRuleId);
			metaDBStepRule.setOrderId(ruleCounter++);
			aSession.save(metaDBStepRule);
		}
		
		return metaDBStep;
	}
	
	 /**
	 * Saves a single transition
	 * @param aSession Hibernate session
	 * @param transition The transition to save
	 * @param process The transition owner process
	 * @param metaDBProcess The already saved owner process
	 * @param stepMap A map containing all already saved steps
	 * @return The saved transition
	 */
	private org.spagic.metadb.model.Transition saveTransition(
			Session aSession, org.eclipse.stp.im.Transition transition,
			org.eclipse.stp.im.Process process,
			org.spagic.metadb.model.Process metaDBProcess,
			Map<String, org.spagic.metadb.model.Step> stepMap) {
		Step sourceStep = transition.getSource();
		Step targetStep = transition.getTarget();

		// Retrieve metaDB steps
		QName sourceStepQName = SpagicUtils.getQNameForStep(process, sourceStep);
		QName targetStepQName = SpagicUtils.getQNameForStep(process, targetStep);

		org.spagic.metadb.model.Step metaDBSourceStep = stepMap
				.get(sourceStepQName.toString());
		org.spagic.metadb.model.Step metaDBTargetStep = stepMap
				.get(targetStepQName.toString());

		org.spagic.metadb.model.Transition metaDBTransition = new org.spagic.metadb.model.Transition();
		metaDBTransition.setProcess(metaDBProcess);

		TransitionId transitionId = new TransitionId(metaDBSourceStep
				.getIdStep(), metaDBTargetStep.getIdStep());
		metaDBTransition.setId(transitionId);
		aSession.save(metaDBTransition);

		return metaDBTransition;
	}
	
	/**
	 * Produce and save the process graph (actually a SVG).
	 * @param aSession Hibernate session
	 * @param metaDBProcess The saved process
	 */
	private void saveProcessGraph(Session aSession,
			org.spagic.metadb.model.Process metaDBProcess) {

		// Create the graph to store on the metadb
		String processGraph = GraphGenerator.getGraphForProcess(metaDBProcess,
				getGraphVizExeLocation(), getGraphVizTempFolder());

		// Check if there is a graph already saved
		ProcessDiagram processDiagram = AuditDBUtils.getProcessDiagram(
				aSession, metaDBProcess);
		if (processDiagram != null) {
			// The diagram already exists: update it
			processDiagram.setSvgDiag(processGraph);
		} else {
			// The diagram doesnt exist: create a new diagram on database
			processDiagram = new ProcessDiagram(metaDBProcess.getIdProcess(),
					metaDBProcess, processGraph);
		}
		aSession.save(processDiagram);
	}
	
	/**
	 * Save the iter and all the related associations
	 * @param aSession Hibernate session
	 * @param imModel Intermediate model containing the iter
	 */
	private void saveIter(Session aSession, StpIntermediateModel imModel, List<org.spagic.metadb.model.Process> metaDBProcessList) {
		EMap<String, Property> imProperties = imModel.getProperties();		
		Property iterNameProperty = null;
		String iterName = null;
		Property iterVersionProperty = null;
		String iterVersion = null;
		org.spagic.metadb.model.Rule ruleForIterTypeDetermination = null;
		
		// Retrieve iter informations
		iterNameProperty = imProperties.get(ImConstants.IM_ITER_NAME);
		if (iterNameProperty == null) {
			// No iter exists
			return;
		}
		iterName = (String)iterNameProperty.getValue();
		
		iterVersionProperty = imProperties.get(ImConstants.ITER_VERSION);
		if (iterVersionProperty == null) {
			// No version exists
			return;
		}
		iterVersion = (String)iterVersionProperty.getValue();
				
		// Create rule for iter determination, if any
		// The rule can also non-exists, if the process belongs to a single iter
		Property ruleForDeterminationProperty = imProperties.get(ImConstants.IM_ITER_RULE_FOR_DETERMINATION);
		if (ruleForDeterminationProperty != null) {
			String ruleForDetermination = (String)ruleForDeterminationProperty.getValue();
			
			ruleForIterTypeDetermination = new org.spagic.metadb.model.Rule();
			ruleForIterTypeDetermination.setName(RULE_DESCRIPTION + iterName);
			ruleForIterTypeDetermination.setExpr(ruleForDetermination);
			ruleForIterTypeDetermination.setRuleUsage(USAGE_ITER_TYPE_DETERMINATION);
			aSession.save(ruleForIterTypeDetermination);
		}
		
		// Create Iter
		org.spagic.metadb.model.PoolType poolType = new org.spagic.metadb.model.PoolType(iterName, ITER_DESCRIPTION, iterVersion);
		poolType.setRule(ruleForIterTypeDetermination);
		aSession.save(poolType);

		
		// Retrieve iter attributes
		Property attributeSizeProperty = imProperties.get(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_LIST_SIZE);
		Integer attributeSize = Integer.valueOf((String)attributeSizeProperty.getValue());
		
		MapProperty mp = (MapProperty)imProperties.get(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_MAP);
		EMap<String, Property> mapValues = mp.getValue();
		
		for (int i = 0; i < attributeSize; i++) {
			Property bp = mapValues.get(MessageFormat.format(
					ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_ID, i));
			
			String attributeId = (String)bp.getValue();
			
			// Save relation attribute-iter
			org.spagic.metadb.model.AttributePool ap = new org.spagic.metadb.model.AttributePool();
			ap.setId(new org.spagic.metadb.model.AttributePoolId(Long.valueOf(attributeId), poolType.getIdPoolType()));
			aSession.save(ap);
		}
		
		
		
		
		// Saves the processes belonging to the iter
		
		// Retrieve first iter process name
		Property firstProcessNameProperty = imProperties.get(ImConstants.IM_ITER_FIRST_PROCESS_NAME);
		ProcessCollection processCollection = imModel.getProcessCollection();
		List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
		int orderCriteria = 1;	// This counter increment only if it's not the first process
		int counter = 0;		// This counter increment always
		for (org.eclipse.stp.im.Process process : processList) {
			
			org.spagic.metadb.model.PoolTypeProcess poolTypeProcess = new org.spagic.metadb.model.PoolTypeProcess();
			poolTypeProcess.setId(new org.spagic.metadb.model.PoolTypeProcessId(metaDBProcessList.get(counter++).getIdProcess(), poolType.getIdPoolType()));
			poolTypeProcess.setPoolType(poolType);
			poolTypeProcess.setOrderCriteria(process.getName().equals(firstProcessNameProperty) ? 0 : orderCriteria++);
			aSession.save(poolTypeProcess);
		}
	}

	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		
		setImFile((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
	
	
}
