/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.registry.utils.JAXRServiceRegistryImpl;
import org.spagic.utils.SpagicLogger;
import org.spagic.utils.SpagicUtilsActivator;
import org.spagic.utils.preference.SpagicPreferenceConstants;


/**
 * The activator class controls the plug-in life cycle
 */
public class SpagicUIActivator extends AbstractUIPlugin implements IPropertyChangeListener {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.spagic.ui";

	// The shared instance
	private static SpagicUIActivator plugin;
	
	private SessionFactory sf = null;
	
	private IServiceRegistry serviceRegistry = null;
	
	public static String HL72XML = "plain2xml";
	public static String XML2HL7 = "xml2plain";
	/**
	 * The constructor
	 */
	public SpagicUIActivator() {
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		configureSpagicStudioHL7(false);
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static SpagicUIActivator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	public void configureSpagicStudioConfDirectory(){
		File cfgDirectory = new File(getSpagicStudioConfigurationPath());
		if (!cfgDirectory.exists())
			cfgDirectory.mkdirs();
	}
	
	public SessionFactory buildSessionFactory(){
		Configuration cfg = new Configuration().configure();
		org.eclipse.core.runtime.Preferences pref = SpagicUtilsActivator.getDefault().getPluginPreferences();
		
		String dbUrl = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_URL);
		String dbDriver = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER);
		String dbUser = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_USER);
		String dbPassword = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD);
		String dbDialect = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_HIB_DIALECT);
		String dbShowSql = pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_SHOW_SQL);
		
		cfg.setProperty("hibernate.connection.driver_class", dbDriver);
        cfg.setProperty("hibernate.connection.url",dbUrl);
        cfg.setProperty("hibernate.connection.username",dbUser);
        cfg.setProperty("hibernate.connection.password",dbPassword);
        
        cfg.setProperty("hibernate.dialect",dbDialect);
        cfg.setProperty("hibernate.show_sql",dbShowSql);
        
		return cfg.buildSessionFactory();
	}
	
	
	

		public IServiceRegistry getServiceRegistry(){
			if (this.serviceRegistry == null){
				initServiceRegistry();
			}
			return serviceRegistry;
		}
		
		public void initServiceRegistry(){
			org.eclipse.core.runtime.Preferences pref = SpagicUtilsActivator.getDefault().getPluginPreferences();
			Properties properties = new Properties();
			properties.setProperty("javax.xml.registry.queryManagerURL", pref.getString(SpagicPreferenceConstants.P_SERVICE_REGISTRY_INQUIRY_URL));
			properties.setProperty("javax.xml.registry.lifeCycleManagerURL", pref.getString(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PUBLISH_URL));
			properties.setProperty("registry.username", pref.getString(SpagicPreferenceConstants.P_SERVICE_REGISTRY_USER));
			properties.setProperty("registry.password", pref.getString(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PASSWORD));
			properties.setProperty("metadb.db.driver", pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER));
			properties.setProperty("metadb.db.user",pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_USER) );
			properties.setProperty("metadb.db.password",pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD));
			properties.setProperty("metadb.db.url", pref.getString(SpagicPreferenceConstants.P_SPAGIC_DB_URL));
			try{
				serviceRegistry = new JAXRServiceRegistryImpl(properties);
			}catch(Throwable  e){
				SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error in Service Registry Initialization ", e);
			}
		}
		
		public  SessionFactory getSessionFactory(){
			if (sf == null)
				this.sf = buildSessionFactory();
			return sf;
		}
		
		public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event) {	
			this.sf = buildSessionFactory();
			initServiceRegistry();
			
		}
		
		
		
		
		public static String getSpagicStudioConfigurationPath(){
			IPath path = getDefault().getStateLocation();
			String cfgDirPath = path.toOSString();
			return cfgDirPath;
		}
		
		public static void copyInCfgDirectory(File destination, String bundleRelatedPath){
			BufferedReader sourceReader = null;
			BufferedWriter bw = null;
			try{
			 sourceReader = new BufferedReader (new InputStreamReader(getResourceURL(bundleRelatedPath).openStream()));
			 bw = new BufferedWriter(new FileWriter(destination));
			 String line = null;
			 while ( (line = sourceReader.readLine()) != null){
				 bw.write(line);
				 bw.write("\n");
				 bw.flush();
			 }
			}catch (Exception e) {
				e.printStackTrace();
			}finally{
				try{
					sourceReader.close();
					bw.close();
				}catch (Exception e) {
					e.printStackTrace();
				}
				
			}
		}
		public static URL getResourceURL(String path) throws IOException {
			if (PLUGIN_ID == null || path == null)
	    		throw new IllegalArgumentException();
			Bundle bundle = Platform.getBundle(PLUGIN_ID);
			URL url = FileLocator.find(bundle, new Path(path), null);
			return FileLocator.resolve(url);
		}
		
		public static File getHL7MapFile(){
			return new File(getSpagicStudioConfigurationPath() + "/hl7map.xml");
		}
		
		public static void configureSpagicStudioHL7(boolean overwrite){
			File hl7MapFile = getHL7MapFile();

			if (overwrite && hl7MapFile.exists()){ 
				hl7MapFile.delete();
			}
			
			if (!hl7MapFile.exists()) 
				copyInCfgDirectory(hl7MapFile, "/conf/hl7map.xml");
		}
		
		public static Map<String, String>  getHL7Map(String transformationType){
			Map<String, String> resultingTransormations = new HashMap<String, String>();
			try{
				SAXReader reader = new SAXReader(false);
				Document hl7Document = reader.read(getHL7MapFile());
				List<org.dom4j.Node> nodeList = hl7Document.selectNodes("/TRANSFORMATIONS/TRANSFORMATION[@type='"+transformationType+"']");
				
				for (org.dom4j.Node n : nodeList){
					resultingTransormations.put(n.valueOf("@name"), n.valueOf("@class"));
				}
			}catch (Exception e) {
				SpagicLogger.error(PLUGIN_ID, e.getMessage(), e);
			}
			return resultingTransormations;	
		}
		
		
		
		

}
