/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.actions;

import java.io.BufferedReader;
import java.io.FileReader;

import javax.xml.registry.infomodel.ClassificationScheme;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class LoadTaxonomiesAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public LoadTaxonomiesAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		String fileName = openFileDialog();
		if (fileName != null){
			try{
				loadTaxonomies(fileName);
				MessageDialog.openInformation(window.getShell(), "Spagic", "Taxonomies Loaded Successfully");
			}catch(Exception e){
				SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
				MessageDialog.openError(window.getShell(),"Spagic", "Error in Loading Taxonomies");
			}
		}
		
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
	
	public void loadTaxonomies(String fileName) throws Exception {
		BufferedReader br  = null;
		IServiceRegistry serviceRegistry = null;
		try{
			serviceRegistry = SpagicUIActivator.getDefault().getServiceRegistry();
			serviceRegistry.initRegistry();

			br = new BufferedReader(new FileReader(fileName));
			String[] tokens = null;
			String line = null;
			
			String taxonomyName = null;
			ClassificationScheme tax = null;
			while ((line = br.readLine()) != null) {
				if (line.startsWith("#")){
					taxonomyName = line.substring(1);
					System.out.println(" taxonomy ["+taxonomyName+"]");
					if (serviceRegistry.getClassificationSchemeByName(taxonomyName) != null ){
						MessageDialog.openInformation(window.getShell(),"Spagic", "Taxonomy already registered ");
					}else{
						tax = serviceRegistry.newClassificationScheme(taxonomyName, taxonomyName);
						tax = serviceRegistry.publishClassificationScheme(tax);
						SpagicLogger.info(SpagicUIActivator.PLUGIN_ID, "-> Loaded taxonomy ["+taxonomyName+"] In Business registry ");
					}
				}else{
					String classificationValue = line.substring(2);
					System.out.println("Add Value["+classificationValue+"]");
					//serviceRegistry.addValueToClassificationScheme(cScheme, classificationValue)
					serviceRegistry.addValueToClassificationScheme(tax, classificationValue);
					SpagicLogger.info(SpagicUIActivator.PLUGIN_ID,"-> Loaded value ["+ classificationValue +"] for taxonomy ["+taxonomyName+"] In Business registry ");
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			throw e;
		}finally{
			try{
				br.close();
			}catch (Exception ioe) {
				ioe.printStackTrace();
			}
			serviceRegistry.finalizeRegistry();
		}
	}
	
	protected String openFileDialog(){
		 FileDialog fd = new FileDialog(window.getShell(), SWT.OPEN);
	     fd.setText("Open a valid Taxonomy file ");
	     
	     String[] filterExt = { "*.tax" };
	     fd.setFilterExtensions(filterExt);
	     String selected = fd.open();
	     System.out.println(selected);
	     return selected;
	}
}