/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.stp.bpmn.Artifact;
import org.eclipse.stp.bpmn.BpmnDiagram;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.view.ObservableAttributeTreeElement;

public class ConfigureIterDialog extends SpagicTitleAreaDialog {

    private Composite aComposite = null;
    
    private CLabel iterStatusLabel = null;
    private CLabel iterStatusLabelTextField = null;
    
    private CLabel iterTypeNameLabel = null;
    private Text iterTypeNameTextField = null;
    
    private CLabel iterFirstProcessLabel = null;
    private CCombo iterFirstProcessCombo = null;
        
    private CLabel ruleExpressionLabel = null;
    private Text ruleExpressionTextArea = null;
   
    private SashForm internalSash  = null; 
    private SashForm sashAttributes  = null;
    private SashForm sashRuleIterType  = null;
    
    private Table tableAttributesForIterType = null;
    private TableViewer tbvAttributesForIterType = null;
    
    private Button btnChooseAttribute = null;
	private BpmnDiagram bpmnDiagram = null;
	
	
	private List<ObservableAttributeTreeElement> candidateAttributes = new ArrayList<ObservableAttributeTreeElement>();
	
	// 
	
	private List<ObservableAttributeTreeElement> selectedAttributesForIter = new ArrayList<ObservableAttributeTreeElement>();
	private String ruleForIterTypeDetermination = null;
	private String firstProcessOfTheIter = null;
	
	private final String CONFIGURED = "Configured";
	private final String NOT_CONFIGURED = "Not Configured";
	private final String TO_SAVE = "To Save";
	
	private String iterStatus = NOT_CONFIGURED;
	
	
    public ConfigureIterDialog( Shell parentShell, BpmnDiagram bpmnDiagram) {
		
		super(parentShell);	
		this.bpmnDiagram = bpmnDiagram;
		initCandidateAttributes();
		initAllProcessesList();
		EAnnotation iterAnnotation =  bpmnDiagram.getEAnnotation(ImConstants.IM_ITER_ANNOTATION);
		if (iterAnnotation != null){
			this.iterStatus = CONFIGURED;
			this.initSelectAttributesForIter(iterAnnotation);
			this.ruleForIterTypeDetermination = iterAnnotation.getDetails().get(ImConstants.IM_ITER_RULE_FOR_DETERMINATION);
		    this.firstProcessOfTheIter = iterAnnotation.getDetails().get(ImConstants.IM_ITER_FIRST_PROCESS_NAME);
		}else{
			this.iterStatus = NOT_CONFIGURED;
			this.selectedAttributesForIter = new ArrayList<ObservableAttributeTreeElement>();
		}
    }
    public void initAllProcessesList(){
    	
    }
    public void initSelectAttributesForIter(EAnnotation iterAnnotation){
    	EMap<String, String> iterAnnotationDetails = iterAnnotation.getDetails();
    	Integer size = Integer.valueOf(iterAnnotationDetails.get(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_LIST_SIZE));
		String attId = null;
		String attName = null;
		for ( int i = 0; i < size; i++){
			attId = iterAnnotationDetails.get(MessageFormat.format(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_ID, i));
			attName = iterAnnotationDetails.get(MessageFormat.format(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_NAME, i));
			this.selectedAttributesForIter.add(new ObservableAttributeTreeElement(attId, attName));
		}
    }
    
    
    @Override
	protected Point getInitialSize() {
		//System.out.println( super.getInitialSize());
		return new Point(600, 400 );
	}
    
	public Map<String, ObservableAttributeTreeElement> getObservableAttributeMapForPool(Pool aPool){
    	
    	Map<String, ObservableAttributeTreeElement> observableAttributeMapForPool = new HashMap<String, ObservableAttributeTreeElement>();
    	
    	EAnnotation observableAttributeAnnotation = null;
    	EMap<String, String> observableAttributeAnnotationDetails = null;
    	Integer observableAttributeAnnotationSize = null;
    	
    	for (Artifact aArtifact : aPool.getArtifacts() ){
    		observableAttributeAnnotation = aArtifact.getEAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE);
    		if (observableAttributeAnnotation != null){
    			observableAttributeAnnotationDetails = observableAttributeAnnotation.getDetails();
    			
    			observableAttributeAnnotationSize = Integer.valueOf(observableAttributeAnnotationDetails.get(ImConstants.IM_OBSERVABLE_ATTRIBUTE_LIST_SIZE));
    			
    			String attId = null;
    			String attName = null;

    			for ( int i = 1; i <= observableAttributeAnnotationSize; i++){
    				attId = observableAttributeAnnotationDetails.get(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_ID, i));
    				attName = observableAttributeAnnotationDetails.get(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_NAME, i));
    				observableAttributeMapForPool.put(attId, new ObservableAttributeTreeElement(attId, attName));
    			}
    		}
    	}
    	
    	return observableAttributeMapForPool;
    
    }
    
    public void initCandidateAttributes(){
    	
    	List<Pool> pools = bpmnDiagram.getPools();
    	Map<String, Map<String, ObservableAttributeTreeElement>> poolsObservableAttributesMaps = new HashMap<String, Map<String,ObservableAttributeTreeElement>>();
    	if (pools != null && pools.size() > 0){
    		for (Pool aPool : pools){
    			poolsObservableAttributesMaps.put(aPool.getID(), getObservableAttributeMapForPool(aPool));
    		}
    		Pool[] poolArray  = new Pool[1];
        	poolArray = pools.toArray(poolArray);
        	
        	if (poolArray.length == 1){
        		// I've only one pool the attribute are the relevant data of the first one
        		this.candidateAttributes.addAll(poolsObservableAttributesMaps.get(poolArray[0].getID()).values());
        		return;
        	}else{
        	
            	
            	Map<String, ObservableAttributeTreeElement> observableAttributeMapForFirstPool  = poolsObservableAttributesMaps.get(poolArray[0].getID());
            	
            	Map<String, ObservableAttributeTreeElement> observableAttributeMapForJPool = null;
            	for( String attributeId : observableAttributeMapForFirstPool.keySet()){
            		// For each attribute in the first pool,
            		// look if it's present on other pool
            		boolean candidate = true;
            		for ( int j=1; j < poolArray.length && candidate; j++){
            			observableAttributeMapForJPool = poolsObservableAttributesMaps.get(poolArray[j].getID());
            			if (observableAttributeMapForJPool.get(attributeId) == null){
            				candidate = false;
            			}
            		}
            		
            		if (candidate){
            			this.candidateAttributes.add(observableAttributeMapForFirstPool.get(attributeId));
            		}
            	}
        	}
    	}
    }
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
	    setTitle(" Iter Configuration ");
	    setMessage( " Select the first process of the iter, and the set of attributes that identify the iter\n ");
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		final ConfigureIterDialog theDialog = this;
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       
        String imgPath = "icons" + File.separator + "add.png";
    	Image addImage = SpagicUIActivator.getImageDescriptor(
				imgPath).createImage();
    	
		aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 4;
    	aComposite.setLayout(layout);
    	
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	
    	// Iter Status
    	iterStatusLabel = new CLabel(aComposite, SWT.RIGHT);
    	iterStatusLabel.setText(" Iter Information Status ");
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	iterStatusLabelTextField = new CLabel(aComposite, SWT.LEFT );
    	iterStatusLabelTextField.setLayoutData(data);
    	iterStatusLabelTextField.setText( iterStatus );
    	

    	//Iter Name
    	iterTypeNameLabel = new CLabel(aComposite, SWT.RIGHT);
    	iterTypeNameLabel.setText(" Iter Name ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	iterTypeNameTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	iterTypeNameTextField.setLayoutData(data);
    	iterTypeNameTextField.setText((bpmnDiagram != null ? ( bpmnDiagram.getName() != null ? bpmnDiagram.getName() : "" ) : ""));
    	iterTypeNameTextField.setEditable(false);
    
    
    	//Iter First Process
    	iterFirstProcessLabel = new CLabel(aComposite, SWT.RIGHT);
    	iterFirstProcessLabel.setText(" First Process ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	iterFirstProcessCombo = new CCombo(aComposite, SWT.BORDER | SWT.FLAT );
    	iterFirstProcessCombo.setLayoutData(data);
    	Pool aPool = null;
    	List<Pool> allProcesses = bpmnDiagram.getPools();
    	
    	for (int i=0; i < allProcesses.size(); i++){
    		aPool = allProcesses.get(i);
    		iterFirstProcessCombo.add(aPool.getName(), i);
    	}
    	if (firstProcessOfTheIter != null)
    		iterFirstProcessCombo.setText(firstProcessOfTheIter);


    	iterFirstProcessCombo.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent selE) {
				theDialog.iterStatus = TO_SAVE;
				theDialog.iterStatusLabelTextField.setText(TO_SAVE);
				
			}

			public void widgetSelected(SelectionEvent selE) {
				widgetDefaultSelected(selE);
				
			}
    		
    	});
    	
    	//Sash Attributes
    	this.sashAttributes = new SashForm(internalSash,  SWT.VERTICAL | SWT.FLAT);
    	
    	Composite compositeAttributes = new Composite(sashAttributes, SWT.NONE);
		final GridLayout layoutcompositeAttributes = new GridLayout();
		layoutcompositeAttributes.marginWidth = 15;
		layoutcompositeAttributes.marginHeight = 10;
		layoutcompositeAttributes.numColumns = 7;
		compositeAttributes.setLayout(layoutcompositeAttributes);
    	
    	CLabel configureAttributesLabel  = new CLabel(compositeAttributes, SWT.LEFT);
    	configureAttributesLabel.setText(" Attributes that identify iter ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	configureAttributesLabel.setLayoutData(data);
    	
    	btnChooseAttribute = new Button(compositeAttributes, SWT.PUSH);
    	//btnChooseAttribute.setText(" Select Attribute ");
    	btnChooseAttribute.setImage(addImage);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 1;
    	btnChooseAttribute.setLayoutData(data);
    	
    	
    	btnChooseAttribute.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {					
					ListObservaleAttributesDialog aListAttributesDialog = new ListObservaleAttributesDialog(getShell(), "Add attribute To Iter ", candidateAttributes);
					aListAttributesDialog.open();
				
					ObservableAttributeTreeElement aAttribute = aListAttributesDialog.getSelectedAttribute();
					boolean alreadyFound = false;
					for (ObservableAttributeTreeElement att : selectedAttributesForIter){
						if (att.getId().equalsIgnoreCase(aAttribute.getId())){
							MessageDialog.openInformation(getShell(), "Spagic", "Attribute already present");
							alreadyFound = true;
							break;
						}
					}
					if ((aAttribute != null) && !alreadyFound){
						theDialog.iterStatus = TO_SAVE;
						theDialog.iterStatusLabelTextField.setText(TO_SAVE);
						selectedAttributesForIter.add(aAttribute);
						tbvAttributesForIterType.setInput(selectedAttributesForIter);
						tbvAttributesForIterType.refresh(true);
					}
				}

			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);
			}
    	});
    	
    	
    	
    	   
        
        this.tableAttributesForIterType = new Table(sashAttributes, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableAttributesForIterType.setLinesVisible(true);
        this.tableAttributesForIterType.setHeaderVisible(true);
        
 		TableColumn column = new  TableColumn(tableAttributesForIterType, SWT.LEFT);
 		column.setText(" Id ");
 		column.setWidth(50);
 		
 		column = new  TableColumn(tableAttributesForIterType, SWT.LEFT);
 		column.setText(" Attribute ");
 		column.setWidth(400);
 		
        this.tbvAttributesForIterType = new TableViewer(tableAttributesForIterType);
        tbvAttributesForIterType.setContentProvider(new ObservableAttributeContentProvider());
        tbvAttributesForIterType.setLabelProvider(new ObservableAttributeLabelProvider());
           
    	
    	tbvAttributesForIterType.setInput(selectedAttributesForIter);
    	
        this.tableAttributesForIterType.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableAttributesForIterType.getSelection();
        			theDialog.iterStatus = TO_SAVE;
    				theDialog.iterStatusLabelTextField.setText(TO_SAVE);
        		    ObservableAttributeTreeElement selectedAttribute = (ObservableAttributeTreeElement)tis[0].getData();
        		    selectedAttributesForIter.remove(selectedAttribute);
        			theDialog.tbvAttributesForIterType.setInput( selectedAttributesForIter );
        			theDialog.tbvAttributesForIterType.refresh();
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        });


        
        this.sashRuleIterType = new SashForm(internalSash,  SWT.VERTICAL | SWT.FLAT);
        Composite ruleExpressionComposite = new Composite(sashRuleIterType, SWT.NONE);
        final GridLayout layoutRuleExprComposite = new GridLayout();
        layoutRuleExprComposite.marginWidth = 15;
        layoutRuleExprComposite.marginHeight = 10;
        layoutRuleExprComposite.numColumns = 10;
    	ruleExpressionComposite.setLayout(layoutRuleExprComposite);
    	
    	/*
    	// -- Rule Expression
    	ruleExpressionLabel = new CLabel(ruleExpressionComposite, SWT.CENTER);
    	ruleExpressionLabel.setText(" Rule for Iter Type Determination ( Boolean XPath Expression ) ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 10;
    	ruleExpressionLabel.setLayoutData(data);
    	
    	
    	ruleExpressionTextArea = new Text(ruleExpressionComposite, SWT.MULTI| SWT.WRAP | SWT.BORDER);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.heightHint = 50;
    	data.horizontalSpan = 10;
    	ruleExpressionTextArea.setLayoutData(data);
       
    	if (ruleForIterTypeDetermination != null){
    		ruleExpressionTextArea.setText(ruleForIterTypeDetermination);
    	}
    	       
    	this.ruleExpressionTextArea.addModifyListener(new ModifyListener(){

			public void modifyText(ModifyEvent modEvent) {
				theDialog.iterStatus = TO_SAVE;
				theDialog.iterStatusLabelTextField.setText(TO_SAVE);
			}
    	});
    	*/
    	
    	this.internalSash.setWeights(new int[]{40,50,5});
        return parent; 
	
	}
	
	
	protected void createButtonsForButtonBar(Composite parent) {
		
		// -- Create OK and Cancel buttons by default
		createButton(parent, IDialogConstants.OK_ID, "Save Iter Data",
				true);
		
		createButton(parent, IDialogConstants.CLOSE_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
	}
	@Override
	protected void okPressed() {
		if (iterStatus.equalsIgnoreCase(CONFIGURED)){
			MessageDialog.openInformation(getShell(), "Spagic", "No Data To Save");
			return;
		}
		if (selectedAttributesForIter.size() == 0){
			MessageDialog.openError(getShell(), "Spagic", "To Configure Iter we need at least on attribute");
			return;
		}
		if (iterFirstProcessCombo.getText() == null){
			MessageDialog.openError(getShell(), "Spagic", "To Configure Iter choose the first process");
			return;
		}
		if ((iterFirstProcessCombo != null) && (iterFirstProcessCombo.getText().trim().length() == 0)){
			MessageDialog.openError(getShell(), "Spagic", "To Configure Iter choose the first process");
			return;
		}
		if (iterStatus.equalsIgnoreCase(TO_SAVE)){
			EAnnotation ea = bpmnDiagram.getEAnnotation(ImConstants.IM_ITER_ANNOTATION);
			bpmnDiagram.getEAnnotations().remove(ea);
		}
		
		EAnnotation ea = EcoreFactory.eINSTANCE.createEAnnotation();
		ea.setSource(ImConstants.IM_ITER_ANNOTATION);
		Map<String, String> details = new HashMap<String, String>();

		details.put(ImConstants.IM_ITER_NAME, iterTypeNameTextField.getText());
		/*
		details.put(ImConstants.IM_ITER_RULE_FOR_DETERMINATION,
				ruleExpressionTextArea.getText());
		*/
		details.put(ImConstants.IM_ITER_RULE_FOR_DETERMINATION_LANGUAGE,
				"XPath");
		details.put(ImConstants.IM_ITER_FIRST_PROCESS_NAME,
				iterFirstProcessCombo.getText());
		details.put(ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_LIST_SIZE, String
				.valueOf(selectedAttributesForIter.size()));

		ObservableAttributeTreeElement obsAttribute = null;
		for (int i = 0; i < selectedAttributesForIter.size(); i++) {
			obsAttribute = (ObservableAttributeTreeElement) selectedAttributesForIter
					.get(i);
			details.put(MessageFormat.format(
					ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_ID, i),
					obsAttribute.getId()); // The rule to associate with the
											// endpoint
			details.put(MessageFormat.format(
					ImConstants.IM_ITER_OBSERVABLE_ATTRIBUTE_NAME, i),
					obsAttribute.getName()); // The relavant data name of the
												// database
		}
		ea.getDetails().putAll(details);
		bpmnDiagram.getEAnnotations().add(ea);
		this.iterStatus = CONFIGURED;
		this.iterStatusLabelTextField.setText(CONFIGURED);
	}
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.CLOSE_ID){
			if (iterStatus.equalsIgnoreCase(TO_SAVE)){
				if (MessageDialog.openConfirm(getShell(), "Spagic", "There are unsaved data, Do you want to exit")){
					close();
				}
			}else if (iterStatus.equalsIgnoreCase(NOT_CONFIGURED)){
				if (MessageDialog.openConfirm(getShell(), "Spagic", "No Iter Configured, Do you want to exit")){
					close();
				}
			}else if (iterStatus.equalsIgnoreCase(CONFIGURED)){
				close();
			}	
		}else{
			super.buttonPressed(buttonId);
		}
	}
	
	
}
