/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.Rule;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.hibernate.Session;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.ui.view.ExtractionRuleObservableAttribute;
import org.spagic.ui.view.ObservableAttributeTreeElement;
import org.spagic.utils.SpagicLogger;
import org.vafada.swtcalendar.SWTCalendarEvent;
import org.vafada.swtcalendar.SWTCalendarListener;

import demo.SWTCalendarDialog;

public class DetailExtractionRuleDialog extends SpagicTitleAreaDialog {

	
    
	
	protected final SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
	
	public static final Integer USAGE_EXTRACT_DATA = 0;
	public static final Integer USAGE_ITER_TYPE_DETERMINATION = 1;
	
	private Composite aComposite = null;
    private CLabel ruleDescriptionLabel = null;
    private Text ruleDescriptionTextField = null;
    
    private CLabel conditionLabel = null;
    private Text conditionTextArea = null;
    
    private CLabel ruleExpressionLabel = null;
    private Text ruleExpressionTextArea = null;

	
    private SashForm internalSash  = null; 

    
    private Group ruleRequireConditionGroup = null;
    private Button ruleSubjectToConditionYesButton = null;
    private Button ruleSubjectToConditionNoButton = null;
    
    private CLabel startDateLabel = null;
    private Text   startDateTextArea = null;
    
    private CLabel endDateLabel = null;
    private Text   endDateTextArea = null;
    
    private CLabel observableAttributeNameLabel = null;
    private Text observableAttributeNameTextField = null;
	
    public boolean status = false;
    public ObservableAttributeTreeElement aObservableAttributeTreeElement = null;
    public ExtractionRuleObservableAttribute extractionRule = null;
    public boolean modeNew = false;
	
    public DetailExtractionRuleDialog(Shell parentShell, ObservableAttributeTreeElement observableAttributeTreeElement) {
		super(parentShell);
		this.aObservableAttributeTreeElement = observableAttributeTreeElement;
		modeNew = true;
	}
    
	public DetailExtractionRuleDialog(Shell parentShell, ObservableAttributeTreeElement observableAttributeTreeElement, ExtractionRuleObservableAttribute extractionRule) {
		this(parentShell, observableAttributeTreeElement);
		this.extractionRule = extractionRule;
		modeNew = false;
	}
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
	    setTitle(modeNew ? "Insert New Rule For Extract Attribute "+ this.aObservableAttributeTreeElement.getName() : " Modify Rule " + extractionRule.getName() );
	    setMessage(modeNew ? " Fill fields " :  " Modify expressions and/or date fields of the rule");

	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       
		aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	aComposite.setLayout(layout);
    	
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	

    	observableAttributeNameLabel = new CLabel(aComposite, SWT.RIGHT);
    	observableAttributeNameLabel.setText(" Relevant Data ( Attribute ) Name");
    	observableAttributeNameTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );   	
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 300;
    	data.horizontalSpan = 9;
    	observableAttributeNameTextField.setText(this.aObservableAttributeTreeElement != null ? this.aObservableAttributeTreeElement.getName() : "");
    	observableAttributeNameTextField.setLayoutData(data);
    	observableAttributeNameTextField.setEditable(false);
    	
    	//    	 Riga 1
    	ruleDescriptionLabel = new CLabel(aComposite, SWT.RIGHT);
    	ruleDescriptionLabel.setText(" Rule Name ");
    	ruleDescriptionTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 300;
    	data.horizontalSpan = 9;
    	ruleDescriptionTextField.setLayoutData(data);
    	ruleDescriptionTextField.setEditable(modeNew);
    	ruleDescriptionTextField.setText(extractionRule != null ? extractionRule.getName() : "");

    	
    	//
    	// -- VALIDITY DATE
    	//
    	
    	startDateLabel = new CLabel(aComposite, SWT.RIGHT);
    	startDateLabel.setText(" Start Date ");
    	
    	startDateTextArea = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 5;
    	startDateTextArea.setEditable(false);
    	
    	// -- startDateTextArea.setText(rule != null ? rule.getStartDate().toString() : "");
    	startDateTextArea.setText(extractionRule != null  ? formatDate(extractionRule.getStartDate()) : "");
    	startDateTextArea.setLayoutData(data);
    	
    	// -- Gestione della data
    	final Button bChangeDateStartDate = new Button(aComposite, SWT.PUSH);
    	bChangeDateStartDate.setText("Change");
        final DetailExtractionRuleDialog theDialog = this;
        bChangeDateStartDate.addListener(SWT.Selection, new Listener() {

              public void handleEvent(Event event) {
            	  
            	  Point parentLocation = theDialog.getShell().getLocation();
            	  Point buttonLocation = bChangeDateStartDate.getLocation();
            	  
            	  Point dialogCalendarLocation  = new Point(parentLocation.x+ buttonLocation.x+100, parentLocation.y + buttonLocation.y+90);
            	  
                  final SWTCalendarDialog cal = new SWTCalendarDialog(Display.getCurrent(), dialogCalendarLocation);

            	  //final CalendarDialog cal = new CalendarDialog(getShell());
                  
            	  cal.addDateChangedListener(new SWTCalendarListener() {

                      public void dateChanged(SWTCalendarEvent calendarEvent) {

                    	  startDateTextArea.setText(formatter.format(calendarEvent.getCalendar().getTime()));
                         
                      }

                  });
                

                  if (startDateTextArea.getText() != null && startDateTextArea.getText().length() > 0) {

                      try {

                          Date d = formatter.parse(startDateTextArea.getText());

                          cal.setDate(d);

                      } catch (ParseException pe) {
                      }

                  }

                  cal.open();
              }

        });
        data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bChangeDateStartDate.setLayoutData(data);
    	
    	final Button bClearStartDate = new Button(aComposite, SWT.PUSH);
    	bClearStartDate.setText("Clear");
    	bClearStartDate.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				startDateTextArea.setText("");
			}

			public void widgetSelected(SelectionEvent e) {
				startDateTextArea.setText("");
				
			}});
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bClearStartDate.setLayoutData(data);
    	
    	endDateLabel = new CLabel(aComposite, SWT.RIGHT);
    	endDateLabel.setText(" End Date ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 5;
    	endDateTextArea = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	endDateTextArea.setLayoutData(data);
    	endDateTextArea.setEditable(false);
    	endDateTextArea.setText(extractionRule != null ? formatDate(extractionRule.getEndDate()) : "");
    	endDateTextArea.setLayoutData(data);
    	
    	// 
//   	 Gestione della data
    	final Button bChangeDateEndDate = new Button(aComposite, SWT.PUSH);
    	bChangeDateEndDate.setText("Change");
        
    	bChangeDateEndDate.addListener(SWT.Selection, new Listener() {

              public void handleEvent(Event event) {
            	  
            	  Point parentLocation = theDialog.getShell().getLocation();
            	  Point buttonLocation = bChangeDateEndDate.getLocation();
            	  
            	  Point dialogCalendarLocation  = new Point(parentLocation.x+ buttonLocation.x+100, parentLocation.y + buttonLocation.y+90);
            	  
                  final SWTCalendarDialog cal = new SWTCalendarDialog(Display.getCurrent(), dialogCalendarLocation);

            	  //final CalendarDialog cal = new CalendarDialog(getShell());
                  
            	  cal.addDateChangedListener(new SWTCalendarListener() {

                      public void dateChanged(SWTCalendarEvent calendarEvent) {

                    	  endDateTextArea.setText(formatter.format(calendarEvent.getCalendar().getTime()));
                         
                      }

                  });
                

                  if (endDateTextArea.getText() != null && endDateTextArea.getText().length() > 0) {

                      try {

                          Date d = formatter.parse(endDateTextArea.getText());

                          cal.setDate(d);

                      } catch (ParseException pe) {
                      }

                  }

                  cal.open();
              }

        });
        data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bChangeDateEndDate.setLayoutData(data);
    	
    	final Button bClearEndDate = new Button(aComposite, SWT.PUSH);
    	bClearEndDate.setText("Clear");
    	bClearEndDate.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				endDateTextArea.setText("");
			}

			public void widgetSelected(SelectionEvent e) {
				endDateTextArea.setText("");
				
			}});
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bClearEndDate.setLayoutData(data);
    	//
    	
    	
    
    	
    
    	ruleRequireConditionGroup = new Group(aComposite, SWT.SHADOW_ETCHED_IN);
    	ruleRequireConditionGroup.setText(" Precondition For Relevant Data Extraction "); 
    	ruleSubjectToConditionYesButton = new Button(ruleRequireConditionGroup, SWT.RADIO);
    	ruleSubjectToConditionYesButton.setText("Yes");
    	ruleSubjectToConditionYesButton.setBounds(5, 10, 70, 30);
    	ruleSubjectToConditionNoButton = new Button(ruleRequireConditionGroup, SWT.RADIO);
    	ruleSubjectToConditionNoButton.setText("No");
    	ruleSubjectToConditionNoButton.setBounds(100, 10, 70, 30);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 10;
    	data.heightHint = 30;
    	ruleRequireConditionGroup.setLayoutData(data);
    	
    	ruleSubjectToConditionYesButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				 conditionTextArea.setEditable(true);
				
			}

			public void widgetSelected(SelectionEvent e) {
				conditionTextArea.setEditable(true);
				
			}
    		
    	});
    	
    	ruleSubjectToConditionNoButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				conditionTextArea.setText(""); 
				conditionTextArea.setEditable(false);
				
			}

			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);
				
			}
    		
    	});
    	
    	//
    	// -- CONDITION COMPOSITE
    	//
    	Composite conditionComposite = new Composite(internalSash, SWT.NONE);
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	conditionComposite.setLayout(layout);
    	
    	
    	conditionLabel = new CLabel(conditionComposite, SWT.CENTER);
    	conditionLabel.setText(" Rule Precondition (XPath) ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 10;
    	conditionLabel.setLayoutData(data);
    	
    	conditionTextArea = new Text(conditionComposite, SWT.MULTI| SWT.WRAP | SWT.BORDER);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.heightHint = 50;
    	data.horizontalSpan = 10;
    	conditionTextArea.setLayoutData(data);

    	
    	Composite ruleExpressionComposite = new Composite(internalSash, SWT.NONE);
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	ruleExpressionComposite.setLayout(layout);
    	//  	 Rule Expression
    	ruleExpressionLabel = new CLabel(ruleExpressionComposite, SWT.CENTER);
    	ruleExpressionLabel.setText(" Rule Expression (XPath) ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 10;
    	ruleExpressionLabel.setLayoutData(data);
    	
    	
    	ruleExpressionTextArea = new Text(ruleExpressionComposite, SWT.MULTI| SWT.WRAP | SWT.BORDER);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.heightHint = 50;
    	data.horizontalSpan = 10;
    	ruleExpressionTextArea.setLayoutData(data);
    	ruleExpressionTextArea.setText(extractionRule != null ? extractionRule.getExpression(): "");
    	if ((extractionRule != null) && (extractionRule.isSubjectToCondition())){
    		ruleSubjectToConditionYesButton.setSelection(true);
    		conditionTextArea.setEditable(true);
    		conditionTextArea.setText(extractionRule.getConditionExpression());
    	}else{
    		ruleSubjectToConditionNoButton.setSelection(true);
    		conditionTextArea.setEditable(false);
    	}
    	
    	
    	
    	// Weight Settings on Sash
    	internalSash.setWeights(new int[] {30, 20, 25});
        return parent; 
	
	}
	
	protected void okPressed() {
		if (modeNew){
			createNewRuleForRelevantDataAttribute();
		}else{
			modifyRuleForForRelevantDataAttribute(extractionRule);
		}
	}
	
	public void modifyRuleForForRelevantDataAttribute(ExtractionRuleObservableAttribute aRuleElement){
		String ruleDescriptionText = ruleDescriptionTextField.getText();
		
		if (ruleDescriptionText.trim().length() == 0){
			MessageDialog.openError(
					getShell(),
					"Spagic",
					"The rule description is required");
			return;
		}
		final String ruleExprText = ruleExpressionTextArea.getText().trim();
		if (ruleExprText.length() == 0){
			MessageDialog.openError(
					getShell(),
					"Spagic",
					"The rule expression is required");
			return;
		}
		final String conditionText = conditionTextArea.getText();
		if (ruleSubjectToConditionYesButton.getSelection() && conditionText.trim().length() == 0){
			MessageDialog.openError(
					getShell(),
					"Spagic",
					"If the rule require a precondition the relative field must be not empty");
			return;
		}
	
		final Integer usage  = USAGE_EXTRACT_DATA;
		
		try{
			MetaDBHelper.doExecuteTx(new IMetaDBCommand(){

				public Object execute(Session aSession) throws Throwable {
					Rule aRule = (Rule)AuditDBUtils.getObjectById(aSession, Rule.class, Long.valueOf(extractionRule.getId()));
					aRule.setRuleUsage(usage);
				
					if ((endDateTextArea.getText()) != null && (endDateTextArea.getText().trim().length() > 0)){
						aRule.setEndDate(formatter.parse(endDateTextArea.getText()));
					}
				
					if ((startDateTextArea.getText()) != null && (startDateTextArea.getText().trim().length() > 0)){
						aRule.setStartDate(formatter.parse(startDateTextArea.getText()));
					}
				
					if (ruleSubjectToConditionYesButton.getSelection())
						aRule.setConditionExpr(conditionText);
					else
						aRule.setConditionExpr(null);
				
					aRule.setExpr(ruleExprText);
					AuditDBUtils.storeRule(aSession, aRule);
					return true;
				}
			});
			close();
			this.status = true;
		}catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");
			this.status = false;
		}		
	}
	public void createNewRuleForRelevantDataAttribute(){
			final String ruleName = ruleDescriptionTextField.getText().trim();
			
			if (ruleName.trim().length() == 0){
				MessageDialog.openError(
						getShell(),
						"Spagic",
						"The rule description is required");
				return;
			}
			final String ruleExprText = ruleExpressionTextArea.getText().trim();
			if (ruleExprText.length() == 0){
				MessageDialog.openError(
						getShell(),
						"Spagic",
						"The rule expression is required");
				return;
			}
			final String conditionText = conditionTextArea.getText();
			if (ruleSubjectToConditionYesButton.getSelection() && conditionText.trim().length() == 0){
				MessageDialog.openError(
						getShell(),
						"Spagic",
						"If the rule require a precondition the relative field must be not empty");
				return;
			}
			
			
			
			final Integer usage  = USAGE_EXTRACT_DATA;
			
			try{
				this.status = (Boolean) MetaDBHelper.doExecuteTx(new IMetaDBCommand(){
					public Object execute(Session aSession) throws Throwable {
				
						Attribute anAttribute = (Attribute)AuditDBUtils.getObjectById(aSession, Attribute.class, Long.valueOf(aObservableAttributeTreeElement.getId()));
				
			
				anAttribute = AuditDBUtils.storeAttribute(aSession, anAttribute);
				if (AuditDBUtils.getRuleByName(aSession, ruleName) != null){
					return false;
				}
				Rule aRule = new Rule();
				aRule.setName(ruleName);
				aRule.setRuleUsage(usage);
				
				if ((endDateTextArea.getText()) != null && (endDateTextArea.getText().trim().length() > 0)){
					aRule.setEndDate(formatter.parse(endDateTextArea.getText()));
				}
				
				if ((startDateTextArea.getText()) != null && (startDateTextArea.getText().trim().length() > 0)){
					aRule.setStartDate(formatter.parse(startDateTextArea.getText()));
				}
				
				if (ruleSubjectToConditionYesButton.getSelection())
					aRule.setConditionExpr(conditionText);
				else
					aRule.setConditionExpr(null);
				
				
				aRule.setAttribute(anAttribute);
				aRule.setExpr(ruleExprText);
				AuditDBUtils.storeRule(aSession, aRule);
				return true;

				}
			});
			if (this.status){
				close();
			} else {
				MessageDialog.openWarning(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(), "Spagic",
						"An Rule with this name already exist !!!");
			}
			
		
		}catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");
			this.status = false;
		}		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.CANCEL_ID){
			close();
		}
		super.buttonPressed(buttonId);
	}


	public boolean isStatus() {
		return status;
	}


	public void setStatus(boolean status) {
		this.status = status;
	}

	private String formatDate(Date d){
		String dateString = "";
		if (d != null){
			dateString =  formatter.format(d);
		}
		return dateString;
	}

	public boolean isModeNew() {
		return modeNew;
	}

	public void setModeNew(boolean modeNew) {
		this.modeNew = modeNew;
	}

	public ExtractionRuleObservableAttribute getExtractionRule() {
		return extractionRule;
	}

	public void setExtractionRule(ExtractionRuleObservableAttribute extractionRule) {
		this.extractionRule = extractionRule;
	}
	
}
