/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.Rule;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.hibernate.Session;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.ui.view.ExtractionRuleObservableAttribute;
import org.spagic.ui.view.ObservableAttributeTreeElement;
import org.spagic.utils.SpagicLogger;

public class ListExtractionRulesDialog extends SpagicTitleAreaDialog {

	private Composite aComposite = null;

	private SashForm internalSash = null;

	private Table tableRules = null;
	private TableViewer tbvRules = null;
	private List<ExtractionRuleObservableAttribute> theRules = null;

	private ExtractionRuleObservableAttribute selectedRule = null;

	private boolean closeOnDoubleClick = false;

	private String message = null;
	private ObservableAttributeTreeElement attribute = null;

	public static final int CREATE_NEW_RULE_BUTTON_ID = 99003;

	// closeOnDoubleClick = True --> Means we use this dialog to select a rule
	// closeOnDoubleClick = False -->

	public ListExtractionRulesDialog(Shell parentShell,
			ObservableAttributeTreeElement observableAttributeTreeElement,
			boolean closeOnDoubleClick, String message) {

		super(parentShell);
		this.theRules = getRulesFromDatabase(observableAttributeTreeElement);
		this.closeOnDoubleClick = closeOnDoubleClick;
		this.attribute = observableAttributeTreeElement;
		this.message = message;
	}

	/**
	 * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	 *      a title and a message
	 */
	public void create() {
		super.create();
		setTitle("Extraction rules for attribute " + attribute.getName());
		setMessage(message);

	}

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {

		this.internalSash = new SashForm(parent, SWT.VERTICAL | SWT.FLAT);
		this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));

		final ListExtractionRulesDialog theDialog = this;

		this.theRules = getRules();

		this.tableRules = new Table(internalSash, SWT.MULTI
				| SWT.FULL_SELECTION | SWT.BORDER);
		this.tableRules.setLinesVisible(true);
		this.tableRules.setHeaderVisible(true);
		TableColumn column = null;
		column = new TableColumn(tableRules, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);

		column = new TableColumn(tableRules, SWT.LEFT);
		column.setText("Expression");
		column.setWidth(300);

		column = new TableColumn(tableRules, SWT.LEFT);
		column.setText("Under Condition");
		column.setWidth(70);

		column = new TableColumn(tableRules, SWT.LEFT);
		column.setText("Condition Expression");
		column.setWidth(300);

		this.tableRules.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		this.tbvRules = new TableViewer(tableRules);
		tbvRules.setContentProvider(new ExtractionRuleContentProvider());
		tbvRules.setLabelProvider(new ExtractionRuleLabelProvider());

		tbvRules.setInput(theRules);

		tbvRules.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {

				if (closeOnDoubleClick) {
					TableItem[] selection = tableRules.getSelection();
					if ((selection != null) && (selection.length > 0)) {
						selectedRule = (ExtractionRuleObservableAttribute) ((TableItem) selection[0])
								.getData();
						close();
					} else {
						MessageDialog.openConfirm(getShell(), "Spagic",
								"Select a rule ");
					}
				} else {
					TableItem[] selection = tableRules.getSelection();
					if ((selection != null) && (selection.length > 0)) {
						selectedRule = (ExtractionRuleObservableAttribute) ((TableItem) selection[0])
								.getData();
						DetailExtractionRuleDialog detailExtractionRuleDialog = new DetailExtractionRuleDialog(
								getShell(), attribute, selectedRule);
						detailExtractionRuleDialog.open();
						theDialog.tbvRules
								.setInput(getRulesFromDatabase(theDialog.attribute));
						theDialog.tbvRules.refresh();
					}
				}
			}
		});

		if (!closeOnDoubleClick) {
			// AGGIUNGO LISTENER PER CANCELLARE LE CODE
			this.tableRules.addKeyListener(new KeyListener() {
				public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
					System.out.println("KeyCode =" + e.keyCode);
					if (e.keyCode == 127) {
						TableItem[] tis = theDialog.tableRules.getSelection();
						ExtractionRuleObservableAttribute selectedRule = (ExtractionRuleObservableAttribute) tis[0]
								.getData();

						boolean result = removeRuleFromDatabase(selectedRule);
						if (result) {
							theDialog.theRules.remove(selectedRule);
							theDialog.tbvRules
									.setInput(getRulesFromDatabase(attribute));
							theDialog.tbvRules.refresh();
						}
					}
				};

				public void keyReleased(org.eclipse.swt.events.KeyEvent e) {

				};

			});

		}

		return parent;

	}

	protected void okPressed() {
		super.okPressed();

	}

	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, CREATE_NEW_RULE_BUTTON_ID, "New Rule", true);
		createButton(parent, IDialogConstants.OK_ID,
				IDialogConstants.CLOSE_LABEL, true);

	}

	public List<ExtractionRuleObservableAttribute> getRules() {
		return this.theRules;
	}

	public ExtractionRuleObservableAttribute getSelectedRule() {
		return selectedRule;
	}

	public void setSelectedRule(ExtractionRuleObservableAttribute selectedRule) {
		this.selectedRule = selectedRule;
	}

	public boolean removeRuleFromDatabase(
			final ExtractionRuleObservableAttribute selectedRule) {
		try {
			MetaDBHelper.doExecuteTx(new IMetaDBCommand() {
				public Object execute(Session aSession) throws Throwable {

					Rule aRule = (Rule) AuditDBUtils.getObjectById(aSession,
							Rule.class, Long.valueOf(selectedRule.getId()));

					aSession.delete(aRule);
					aSession.flush();
					return true;

				}
			});
			return true;

		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			return false;
		}
	}

	public List<ExtractionRuleObservableAttribute> getRulesFromDatabase(
			final ObservableAttributeTreeElement observableAttributeTreeElement) {

		final String expressionLanguage = "XPath";

		try {
			List<ExtractionRuleObservableAttribute> result = null;

			result = (List<ExtractionRuleObservableAttribute>) MetaDBHelper
					.doExecuteNoTx(new IMetaDBCommand() {
						public Object execute(Session aSession)
								throws Throwable {

							List<ExtractionRuleObservableAttribute> rulesForObservableAttribute = new ArrayList<ExtractionRuleObservableAttribute>();
							Attribute anAttribute = (Attribute) AuditDBUtils
									.getObjectById(
											aSession,
											Attribute.class,
											Long
													.valueOf(observableAttributeTreeElement
															.getId()));
							ExtractionRuleObservableAttribute aRuleElement = null;
							for (Rule ruleForAttribute : anAttribute.getRules()) {

								aRuleElement = new ExtractionRuleObservableAttribute(
										String.valueOf(ruleForAttribute
												.getIdRule()), ruleForAttribute
												.getName(),
										ruleForAttribute.getExpr(),
										expressionLanguage);

								if (ruleForAttribute.getConditionExpr() != null) {
									aRuleElement.setSubjectToCondition(true);
									aRuleElement
											.setConditionExpression(ruleForAttribute
													.getConditionExpr());
									aRuleElement
											.setConditionLanguage(expressionLanguage);
								}

								aRuleElement.setStartDate(ruleForAttribute
										.getStartDate());
								aRuleElement.setEndDate(ruleForAttribute
										.getEndDate());
								rulesForObservableAttribute.add(aRuleElement);

							}
							return rulesForObservableAttribute;

						}
					});
			return result;
		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			return new ArrayList<ExtractionRuleObservableAttribute>();
		}
	}

	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == CREATE_NEW_RULE_BUTTON_ID) {
			DetailExtractionRuleDialog aDetailExtractionRuleDialog = new DetailExtractionRuleDialog(
					getShell(), this.attribute);
			aDetailExtractionRuleDialog.open();
			if (aDetailExtractionRuleDialog.isStatus()) {
				this.tbvRules.setInput(getRulesFromDatabase(attribute));
				this.tbvRules.refresh();
			}
		} else {
			super.buttonPressed(buttonId);
		}
	}

}
