/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.spagic.ui.view.ObservableAttributeTreeElement;

public class ListObservaleAttributesDialog extends SpagicTitleAreaDialog {
	
    private SashForm internalSash  = null; 
    
    private Table tableAttribute = null;
    private TableViewer tbvAttribute = null;
    private List<ObservableAttributeTreeElement> listAttribute = null;

   

  
    public static final int SELECT_ATTRIBUTE_ID = 99003;
    
    private ObservableAttributeTreeElement selectedAttribute = null;
    private String message = null;
    
	
	
	public ListObservaleAttributesDialog(Shell parentShell, String message, List<ObservableAttributeTreeElement> providedAttributeList ) {
		super(parentShell);
		this.message = message;
		this.listAttribute = providedAttributeList;
	}
	
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Candidate Attributes For Iter");
	    if (message != null)
	    	setMessage(message);
	    else
	    	setMessage("Candidate Attributes For Iter");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
        this.tableAttribute = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableAttribute.setLinesVisible(true);
        this.tableAttribute.setHeaderVisible(true);
        TableColumn column = null;
		column = new TableColumn(tableAttribute, SWT.LEFT);
		column.setText("Id");
		column.setWidth(50);
		column = new TableColumn(tableAttribute, SWT.LEFT);
		column.setText("Name");
		column.setWidth(450);
        this.tbvAttribute = new TableViewer(tableAttribute);
        tbvAttribute.setContentProvider(new ObservableAttributeContentProvider());
        tbvAttribute.setLabelProvider(new ObservableAttributeLabelProvider());
     
      
        
        tbvAttribute.setInput(listAttribute);    
        
        final ListObservaleAttributesDialog  theDialog = this;
        
        
        this.tableAttribute.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		
        		
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        this.tbvAttribute.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				TableItem[] selections = theDialog.tableAttribute.getSelection();
				if (selections.length > 0){
					theDialog.selectedAttribute = (ObservableAttributeTreeElement)selections[0].getData();
					setMessage("Selected Attribute " + theDialog.selectedAttribute.getName());
					
				}
			}
        	
        });
        
        this.tbvAttribute.addDoubleClickListener(new IDoubleClickListener(){
        	
			public void doubleClick(DoubleClickEvent event) {
				TableItem[] selection = tableAttribute.getSelection();
				if ((selection != null) && (selection.length > 0)){
	    			selectedAttribute = (ObservableAttributeTreeElement)((TableItem)selection[0]).getData();
	    			close();
	    		}else{
	    			MessageDialog.openInformation(getShell(),"Spagic", "Select an Attribute to associate to catalog");
	    		}
				
			}
	
        });
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.CANCEL_ID)
			this.selectedAttribute = null;
		super.buttonPressed(buttonId);
		
	}


	protected void createButtonsForButtonBar(Composite parent) {
		
		
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
		
	}

	
	
	




	public ObservableAttributeTreeElement getSelectedAttribute() {
		return selectedAttribute;
	}




	public void setSelectedAttribute(ObservableAttributeTreeElement selectedAttribute) {
		this.selectedAttribute = selectedAttribute;
	}
	
}
