/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.XmlNamespace;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;
import org.hibernate.Session;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.SpagicLogger;

public class ListXmlNamespacesDialog extends SpagicTitleAreaDialog {
	
    private Composite aComposite = null;
	
    private SashForm internalSash  = null; 
    
    private Table tableNs = null;
    private TableViewer tbvNs = null;
    private List<XmlNamespace> listNs = null;
   


    public static final int NEW_NS_ID = 99002;
    
    
    private XmlNamespace selectedXmlNamespace = null;
    
	public ListXmlNamespacesDialog(Shell parentShell) {
		super(parentShell);
		this.listNs = new ArrayList<XmlNamespace>();
	}
	
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Namespace Repository");
	    setMessage("Namespace Repository");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
        this.tableNs = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableNs.setLinesVisible(true);
        this.tableNs.setHeaderVisible(true);
        TableColumn column = null;
		column = new TableColumn(tableNs, SWT.LEFT);
		column.setText("Prefix");
		column.setWidth(200);
		column = new TableColumn(tableNs, SWT.LEFT);
		column.setText("Value");
		column.setWidth(400);
		
        this.tbvNs = new TableViewer(tableNs);
        tbvNs.setContentProvider(new XmlNamespaceContentProvider());
        tbvNs.setLabelProvider(new XmlNamespaceLabelProvider());
        listNs = getNamespaces();
        tbvNs.setInput(listNs);    
        
        final ListXmlNamespacesDialog  theDialog = this;
        
        this.tableNs.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableNs.getSelection();
        			
        			if (tis.length > 0){
        				if (deleteNamespace((XmlNamespace)tis[0].getData())){
        					theDialog.tbvNs.setInput(getNamespaces());
        					theDialog.tbvNs.refresh(true);
        					theDialog.tbvNs.refresh();
        				}else{
        					MessageDialog.openError(
        		        			getShell(),
        		        			"Spagic",
        		        			"Cannot delete attribute");
        				}
        			}

        			
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        this.tbvNs.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				TableItem[] selections = theDialog.tableNs.getSelection();
				if (selections.length > 0){
					theDialog.selectedXmlNamespace = (XmlNamespace)selections[0].getData();
					setMessage("Selected Namespace " + theDialog.selectedXmlNamespace.getPrefix());
					
				}
			}
        	
        });
        
        this.tbvNs.addDoubleClickListener(new IDoubleClickListener(){
        	
			public void doubleClick(DoubleClickEvent event) {
				TableItem[] selection = tableNs.getSelection();
				if ((selection != null) && (selection.length > 0)){
	    			selectedXmlNamespace = (XmlNamespace)((TableItem)selection[0]).getData();
	    			setMessage("Selected Namespace " + theDialog.selectedXmlNamespace.getPrefix());
	    		}else{
	    			MessageDialog.openConfirm(
							getShell(),
							"Spagic",
							"Select a Namespace ");
	    		}
				
			}
	
        });
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == NEW_NS_ID){
			NewXMLNamespaceDialog aNewXMLNamespaceDialog = new NewXMLNamespaceDialog(getShell());
			aNewXMLNamespaceDialog.open();
			this.tbvNs.setInput(getNamespaces());
			this.tbvNs.refresh(true);
			this.internalSash.layout();
			
		}
		super.buttonPressed(buttonId);
	}


	protected void createButtonsForButtonBar(Composite parent) {
		
		createButton(parent, NEW_NS_ID, "New Namespace",
				true);
		
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
		
	}

	public boolean deleteNamespace(final XmlNamespace namespace) {
		
		try{
			MetaDBHelper.doExecuteTx(new IMetaDBCommand(){

				public Object execute(Session aSession) throws Throwable {
				
					AuditDBUtils.deleteXmlNamespace(aSession, namespace);
					return true;
				}		
			});
		
			return true;
		} catch (Exception e) {
			
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");
			return false;
		}
		
	}
	
	public List<XmlNamespace> getNamespaces(){
		try{
			List<XmlNamespace> namespaces = (List<XmlNamespace> )MetaDBHelper.doExecuteNoTx(new IMetaDBCommand(){

				public Object execute(Session aSession) throws Throwable {
			
					return AuditDBUtils.getXmlNamespaces(aSession);
				}
			});
			
			return namespaces;
		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");

			return new ArrayList<XmlNamespace>();
		}
			
			
	}




	



	
	
}
