/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.XmlNamespace;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.hibernate.Session;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.SpagicLogger;

public class NewXMLNamespaceDialog extends SpagicTitleAreaDialog {

	// final SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");

	private Composite aComposite = null;
	private CLabel namespacePrefixLabel = null;
	private Text namespacePrefixText = null;

	private CLabel namespaceValueLabel = null;
	private Text namespaceValueTextField = null;

	public NewXMLNamespaceDialog(Shell parentShell) {
		super(parentShell);
	}

	/**
	 * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	 *      a title and a message
	 */
	public void create() {
		super.create();

		setTitle("New Namespace");
		setMessage("New Namespace ");

	}

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {

		aComposite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout();
		layout.marginWidth = 15;
		layout.marginHeight = 10;
		layout.numColumns = 5;
		aComposite.setLayout(layout);
		aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;

		// Riga 2
		namespacePrefixLabel = new CLabel(aComposite, SWT.RIGHT);
		namespacePrefixLabel.setText(" Prefix ");
		namespacePrefixText = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		namespacePrefixText.setLayoutData(data);

		// Riga 2
		namespaceValueLabel = new CLabel(aComposite, SWT.RIGHT);
		namespaceValueLabel.setText(" Value ");
		namespaceValueTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		namespaceValueTextField.setLayoutData(data);

		return parent;

	}

	protected void okPressed() {

		
		try {

			if (namespacePrefixText.getText().trim().length() > 0
					&& namespaceValueTextField.getText().trim().length() > 0) {

				MetaDBHelper.doExecuteTx(new IMetaDBCommand() {

					public Object execute(Session aSession) throws Throwable {

						XmlNamespace xmlNamespace = new XmlNamespace();

						xmlNamespace.setPrefix(namespacePrefixText.getText());
						xmlNamespace
								.setValue(namespaceValueTextField.getText());
						AuditDBUtils.storeXmlNamespace(aSession, xmlNamespace);
						return true;
					}
				});

				close();
			} else {
				MessageDialog
						.openError(getShell(), "Spagic",
								"To add a namespace you must set the prefix and the value");
			}
		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");

		}
	}

	@Override
	protected void buttonPressed(int buttonId) {

		super.buttonPressed(buttonId);
	}

}
