/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dnd.handler;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.stp.bpmn.DataObject;
import org.eclipse.stp.bpmn.NamedBpmnObject;
import org.eclipse.stp.bpmn.dnd.AbstractEAnnotationDnDHandler;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.dialog.ListExtractionRulesDialog;
import org.spagic.ui.view.ObservableAttributeTreeElement;
import org.spagic.ui.view.ExtractionRuleObservableAttribute;



public class ObservableAttributeDndHandler extends AbstractEAnnotationDnDHandler {

	private ObservableAttributeTreeElement observableAttributeTreeElement = null;
	
	
	
	public ObservableAttributeDndHandler(ObservableAttributeTreeElement observableAttributeTreeElement) {
		
		this.observableAttributeTreeElement = observableAttributeTreeElement;
		
	}
	
	/**
	 * this method returns a command that will create an annotation
	 * on the semantic element of the edit part passed in parameter.
	 */
	public Command getDropCommand(IGraphicalEditPart hoverPart, int index, 
			Point dropLocation) {
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		EModelElement modelElement = (EModelElement) hoverPart.resolveSemanticElement();
		
		// Check if there's alredy a relevant data with id = id of relevantdata just dropped
		if (modelElement.getEAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE) != null){
			EMap<String, String> details = modelElement.getEAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE).getDetails();
			Integer oldsize = Integer.valueOf(details.get(ImConstants.IM_OBSERVABLE_ATTRIBUTE_LIST_SIZE));
			String reldataid = null;
			String reldataName = null;
			for ( int i = 1; i <= oldsize; i++){
				reldataid = details.get(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_ID, i));
				reldataName = details.get(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_NAME, i));
				if (reldataid.equalsIgnoreCase(this.observableAttributeTreeElement.getId())){
					MessageDialog.openError(shell, "Spagic", " This Set of Relevant Data contains alredy a relevant data with the same id called "+ reldataName);
					CompoundCommand co = new CompoundCommand("");
					return co;
				}
			}
		}
		
		ExtractionRuleObservableAttribute choosenRule = null;
		if ( this.observableAttributeTreeElement.getRules().size() > 1 ){
			
			MessageDialog.openInformation(shell, "Spagic", "This relevant data could be produced by more than one rule we must choose one");
			ListExtractionRulesDialog rulesDialog = new ListExtractionRulesDialog(shell, this.observableAttributeTreeElement, true, "Double Click to Choose Rule to extract " + this.observableAttributeTreeElement.getName());
			rulesDialog.open();
			choosenRule = rulesDialog.getSelectedRule();
			if (choosenRule == null){
				MessageDialog.openError(shell, "Spagic", "Cannot add relevant data a rule must be choosed ");
				CompoundCommand co = new CompoundCommand("");
				return co;
			}
		}else{
			choosenRule = (ExtractionRuleObservableAttribute)observableAttributeTreeElement.getRules().values().iterator().next();
		}
		
		
		
		if (modelElement.getEAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE) != null){
			EMap<String, String> details = modelElement.getEAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE).getDetails();
			
			// 1 - Update the imrelevantdatasizeattribute
			Integer oldsize = Integer.valueOf(details.get(ImConstants.IM_OBSERVABLE_ATTRIBUTE_LIST_SIZE));
			String  newSize = String.valueOf(oldsize + 1);
			
			details.put(ImConstants.IM_OBSERVABLE_ATTRIBUTE_LIST_SIZE, newSize);
			// 
			
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_ID, newSize),observableAttributeTreeElement.getId()); // The relevant data id in the database
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_NAME, newSize), observableAttributeTreeElement.getName()); // The relavant data name of the database
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_ID, newSize),choosenRule.getId()); // The rule to associate with the endpoint
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_NAME, newSize),choosenRule.getName());
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_EXPRESSION, newSize),choosenRule.getExpression()); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_LANGUAGE,newSize),choosenRule.getExpressionLanguage());
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_SUBJECT_TO_CONDITION,newSize),String.valueOf(choosenRule.isSubjectToCondition())); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_CONDITION_EXPRESSION,newSize),choosenRule.getConditionExpression() != null ? choosenRule.getConditionExpression() : "" ); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_CONDITION_EXPRESSION_LANGUAGE,newSize),choosenRule.getConditionLanguage() != null ? choosenRule.getConditionLanguage() : "" );
			
			CompoundCommand co = new CompoundCommand("");
			return co;
		}else{
			Map<String, String> details = new HashMap<String, String>();
			String  newSize = String.valueOf(1);
			
			details.put(ImConstants.IM_OBSERVABLE_ATTRIBUTE_LIST_SIZE, newSize);
			// 
			
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_ID, newSize),observableAttributeTreeElement.getId()); // The relevant data id in the database
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_NAME, newSize), observableAttributeTreeElement.getName()); // The relavant data name of the database
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_ID, newSize),choosenRule.getId()); // The rule to associate with the endpoint
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_NAME, newSize),choosenRule.getName());
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_EXPRESSION, newSize),choosenRule.getExpression()); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_LANGUAGE,newSize),choosenRule.getExpressionLanguage());
			
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_RULE_SUBJECT_TO_CONDITION,newSize),String.valueOf(choosenRule.isSubjectToCondition())); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_CONDITION_EXPRESSION,newSize),choosenRule.getConditionExpression() != null ? choosenRule.getConditionExpression() : "" ); // The rule expression
			details.put(MessageFormat.format(ImConstants.IM_OBSERVABLE_ATTRIBUTE_CONDITION_EXPRESSION_LANGUAGE,newSize),choosenRule.getConditionLanguage() != null ? choosenRule.getConditionLanguage() : "" );
			
			return createEAnnotationDropCommand(
					createAnnotation(ImConstants.IM_OBSERVABLE_ATTRIBUTE, details),
					(EModelElement) hoverPart.resolveSemanticElement());
		
		}

	}

	/**
	 * the file DnD only enables the drop of one element.
	 */
	public int getItemCount() {
		return 1;
	}

	public Image getMenuItemImage(IGraphicalEditPart hoverPart, int index) {
		return SpagicUIActivator.getImageDescriptor("icons/ico_relevant_data.png").createImage();
		
	}

	public String getMenuItemLabel(IGraphicalEditPart hoverPart, int index) {
		EObject element = hoverPart.resolveSemanticElement();
		String suffix = " on " + getShapeLabel((NamedBpmnObject) element);
        return "Configure Relevant Data on "+ suffix;
	}

	/**
	 * the file drop should have the lowest priority.
	 */
	public int getPriority() {
		return 0;
	}

	
	/**
     * @param namedElement
     * @return a label for a bpmn shape
     */
    protected String getShapeLabel(NamedBpmnObject namedElement) {
        String name = namedElement.getName() != null && 
            namedElement.getName().length() > 0 ? namedElement.getName() : "";
        if (name.length() > 18) {
            name = name.substring(0, 12) + "...";
        }
       if (namedElement instanceof DataObject) {
            String shape = "object";
            if (name.indexOf(shape) == -1) {
                return "data " + shape + " " + name;
            } else {
                return name;
            }
        }
        return name;
    }

    @Override
    public boolean isEnabled(IGraphicalEditPart hoverPart, int index) {
    	EModelElement emodelElement = (EModelElement) hoverPart.resolveSemanticElement();


    	// relevant data coul be attached only on data object
    	if (emodelElement instanceof DataObject){
			return true;
    	}else
    		return false;
    	}
    }
