/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.helpers;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Rule;
import org.spagic.metadb.model.XmlNamespace;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;

public class MetaDBHelper {

	public static Object doExecuteTx(IMetaDBCommand metaDBCommand){
		SessionFactory aSessionFactory = SpagicUIActivator.getDefault().getSessionFactory();
		Session aSession = null;
		Transaction tx = null;
		Object result = null;
		try{
			aSession = aSessionFactory.openSession();
			tx = aSession.beginTransaction();
			result = metaDBCommand.execute(aSession);
			tx.commit();
			return result;
		}catch (Throwable t) {
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			throw new RuntimeException(t);
		}finally{
			if (aSession != null && aSession.isOpen())
				aSession.close();
			tx = null;
			aSession = null;
		}
		
	}
	
	public static Object doExecuteNoTx(IMetaDBCommand metaDBCommand){
		SessionFactory aSessionFactory = SpagicUIActivator.getDefault().getSessionFactory();
		Session aSession = null;
		Object result = null;
		try{
			aSession = aSessionFactory.openSession();
			result = metaDBCommand.execute(aSession);
			return result;
		}catch (Throwable t) {
			throw new RuntimeException(t);
		}finally{
			if (aSession != null && aSession.isOpen())
				aSession.close();
			aSession = null;
		}
	}
	
	public static Map<String, String> getNameSpaceMap(){
		Map<String, String> nameSpaceMap = new HashMap<String, String>();
		try{
			List<XmlNamespace> namespaces = (List<XmlNamespace> )doExecuteNoTx(new IMetaDBCommand(){

				public Object execute(Session aSession) throws Throwable {
			
					return AuditDBUtils.getXmlNamespaces(aSession);
				}
			});
			
			for ( XmlNamespace aXmlNamespace : namespaces){
				nameSpaceMap.put(aXmlNamespace.getPrefix(), aXmlNamespace.getValue());
			}

		}catch (Throwable t) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, t.getMessage(), t);
		}
		return nameSpaceMap;
	}
	
	
}
