/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.helpers;

import org.spagic.metadb.model.Process;

import java.util.ArrayList;
import java.util.List;

import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;



public class RegistryHelper {
	
	public static IServiceRegistry getServiceRegistry(){
		return SpagicUIActivator.getDefault().getServiceRegistry();
	}
	
	public static List<String> getTaxonomies() {
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			List<ClassificationScheme> lTaxs= getServiceRegistry().getAllClassificationScheme();
			for (ClassificationScheme tax : lTaxs){
				l.add(tax.getName().getValue());
			}
		}catch(Exception e){
			
			
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error getting the list of taxonomies", e);
			
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
		
	}
	
	public static List<String> getTaxonomyValues(String taxonomyName){
		
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			l = sevRegistry.getClassificationSchemeValues(taxonomyName);
		}catch(Exception e){
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error getting the list of taxonomies", e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
	}
	
	public static List<String> getOrganizations() {
		
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			List<Organization> lOrgs=sevRegistry.getAllOrganizations();
			for (Organization org : lOrgs){
				l.add(org.getName().getValue());
			}
		}catch(Exception e){
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error getting the list of taxonomies", e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
	
	}
	
	public static List<String> getClassificationForService(Process process) {
		List<String> l = new ArrayList<String>();
		IServiceRegistry sevRegistry = null;
		try{
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			Organization org = sevRegistry.getOrganizationByName(process.getUddiOrg());
			Service service = sevRegistry.getServiceByOrganizationAndServiceName(org, process.getUddiName());
		
			return sevRegistry.getAllClassificationsForService(org, service);
		}catch(Exception e){
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error getting the list of taxonomies", e);
		}finally{
			sevRegistry.finalizeRegistry();
		}
		return l;
	}
	
	public static void removeServiceFromServiceRegistry(Process process) {
		
		IServiceRegistry sevRegistry = null;
		Session aSession = null;
		Transaction tx = null;
		try{
			aSession = SpagicUIActivator.getDefault().getSessionFactory().openSession();
			tx = aSession.beginTransaction();
			sevRegistry = getServiceRegistry();
			sevRegistry = getServiceRegistry();
			sevRegistry.initRegistry();
			Organization org = sevRegistry.getOrganizationByName(process.getUddiOrg());
			Service service = sevRegistry.getServiceByOrganizationAndServiceName(org, process.getUddiName());
			process.setUddiName(null);
			process.setUddiOrg(null);
			sevRegistry.deleteService(org, service);
			tx.commit();
		}catch (Throwable t) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, "Error getting the list of taxonomies", t);
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			throw new RuntimeException(t);
		}finally{
			if ((aSession != null) && (aSession.isOpen())){
				aSession.close();
			}
			if (sevRegistry != null){
				sevRegistry.finalizeRegistry();
			}
		}
	}
}
