/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.resource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.stp.im.resources.AbstractAskFileBasedResourceImporter;
import org.eclipse.stp.im.resources.IImResource;
import org.eclipse.stp.im.resources.datasources.DataSourceImResource;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;

public class SpagicCfgFileResourceImporter extends AbstractAskFileBasedResourceImporter {

	protected Map<String, String> smxJndiNamespaces = null;
	
	public SpagicCfgFileResourceImporter(){
		initNamespacesJndi();
	}
	
	
	
	public List<IImResource> importResources(String resourceType) {
		if (resourceType.equalsIgnoreCase(IImResource.DATA_SOURCE_IM_RESOURCE_TYPE))
			return importDataSources();
		else
			return new ArrayList<IImResource>();
	}
	
	private void initNamespacesJndi(){
		
		this.smxJndiNamespaces = new HashMap<String, String>();
		this.smxJndiNamespaces.put("jencks","http://jencks.org/2.0"); 
		this.smxJndiNamespaces.put("amq","http://activemq.org/config/1.0");
		this.smxJndiNamespaces.put("amqra","http://activemq.org/ra/1.0");
		this.smxJndiNamespaces.put("util","http://www.springframework.org/schema/util");
		this.smxJndiNamespaces.put("soapenv","http://schemas.xmlsoap.org/soap/envelope/");
		this.smxJndiNamespaces.put("xsd","http://www.w3.org/2001/XMLSchema"); 
		this.smxJndiNamespaces.put("xsi","http://www.w3.org/2001/XMLSchema-instance");
	
	}
	public List<IImResource> importDataSources(){
			List<IImResource> listDsBean = new ArrayList<IImResource>();	
			try{
				SAXReader reader = new SAXReader(false);
				reader.setValidation(false);
				reader.setIncludeExternalDTDDeclarations(false);
				reader.setIncludeInternalDTDDeclarations(false);
				String loadDtdFeature =
					"http://apache.org/xml/features/nonvalidating/load-external-dtd";;
				reader.setFeature(loadDtdFeature, false);
				
				Document jndiDocument = reader.read(getFileName());
				
//				<bean id="mysql-ds" 
//	                class="org.apache.commons.dbcp.BasicDataSource" 
//	                destroy-method="close"> 
//	                <property name="driverClassName" value="com.mysql.jdbc.Driver"/> 
//	                <property name="url" value="jdbc:mysql://athos:3306/smx2"/> 
//	                <property name="username" value="smx2"/> 
//	                <property name="password" value="smx2"/> 
//	          </bean> 
				
				XPath xPathExpression = new DefaultXPath("/beans/util:map[@id='jndiEntries']/entry");
				
				xPathExpression.setNamespaceURIs(this.smxJndiNamespaces);
				List<Node> entryNodes = xPathExpression.selectNodes(jndiDocument);
				
				Node propNode = null;
				for (Node en : entryNodes){
					String nodeKey = en.valueOf("@key");
					//System.out.println("entry" + nodeKey);
					if (nodeKey.startsWith("java:comp/env/jdbc/")){
						//System.out.println("entry" + nodeKey +" IS A DS");
						String name = nodeKey.substring("java:comp/env/jdbc/".length());
//						<property name="driverClassName" value="com.mysql.jdbc.Driver"/> 
//		                  <property name="url" value="jdbc:mysql://athos:3306/smx2"/> 
//		                  <property name="username" value="smx2"/> 
//		                  <property name="password" value="smx2"/> 
						propNode = en.selectSingleNode("bean/property[@name='driverClassName']");
						String driverClassName = propNode.valueOf("@value");
						propNode = en.selectSingleNode("bean/property[@name='url']");
						String jdbcUrl = propNode.valueOf("@value");
						propNode = en.selectSingleNode("bean/property[@name='username']");
						String user = propNode.valueOf("@value");
						propNode = en.selectSingleNode("bean/property[@name='password']");
						String password = propNode.valueOf("@value");
						
						IImResource res = new DataSourceImResource(name);
		            	res.setProperty(DataSourceImResource.DATASOURCE_URL,jdbcUrl);
		            	res.setProperty(DataSourceImResource.DATASOURCE_DRIVER, driverClassName);
		            	res.setProperty(DataSourceImResource.DATASOURCE_USER,user);
		            	res.setProperty(DataSourceImResource.DATASOURCE_PASSWORD,password);
						
						
						listDsBean.add(res);
						
						
					}else{
						//System.out.println("entry" + nodeKey + " IS NOT A DS ");
					}
				}
				}catch (Exception e) {
					SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);

				}
				return listDsBean;
			}
}
