/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.resource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.stp.im.resources.IImResource;
import org.eclipse.stp.im.resources.IImResourceImporter;
import org.eclipse.stp.im.resources.datasources.DataSourceImResource;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;
import org.spagic.utils.SpagicUtilsActivator;
import org.spagic.utils.preference.SpagicPreferenceConstants;

public class SpagicOnLineResourceImporter implements IImResourceImporter {

	protected Map<String, String> smxJndiNamespaces = null;
	
	public SpagicOnLineResourceImporter(){
		initNamespacesJndi();
	}
	public List<IImResource> importResources(String resourceType) {
		if (resourceType.equalsIgnoreCase(IImResource.DATA_SOURCE_IM_RESOURCE_TYPE))
			return importDataSources();
		else
			return new ArrayList<IImResource>();
	}
	
	private void initNamespacesJndi(){
		
		this.smxJndiNamespaces = new HashMap<String, String>();
		this.smxJndiNamespaces.put("jencks","http://jencks.org/2.0"); 
		this.smxJndiNamespaces.put("amq","http://activemq.org/config/1.0");
		this.smxJndiNamespaces.put("amqra","http://activemq.org/ra/1.0");
		this.smxJndiNamespaces.put("util","http://www.springframework.org/schema/util");
		this.smxJndiNamespaces.put("soapenv","http://schemas.xmlsoap.org/soap/envelope/");
		this.smxJndiNamespaces.put("xsd","http://www.w3.org/2001/XMLSchema"); 
		this.smxJndiNamespaces.put("xsi","http://www.w3.org/2001/XMLSchema-instance");
	
	}
	public List<IImResource> importDataSources(){
		List<IImResource> listDsBean = new ArrayList<IImResource>();
		String strURL = SpagicUtilsActivator.getDefault().getPluginPreferences().getString(SpagicPreferenceConstants.P_SMX_GET_RESOURCES);
		//strURL = "http://localhost:8080/getResources/";
		PostMethod post = new PostMethod(strURL);
		String soapBodyContent = "<REQUESTED_RESOURCES><ITEM id='DATASOURCES'/></REQUESTED_RESOURCES>";
		
        RequestEntity entity = new StringRequestEntity(getSoapRequestForXmlString(soapBodyContent));
        post.setRequestEntity(entity);
        // consult documentation for your web service
        post.setRequestHeader("SOAPAction", "");
        // Get HTTP client
        org.apache.commons.httpclient.HttpClient httpclient = new org.apache.commons.httpclient.HttpClient();
        // Execute request
        try {
            int result = httpclient.executeMethod(post);
            // Display status code
            System.out.println("Response status code: " + result);
            // Display response
            System.out.println("Response body: ");
            String soapResponse = post.getResponseBodyAsString();
            
            Document dom = DocumentHelper.parseText(soapResponse);
            
            XPath xPathExpression = new DefaultXPath("/soapenv:Envelope/soapenv:Body/DATASOURCES/DATASOURCE");
			
			xPathExpression.setNamespaceURIs(this.smxJndiNamespaces);
			
			List<Node> dsNodes = xPathExpression.selectNodes(dom);
           
            IImResource res = null;
          
            for (Node dsNode : dsNodes){
            	res = new DataSourceImResource(dsNode.valueOf("@name"));
            	res.setProperty(DataSourceImResource.DATASOURCE_URL,dsNode.valueOf("@url"));
            	res.setProperty(DataSourceImResource.DATASOURCE_DRIVER, dsNode.valueOf("@driver"));
            	res.setProperty(DataSourceImResource.DATASOURCE_USER,dsNode.valueOf("@user"));
            	res.setProperty(DataSourceImResource.DATASOURCE_PASSWORD,dsNode.valueOf("@password"));
            	listDsBean.add(res);
            }
        }catch (Exception e) {
        	SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);
        } finally {
            // Release current connection to the connection pool once you are done
            post.releaseConnection();
        }
        return listDsBean;
	}
	
	public static String getSoapRequestForXmlString(String xmlString){
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
		sb.append("<soapenv:Body>");
		sb.append(xmlString);
		sb.append("</soapenv:Body>");
		sb.append("</soapenv:Envelope>");
		return sb.toString();
	}
	
	
	
	
	
}
