/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.view;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.Catalog;
import org.spagic.metadb.model.Rule;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.hibernate.Session;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.dialog.ListExtractionRulesDialog;
import org.spagic.ui.dialog.NewCatalogObservableAttributeDialog;
import org.spagic.ui.dialog.NewObservableAttributeDialog;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.SpagicLogger;

public class ObservableAttributeView extends ViewPart implements ISelectionListener {
	
	private static Object[] EMPTY_ARRAY = new Object[0];
	private TreeViewer tree = null;
	private Menu menu = null;
	protected Action addRelevantDataAction, removeRelevantDataAction,
			configureRelevantDataAction, addCatalogAction = null;

	public ObservableAttributeView() {
		super();
	}

	@Override
	public void dispose() {
		/*
		 */
	}

	@Override
	public void createPartControl(Composite parent) {

		System.out.println(" ObservableAttributeView View Create Part Control");
		tree = new TreeViewer(parent);
		tree.setContentProvider(new ObservableAttributeViewContentProvider());
		tree.setLabelProvider(new ObservableAttributeViewLabelProvider());
		int ops = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transfers = new Transfer[] { LocalSelectionTransfer
				.getTransfer() };
		ObservableAttributeViewDragAdapter drag = new ObservableAttributeViewDragAdapter();

		tree.addDragSupport(ops, transfers, drag);
		tree.addDropSupport(ops, transfers, new ObservableAttributeViewDropAdapter(tree));

		tree.setInput(getRelevantDataTree());

		createActions();
		createMenus();
		createToolbar();

	}

	public CatalogObservableAttributeTreeElement getRelevantDataTreeFromDataBase() {
		CatalogObservableAttributeTreeElement root = new CatalogObservableAttributeTreeElement("root",
				"Relevant Data");
		Session aSession = null;
		String expressionLanguage = "XPath";
		try{

			aSession = SpagicUIActivator.getDefault().getSessionFactory().openSession();
			List<Catalog> catalogs  = AuditDBUtils.getCatalogs(aSession);
			List<Attribute> attributeForCatalogs = null;
			CatalogObservableAttributeTreeElement aCatalogTreeElement = null;
			ObservableAttributeTreeElement observableAttributeTreeElement = null;
			ExtractionRuleObservableAttribute aRuleElement = null;
			for (Catalog aCatalog : catalogs){
				aCatalogTreeElement = new CatalogObservableAttributeTreeElement(String.valueOf(aCatalog.getIdCatalog()), aCatalog.getName());
				attributeForCatalogs = AuditDBUtils.getAttributesForCatalog(aSession, aCatalog);
				for (Attribute anAttribute: attributeForCatalogs){
					observableAttributeTreeElement = new ObservableAttributeTreeElement(String.valueOf(anAttribute.getIdAttribute()), anAttribute.getName());
					
					for ( Rule ruleForAttribute : anAttribute.getRules()){
						
						 aRuleElement = new ExtractionRuleObservableAttribute(String.valueOf(ruleForAttribute.getIdRule()),
								ruleForAttribute.getName(), ruleForAttribute.getExpr(), expressionLanguage);
						
						if (ruleForAttribute.getConditionExpr() != null){
							aRuleElement.setSubjectToCondition(true);
							aRuleElement.setConditionExpression(ruleForAttribute.getConditionExpr());
							aRuleElement.setConditionLanguage(expressionLanguage);
						}
						aRuleElement.setStartDate(ruleForAttribute.getStartDate());
						aRuleElement.setEndDate(ruleForAttribute.getEndDate());
						observableAttributeTreeElement.addRule(aRuleElement);
					}
					
					aCatalogTreeElement.addRelevantDataTreeElement(observableAttributeTreeElement);
				}
				root.addCatalog(aCatalogTreeElement);
			}
			
			List<Attribute> attributes = AuditDBUtils.getAttributes(aSession);
			for (Attribute anAttribute: attributes){
				observableAttributeTreeElement = new ObservableAttributeTreeElement(String.valueOf(anAttribute.getIdAttribute()), anAttribute.getName());
				
				for ( Rule ruleForAttribute : anAttribute.getRules()){
					
					 aRuleElement = new ExtractionRuleObservableAttribute(String.valueOf(ruleForAttribute.getIdRule()),
							ruleForAttribute.getName(), ruleForAttribute.getExpr(), expressionLanguage);
					
					if (ruleForAttribute.getConditionExpr() != null){
						aRuleElement.setSubjectToCondition(true);
						aRuleElement.setConditionExpression(ruleForAttribute.getConditionExpr());
						aRuleElement.setConditionLanguage(expressionLanguage);
					}
					observableAttributeTreeElement.addRule(aRuleElement);
				}
				root.addRelevantDataTreeElement(observableAttributeTreeElement);
			}
			return root;
		} finally {
			
			if ((aSession != null) && (aSession.isOpen())){
				aSession.close();
			}
		}
	}
	

	public CatalogObservableAttributeTreeElement getRelevantDataTree() {
		// return getMockRelevantDataTree();
		return getRelevantDataTreeFromDataBase();
	}

	@Override
	public void setFocus() {
	}

	protected void createActions() {

		addCatalogAction = new Action("Add New Catalog") {
			public void run() {
				addNewCatalog();
			}
		};
		
		addCatalogAction.setToolTipText("Add New Catalog");
		addCatalogAction.setImageDescriptor(SpagicUIActivator
				.getImageDescriptor("icons/ico_catalog_add.png"));
		
		addRelevantDataAction = new Action("Add New Relevant Data") {
			public void run() {
				addNewRelevantData();
			}
		};
		
		addRelevantDataAction.setToolTipText("Add New Relevant Data");
		addRelevantDataAction.setImageDescriptor(SpagicUIActivator
				.getImageDescriptor("icons/ico_relevant_data_add.png"));

		removeRelevantDataAction = new Action("Remove") {
			public void run() {
				remove();
			}
		};

		removeRelevantDataAction.setToolTipText("Remove Relevant Data / Catalog");
		removeRelevantDataAction.setImageDescriptor(SpagicUIActivator
				.getImageDescriptor("icons/ico_delete.png"));

		configureRelevantDataAction = new Action("Configure Relevant Data") {
			public void run() {
				configureRelevantData();
			}
		};

		configureRelevantDataAction.setToolTipText("Configure Relevant Data");
		configureRelevantDataAction
				.setImageDescriptor(SpagicUIActivator
						.getImageDescriptor("icons/ico_relevant_data_configure.png"));
	}

	protected void configureRelevantData(){
		if (tree.getSelection().isEmpty()) {
			return;
		}
		IStructuredSelection selection = (IStructuredSelection) tree
				.getSelection();

		Object dataOfSelection = ((IStructuredSelection) selection)
				.getFirstElement();

		if ((dataOfSelection != null)
				&& (dataOfSelection instanceof ObservableAttributeTreeElement)) {
			ListExtractionRulesDialog aRelevantDataRulesDialog = new ListExtractionRulesDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), 
																							(ObservableAttributeTreeElement)dataOfSelection,
																							false, 
																							"Configure Rules For Relevant Data");
			aRelevantDataRulesDialog.open();
			this.tree.setInput(getRelevantDataTree());
			this.tree.refresh(true);
		}
	}
	
	protected void addNewRelevantData() {

		NewObservableAttributeDialog newObservableAttributeDialog = new NewObservableAttributeDialog(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
		newObservableAttributeDialog.open();
		if (newObservableAttributeDialog.isStatus()){
			this.tree.setInput(getRelevantDataTree());
			this.tree.refresh(true);
		}
		
	}
	
	protected void addNewCatalog() {
		NewCatalogObservableAttributeDialog newCatalogObservableAttributeDialog = new NewCatalogObservableAttributeDialog(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
		newCatalogObservableAttributeDialog.open();
		if (newCatalogObservableAttributeDialog.isStatus()){
			this.tree.setInput(getRelevantDataTree());
			this.tree.refresh(true);
		}
		
	}

	protected void remove() {
		if (tree.getSelection().isEmpty()) {
			return;
		}
		IStructuredSelection selection = (IStructuredSelection) tree
				.getSelection();

		Object dataOfSelection = ((IStructuredSelection) selection)
				.getFirstElement();

		if ((dataOfSelection != null)
				&& (dataOfSelection instanceof ObservableAttributeTreeElement)) {

			if (MessageDialog.openConfirm(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(),
					"Delete Relevant Data", "Delete Relevant Data "
							+ ((ObservableAttributeTreeElement) dataOfSelection)
									.getName())) {
				
				// Delete Relevant Data From DB
				
				boolean result = removeRelevantData((ObservableAttributeTreeElement) dataOfSelection);

				if (result) {
					tree.setInput(getRelevantDataTree());
					tree.refresh(true);
				}
			}
		}
		if ((dataOfSelection != null)
				&& (dataOfSelection instanceof CatalogObservableAttributeTreeElement)) {
			if (MessageDialog
					.openConfirm(
							PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getShell(),
							"Delete Catalog",
							"The Delete of Catalog "
									+ ((CatalogObservableAttributeTreeElement) dataOfSelection)
											.getName()
									+ " not imply the delete of relevant data attribute but only the association of this with the catalog, Continue ?")) {
				// Delete Relevant Data From DB
				boolean result = removeCatalog((CatalogObservableAttributeTreeElement) dataOfSelection);

				if (result) {
					tree.setInput(getRelevantDataTree());
					tree.refresh(true);
				}
			}
		}

	}

	protected boolean removeCatalog(final CatalogObservableAttributeTreeElement catTreeElement) {
		try{
			MetaDBHelper.doExecuteTx(new IMetaDBCommand(){

				public Object execute(Session aSession) throws Throwable {

					
					Catalog catalogInDB = (Catalog) AuditDBUtils.getObjectById(
					aSession, Catalog.class, Long.valueOf(catTreeElement
							.getId()));

					
			
					AuditDBUtils.deleteCatalog(aSession, catalogInDB);
					return true;
				}
			});
			return true;
		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");
			return false;
		}		
	}

	protected boolean removeRelevantData(
			final ObservableAttributeTreeElement relDataTreeElement) {
		try {
			MetaDBHelper.doExecuteTx(new IMetaDBCommand() {

				public Object execute(Session aSession) throws Throwable {

					// Remove the relations of this catlaog with attributes
					org.spagic.metadb.model.Attribute attributeInDb = (Attribute) AuditDBUtils
							.getObjectById(aSession, Attribute.class, Long
									.valueOf(relDataTreeElement.getId()));

					for (Rule rule : attributeInDb.getRules()) {
						aSession.delete(rule);
						aSession.flush();
					}

					//
					AuditDBUtils.deleteAttribute(aSession, attributeInDb);

					return true;
				}
			});
			return true;

		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");
			return false;
		}
	}

	
	protected void createMenus() {
		IMenuManager menuManager = new MenuManager();
		fillMenu(menuManager);
		this.menu = ((MenuManager) menuManager).createContextMenu(this.tree
				.getTree());
		this.tree.getTree().setMenu(menu);

	}

	protected void fillMenu(IMenuManager rootMenuManager) {
		rootMenuManager.add(addCatalogAction);
		rootMenuManager.add(addRelevantDataAction);
		rootMenuManager.add(configureRelevantDataAction);
		rootMenuManager.add(removeRelevantDataAction);
	}

	protected void createToolbar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
		toolbarManager.add(addCatalogAction);
		toolbarManager.add(addRelevantDataAction);
		toolbarManager.add(configureRelevantDataAction);
		toolbarManager.add(removeRelevantDataAction);
	}

	private class ObservableAttributeViewLabelProvider extends LabelProvider {
		private Map<ImageDescriptor, Image> imageCache = new HashMap<ImageDescriptor, Image>(
				11);

		/*
		 * @see ILabelProvider#getImage(Object)
		 */
		public Image getImage(Object element) {
			ImageDescriptor descriptor = null;
			if (element instanceof CatalogObservableAttributeTreeElement) {
				descriptor = SpagicUIActivator
						.getImageDescriptor("icons/ico_catalog.png");
			} else if (element instanceof ObservableAttributeTreeElement) {
				descriptor = SpagicUIActivator
						.getImageDescriptor("icons/ico_relevant_data.png");
			} else {
				throw unknownElement(element);
			}

			// obtain the cached image corresponding to the descriptor
			Image image = (Image) imageCache.get(descriptor);
			if (image == null) {
				image = descriptor.createImage();
				imageCache.put(descriptor, image);
			}
			return image;
		}

		/*
		 * @see ILabelProvider#getText(Object)
		 */
		public String getText(Object element) {
			if (element instanceof ObservableAttributeTreeModel) {
				return ((ObservableAttributeTreeModel) element).getName();
			} else {
				throw unknownElement(element);
			}
		}

		public void dispose() {
			for (Iterator<Image> i = imageCache.values().iterator(); i
					.hasNext();) {
				((Image) i.next()).dispose();
			}
			imageCache.clear();
		}

		protected RuntimeException unknownElement(Object element) {
			return new RuntimeException(
					"Unknown type of element in tree of type "
							+ element.getClass().getName());
		}

	}

	private class ObservableAttributeViewContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof CatalogObservableAttributeTreeElement) {
				CatalogObservableAttributeTreeElement cat = (CatalogObservableAttributeTreeElement) parentElement;
				return concat(cat.getChildCatalogs().toArray(), cat
						.getChildRelevantData().toArray());
			}
			return EMPTY_ARRAY;
		}

		protected Object[] concat(Object[] object, Object[] more) {
			Object[] both = new Object[object.length + more.length];
			System.arraycopy(object, 0, both, 0, object.length);
			System.arraycopy(more, 0, both, object.length, more.length);
			return both;
		}

		public Object getParent(Object element) {
			if (element instanceof ObservableAttributeTreeModel) {
				return ((ObservableAttributeTreeModel) element).getParent();
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		public void dispose() {

		}

		public void inputChanged(Viewer arg0, Object arg1, Object arg2) {

		}

	}

	
	

	/**
	 * Simple drag adapter that fills the LocalSelectionTransfer with the
	 * selection
	 */
	private class ObservableAttributeViewDragAdapter extends DragSourceAdapter {
		@Override
		public void dragStart(org.eclipse.swt.dnd.DragSourceEvent event) {

			System.out.println("=== Drag Start ===");
			LocalSelectionTransfer.getTransfer().setSelection(
					tree.getSelection());
		}
	}

	private class ObservableAttributeViewDropAdapter extends ViewerDropAdapter {

		protected CatalogObservableAttributeTreeElement catalogObservableAttributeTreeElement = null;

		protected ObservableAttributeViewDropAdapter(Viewer viewer) {
			super(viewer);
		}
		
		protected boolean linkObservableAttributeWithCatalog(final ObservableAttributeTreeElement observableAttributeTreeElement,
														  final CatalogObservableAttributeTreeElement catalogObservableAttributeTreeElement ){
			
			boolean result = (Boolean)MetaDBHelper.doExecuteTx(new IMetaDBCommand(){
			public Object execute(Session aSession) throws Throwable {
				Attribute attribute =(Attribute)AuditDBUtils.getObjectById(aSession,Attribute.class, Long.valueOf(
								((ObservableAttributeTreeElement)observableAttributeTreeElement).getId()));
				Catalog catalog = (Catalog)AuditDBUtils.getObjectById(aSession, Catalog.class, Long.valueOf(catalogObservableAttributeTreeElement.getId()));
				if (AuditDBUtils.isAttributeLinkedToCatalog(aSession, attribute, catalog))
					return false;
				AuditDBUtils.linkAttributeToCatalog(aSession, attribute, catalog);
				
				return true;	
			}
			
			
		});
			return result;
		
		}
		@Override
		public boolean performDrop(Object data) {
		
			Object dataOfSelection = null;
			try {
				if (data instanceof IStructuredSelection) {
					dataOfSelection = ((IStructuredSelection) data)
							.getFirstElement();
				}
				if ((dataOfSelection != null)
						&& (dataOfSelection instanceof ObservableAttributeTreeElement)) {
					if (MessageDialog
							.openConfirm(
									PlatformUI.getWorkbench()
											.getActiveWorkbenchWindow()
											.getShell(),
									"Spagic",
									"Associate Attribute "
											+ ((ObservableAttributeTreeElement) dataOfSelection)
													.getName()
											+ " with Catalog ["
											+ catalogObservableAttributeTreeElement.getName()
											+ "]")) {

						if (linkObservableAttributeWithCatalog((ObservableAttributeTreeElement)dataOfSelection,catalogObservableAttributeTreeElement)){
							tree.setInput(getRelevantDataTree());
							tree.refresh(true);
						}else{
							MessageDialog.openError(PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getShell(), "Spagic",
									"The attribute is already associated with this catalog ");
						}
						
					}
				}
			} catch (Exception e) {
				SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
				MessageDialog.openError(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(), "Spagic",
						"An error is occurred [" + e.getMessage() + "]");
				return false;
			}	

			return true;

		}
		
		@Override
		public boolean validateDrop(Object target, int op,
				TransferData transferData) {

			if (target instanceof CatalogObservableAttributeTreeElement) {
				System.out.println(" Target [" + target
						+ "] CatalogObservableAttributeTreeElement");
				catalogObservableAttributeTreeElement = (CatalogObservableAttributeTreeElement) target;
				return true;
			} else {
				return false;
			}
		}

	}

	public void selectionChanged(IWorkbenchPart workbenchPart,
			ISelection selection) {

		System.out.println(" Relevant Data View Selection Changed");

	}

	protected Object transformSelection(Object selected) {

		return selected;

	}

}
