/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.wizards;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.RegistryHelper;



/**
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ClassificationPage extends WizardPage {


	 private CLabel serviceClassificationLabel = null;
	
	 private SashForm internalSash  = null; 
	    
	 private Table tableClassificazioni = null;
	 private TableViewer tbvClassificazioni = null;
	    
	 private Combo aComboTaxonomies = null;
	 private Combo aComboTaxonomiesValues = null;
	    
	 private List<String> taxonomies = null;
	 private String[] aComboTaxonomyItems =  null;
	    
	 private List<String> taxonomiesValues = null;
	 private String[] aComboTaxonomyValuesItems = null;
    
	/** selection. */
	private ISelection selection;
	/** checkbutton to complex sample. */
	private ServiceRegistryWizard wiz = null;

	
	/**
	 * Constructor for SampleNewWizardPage.
	 * @param selectionParam : current selection
	 */
	public ClassificationPage(final ISelection selectionParam) {
		//tell the parent constructor and give it the image to print
		super("Process Name and Organization in Registry" , "Process Name and Organization in Registry",
				SpagicUIActivator.getImageDescriptor(
						"icons" + File.separator
						+ "spagic_logo.jpg"));
		//description
		setDescription("Process Name and Organization in Registry");
		this.selection = selectionParam;
		this.wiz = wiz;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 * @param parent : parent composite
	 */
	public final void createControl(final Composite parent) {
		Composite mainComposite = new Composite( parent, SWT.NONE );
        mainComposite.setLayout( new GridLayout(  ) );
        mainComposite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

        createPageContent( mainComposite );

        setControl( mainComposite );
        setPageComplete( false );
	}
	
	public final void createPageContent(final Composite parent){
		final ClassificationPage thisPage = this;
		
		this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
	    this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
	       	
		Composite aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	aComposite.setLayout(layout);
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		this.serviceClassificationLabel = new CLabel(aComposite,  SWT.RIGHT);
        serviceClassificationLabel.setText(" Taxonomy ");
        aComboTaxonomies = new Combo(aComposite, SWT.READ_ONLY);
        
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = 200;
    	data.horizontalSpan = 4;
    	this.taxonomies = RegistryHelper.getTaxonomies();
    	this.aComboTaxonomyItems = new String[taxonomies.size()];
    	aComboTaxonomyItems  = taxonomies.toArray(aComboTaxonomyItems);
    	aComboTaxonomies.setItems(aComboTaxonomyItems);
    	aComboTaxonomies.setLayoutData(data);
       
    	
    	aComboTaxonomiesValues = new Combo(aComposite, SWT.READ_ONLY);
    	data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = 200;
    	data.horizontalSpan = 4;
    	this.taxonomiesValues = new ArrayList<String>();
    	this.aComboTaxonomyValuesItems = new String[taxonomiesValues.size()];
    	aComboTaxonomyValuesItems  = taxonomiesValues.toArray(aComboTaxonomyValuesItems);
    	aComboTaxonomiesValues.setItems(aComboTaxonomyValuesItems);
    	aComboTaxonomiesValues.setLayoutData(data);
    	
    	Button btnAddClassification = new Button(aComposite, SWT.FLAT);
    	btnAddClassification.setText(" Add Class. ");
    	

    	final Combo theCombo = this.aComboTaxonomies;
    	this.aComboTaxonomies.addSelectionListener(new SelectionAdapter() {    		
            public void widgetSelected(SelectionEvent event) {                   
            	String taxonomy = theCombo.getText();
            	System.out.println(" Selected Taxonomy "+ taxonomy );
            	thisPage.taxonomiesValues = RegistryHelper.getTaxonomyValues(taxonomy);
            
            	thisPage.aComboTaxonomyValuesItems = new String[taxonomiesValues.size()];
            	thisPage.aComboTaxonomyValuesItems  = thisPage.taxonomiesValues.toArray(thisPage.aComboTaxonomyValuesItems);
            	thisPage.aComboTaxonomiesValues.setItems(thisPage.aComboTaxonomyValuesItems);
            }
    	});
         
    	btnAddClassification.addListener(SWT.Selection, 
    		new Listener() {
            	public void handleEvent(Event event) {
            		String taxonomy = thisPage.aComboTaxonomies.getText();
            		String taxValue = thisPage.aComboTaxonomiesValues.getText();
            		if (   (taxonomy == null) 
            			|| (taxonomy.trim().length() == 0)
            			|| (taxValue == null) 
            			|| (taxValue.trim().length() == 0)){
            			
            			MessageDialog.openInformation(
								getShell(),
								"Spagic",
								"To Classify The Service Select a Taxonomy and a taxonomy value");
            		}else{
            			String clsString = taxonomy + ";" + taxValue;
            			if (thisPage.wiz.getClassifications().contains(clsString)){
            				MessageDialog.openInformation(
    								getShell(),
    								"Spagic",
    								"This service is already Classified as ["+taxonomy+"]["+taxValue+"]");
            			}else{
            				thisPage.wiz.getClassifications().add(clsString);
            				tbvClassificazioni.setInput(thisPage.wiz.getClassifications());
            				tbvClassificazioni.refresh();
            			}
            		}
            	}       
            });
       
        this.tableClassificazioni = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION);
        this.tableClassificazioni.setLinesVisible(true);
        this.tableClassificazioni.setHeaderVisible(true);
        TableColumn column = null;
		column = new  TableColumn(tableClassificazioni, SWT.LEFT);
		column.setText("Taxonomy");
		column.setWidth(200);
		column = new TableColumn(tableClassificazioni, SWT.LEFT);
		column.setText("Classification ");
		column.setWidth(200);
        this.tbvClassificazioni = new TableViewer(tableClassificazioni);
        tbvClassificazioni.setContentProvider(new ClassificationContentProvider());
        tbvClassificazioni.setLabelProvider(new ClassificationLabelProvider());
        tbvClassificazioni.setInput(thisPage.wiz.getClassifications());    
    	
        
        this.tableClassificazioni.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = thisPage.tableClassificazioni.getSelection();
        		
        			
        				for (int i=0; i < tis.length; i++){
        					thisPage.wiz.getClassifications().remove(tis[i].getData());
        				}
        				thisPage.tbvClassificazioni.setInput(thisPage.wiz.getClassifications());
        				thisPage.tbvClassificazioni.refresh();
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
	
	}

	

	

	

	

	

	/**
	 * Tests if the current workbench selection is a suitable container to use.
	 */
	private void initialize() {
		if (wiz.isPublishedInRegistry())
			wiz.setClassifications(RegistryHelper.getClassificationForService(this.wiz.getProcess()));
		else
			wiz.setClassifications(new ArrayList<String>());
		tbvClassificazioni.setInput(wiz.getClassifications());
		tbvClassificazioni.refresh();
		setPageComplete(true);
	}

	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */
	private void handleBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
				"Select new file container");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				//containerText.setText(((Path) result[0]).toString());
			}
		}
	}

	

	
	
	@Override
	public void setWizard(IWizard newWizard) {
		this.wiz = (ServiceRegistryWizard)newWizard;
		super.setWizard(newWizard);
	}
	
	@Override
	public void setVisible(boolean visible) {
		if (visible)
			initialize();
		super.setVisible(visible);
	}

	
	public class ClassificationLabelProvider implements ITableLabelProvider
	{
		
		public ClassificationLabelProvider(){
		}
		
		public String getColumnText(Object element, int column_index) {
				 if (element instanceof String) {
					String[] colonne = ((String)element).split(";");
					return colonne[column_index];
					
				 }else{
					 return "";
				 }
	  }

	  public void addListener(ILabelProviderListener ilabelproviderlistener)
	  {
	  }

	  public void dispose()
	  {
	  }

	  public boolean isLabelProperty(Object obj, String s)
	  {
	    return false;
	  }

	  public void removeListener(ILabelProviderListener ilabelproviderlistener)
	  {
	  }

	  public Image getColumnImage(Object element, int column_index)
	  {
	      return null;
	    
	  }
	}
	
	public class ClassificationContentProvider  implements IStructuredContentProvider {
		
		
		
		public ClassificationContentProvider(){
			 super(); 
			
		}
	  
	
		public Object[] getElements(Object element){
			List<String> c = (List<String>) (element); 
			Object[] arr = c.toArray();
			return arr;
		}	

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object old_object, Object new_object)
		{
		}
	  
	  
}
	

}
