/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.wizards;

import java.io.File;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.RegistryHelper;



/**
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ProcessNameAndOrganizationPage extends WizardPage {

	private SashForm internalSash  = null;
	
	private CLabel statusLabel = null;
	private Text statusTextField = null;
	
	private CLabel serviceNameInDatabaseLabel = null;
	private Text serviceNameInDatabaseTextField = null;
    
    private CLabel serviceNameInRegistryLabel = null;
    private Text serviceNameInRegistryTextField = null;
    
  
    
    private CLabel ownerOrganization = null;
    private Combo aComboOrganizations = null;
    
    private List<String> organizations =null;
    private String[] aComboOrganizationItems = null;
    
	/** selection. */
	private ISelection selection;
	/** checkbutton to complex sample. */
	private ServiceRegistryWizard wiz = null;

	
	/**
	 * Constructor for SampleNewWizardPage.
	 * @param selectionParam : current selection
	 */
	public ProcessNameAndOrganizationPage(final ISelection selectionParam) {
		//tell the parent constructor and give it the image to print
		super("Process Name and Organization in Registry" , "Process Name and Organization in Registry",
				SpagicUIActivator.getImageDescriptor(
						"icons" + File.separator
						+ "spagic_logo.jpg"));
		//description
		setDescription("Process Name and Organization in Registry");
		this.selection = selectionParam;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 * @param parent : parent composite
	 */
	public final void createControl(final Composite parent) {
		Composite mainComposite = new Composite( parent, SWT.NONE );
        mainComposite.setLayout( new GridLayout(  ) );
        mainComposite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

        createPageContent( mainComposite );

        setControl( mainComposite );
        setPageComplete( false );
	}
	
	public final void createPageContent(final Composite parent){
		final ProcessNameAndOrganizationPage thisPage = this;

		this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
		Composite aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 5;
    	aComposite.setLayout(layout);
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	GridData data = null;
    	
    	
    	statusLabel = new CLabel(aComposite, SWT.RIGHT);
    	statusLabel.setText(" Status ");
    	
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 4;
    	
    	statusTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	statusTextField.setLayoutData(data);
    	statusTextField.setEditable(false);
    	statusTextField.setText("");
   
    
    	serviceNameInDatabaseLabel = new CLabel(aComposite, SWT.RIGHT);
    	serviceNameInDatabaseLabel.setText(" Process Name ");
    	
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 4;
    	
    	serviceNameInDatabaseTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	serviceNameInDatabaseTextField.setLayoutData(data);
    	serviceNameInDatabaseTextField.setEditable(false);
    	serviceNameInDatabaseTextField.setText("");
    
    	
    	
        // Riga 2
        serviceNameInRegistryLabel = new CLabel(aComposite, SWT.RIGHT);
        serviceNameInRegistryLabel.setText(" Name in Registry as ");
        	
        data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = 200;
        data.horizontalSpan = 4;
        		
        serviceNameInRegistryTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
        serviceNameInRegistryTextField.setLayoutData(data);
        serviceNameInRegistryTextField.setEditable(false);
        serviceNameInRegistryTextField.setText("");
        serviceNameInRegistryTextField.addModifyListener( new ModifyListener() {
        	      public void modifyText(ModifyEvent e) {
        	    	thisPage.wiz.setInRegistryName(((Text)e.widget).getText()); 
        	    	evaluatePageComplete();
        	      }
        });
     
        CLabel filler = new CLabel(aComposite,SWT.RIGHT);
        filler.setText(" ");
        data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = 200;
        data.horizontalSpan = 5;
        filler.setLayoutData(data);
        
        
    	// Riga 1
    	ownerOrganization = new CLabel(aComposite, SWT.RIGHT);
    	ownerOrganization.setText(" Organization ");
    
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 4;
    
    		aComboOrganizations = new Combo(aComposite, SWT.READ_ONLY);
        	this.organizations = RegistryHelper.getOrganizations();
    		this.aComboOrganizationItems = new String[organizations.size()];
    		aComboOrganizationItems  = organizations.toArray(aComboOrganizationItems);
    		aComboOrganizations.setItems(aComboOrganizationItems);
    		aComboOrganizations.setLayoutData(data);
    		aComboOrganizations.addModifyListener(new ModifyListener(){

				public void modifyText(ModifyEvent me) {
					System.out.println(me);
					thisPage.wiz.setInRegistryOrg(((Combo)me.widget).getText());
					evaluatePageComplete();
					
				}

    			
    			
    		});

    	filler = new CLabel(aComposite,SWT.RIGHT);
    	filler.setText(" ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 5;
    	filler.setLayoutData(data);
	}
	

	
	private void initialize() {
		if (( this.wiz.getProcess().getUddiOrg() != null ) && (this.wiz.getProcess().getUddiName() != null)){
			statusTextField.setText(" Published ");
			serviceNameInDatabaseTextField.setText(this.wiz.getProcess() != null ? this.wiz.getProcess().getName() : "");
			serviceNameInRegistryTextField.setText(this.wiz.getProcess().getUddiName());
			serviceNameInRegistryTextField.setEditable(false);
			aComboOrganizations.setText(this.wiz.getProcess().getUddiOrg());
			aComboOrganizations.setEnabled(false);
			setPageComplete(true);
		}else{
			statusTextField.setText(" Not Published ");
			serviceNameInDatabaseTextField.setText(this.wiz.getProcess() != null ? this.wiz.getProcess().getName() : "");
			//Propose as default the process Name in database
			serviceNameInRegistryTextField.setText(this.wiz.getProcess() != null ? this.wiz.getProcess().getName() : "");
			serviceNameInRegistryTextField.setEditable(true);
			aComboOrganizations.setEnabled(true);
			setPageComplete(false);
		}
		evaluatePageComplete();
		
	
	}

	@Override
	public void setWizard(IWizard newWizard) {
		this.wiz = (ServiceRegistryWizard)newWizard;
		super.setWizard(newWizard);
	}

	
	@Override
	public void setVisible(boolean visible) {
		if (visible)
			initialize();
		super.setVisible(visible);
	}

	
	public void evaluatePageComplete(){
		if ((this.aComboOrganizations.getText().trim().length() > 0) 
			&& (this.serviceNameInRegistryTextField.getText().trim().length() > 0)){
			setPageComplete(true);
		}else{
			setPageComplete(false);
		}
	}

	

	

	

	
	
}
