/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.wizards;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Process;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.helpers.IMetaDBCommand;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.SpagicLogger;

/**
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class SelectProcessFromDBPage extends WizardPage {

	private SashForm internalSash  = null; 
	private Table tableProcess = null;
    private TableViewer tbvProcess = null; 
    private List<Process> theProcesses = null;

	/** selection. */
	private ISelection selection;
	/** checkbutton to complex sample. */
	private ServiceRegistryWizard wiz = null;

	@Override
	public void setWizard(IWizard newWizard) {
		this.wiz = (ServiceRegistryWizard)newWizard;
		super.setWizard(newWizard);
	}
	/**
	 * Constructor for SampleNewWizardPage.
	 * @param selectionParam : current selection
	 */
	public SelectProcessFromDBPage(final ISelection selectionParam) {
		//tell the parent constructor and give it the image to print
		super("Select Process Page" , "Select a Process to configure In Registry from List",
				SpagicUIActivator.getImageDescriptor(
						"icons" + File.separator
						+ "spagic_logo.jpg"));
		//description
		setDescription("Select the process you want to configure in business process");
		this.selection = selectionParam;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 * @param parent : parent composite
	 */
	public final void createControl(final Composite parent) {
		Composite mainComposite = new Composite( parent, SWT.NONE );
        mainComposite.setLayout( new GridLayout(  ) );
        mainComposite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

        createPageContent( mainComposite );

        setControl( mainComposite );
        setPageComplete( false );
	}
	
	public final void createPageContent(final Composite parent){
		this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	

        this.theProcesses = retrieveProcessesFromDb();
    	
    	
        this.tableProcess = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableProcess.setLinesVisible(true);
        this.tableProcess.setHeaderVisible(true);
        TableColumn column = null;
		column = new  TableColumn(tableProcess, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new  TableColumn(tableProcess, SWT.LEFT);
		column.setText("Ver.");
		column.setWidth(50);
		column = new  TableColumn(tableProcess, SWT.LEFT);
		column.setText("Published");
		column.setWidth(50);
		this.tableProcess.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
        this.tbvProcess = new TableViewer(tableProcess);
        tbvProcess.setContentProvider(new ProcessContentProvider());
        tbvProcess.setLabelProvider(new ProcessLabelProvider());
        
        tbvProcess.setInput(theProcesses);
        
        final SelectProcessFromDBPage thisPage = this;
        this.tableProcess.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent ev) {
			
				System.out.println("ev.data" + ev.item.getData().getClass().getName());
				thisPage.wiz.setProcess((Process)ev.item.getData());
				thisPage.setPageComplete(true);
			}

			public void widgetSelected(SelectionEvent ev) {
				widgetDefaultSelected(ev);
				
			}
        	
        });
        
        createContextMenu(tbvProcess.getControl());
		
	}
	
	private void createContextMenu( Control menuControl )
    {
        MenuManager menuMgr = new MenuManager( " ServiceRegistry" );

        final SelectProcessFromDBPage thisPage = this;
        menuMgr.add( new Action(" Remove From Registry "){
        			public void run() {
        				if (thisPage.wiz.getProcess() != null && thisPage.wiz.isPublishedInRegistry()){
        					if (removeFromServiceRegistry(thisPage.wiz.getProcess())){
        						thisPage.tbvProcess.setInput(retrieveProcessesFromDb());
        						thisPage.wiz.setProcess(null);
        						thisPage.setPageComplete(false);
        					}
        				}
        				
        			};
        });

        org.eclipse.swt.widgets.Menu menu = menuMgr.createContextMenu( menuControl );
        menuControl.setMenu( menu );
    }

	

	
	public boolean  removeFromServiceRegistry(Process process) {
			
		Session session = null;
		Transaction tx = null;
		org.spagic.registry.utils.IServiceRegistry serviceRegistry = null;
		boolean registryOk = false;
		boolean databaseOk = false;
		Organization uddiOrg = null;
		Service serviceInRegistry = null;
		List<ServiceBinding> bindings = new ArrayList<ServiceBinding>();
		try{
			 
			session = SpagicUIActivator.getDefault().getSessionFactory().openSession();
			tx = session.beginTransaction();
			serviceRegistry = SpagicUIActivator.getDefault().getServiceRegistry();
			Process aProcess = AuditDBUtils.getProcessById(session, process.getIdProcess());
			uddiOrg = serviceRegistry.getOrganizationByName(aProcess.getUddiOrg());
			// Get Services to recover in case of failure
			serviceInRegistry = serviceRegistry.getServiceByOrganizationAndServiceName(uddiOrg, aProcess.getUddiName());
			serviceRegistry.deleteService(uddiOrg, serviceInRegistry);

			bindings.addAll(serviceInRegistry.getServiceBindings());
			registryOk = true;
			
			aProcess.setUddiName(null);
			aProcess.setUddiOrg(null);
			session.flush();
			tx.commit();
			databaseOk = true;
			
			return true;
		}catch (Throwable t) {
			if (registryOk && !databaseOk){
				// PUBLICATION OK IN REGISTRY BUT NOT IN DATABASE
				// RECREATE ON REGISTRY
				try{
					serviceInRegistry = serviceRegistry.publishService(uddiOrg, serviceInRegistry, bindings);
				}catch (Exception e) {
					SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);
				}
			}
			
			
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, t.getMessage(), t);
			return false;
		}finally{
			if (session != null && session.isOpen()){
				session.close();
			}
			serviceRegistry.finalizeRegistry();
			
		}
	
	}
	

	public List<Process> retrieveProcessesFromDb(){
		return (List<Process>) MetaDBHelper.doExecuteNoTx(new IMetaDBCommand(){

			public Object execute(Session session) throws Throwable {
				return AuditDBUtils.getAllProcesses(session); 
			}
		});
	};
	

	

	



	
	public class ProcessContentProvider  implements IStructuredContentProvider {
		
		
		
		public ProcessContentProvider(){
			 super(); 
			
		}
	  
	
		public Object[] getElements(Object element){
			Collection<Process> c = (Collection<Process>)element; 
			Object[] arr = c.toArray();
			return arr;
			
		}	

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object old_object, Object new_object)
		{
		}
	  
	  
	}
	
	public class ProcessLabelProvider implements ITableLabelProvider
	{
		
		public ProcessLabelProvider(){
		}
		
		public String getColumnText(Object element, int column_index) {
			if (element instanceof Process) {
				
				 if (column_index == 0){
					 return String.valueOf(((Process)element).getName());
				 }else if (column_index == 1){
					 return String.valueOf( ((Process)element).getProcessVersion() != null ? ((Process)element).getProcessVersion() : "");
				 } else if (column_index == 2){
					 return (((((Process)element).getUddiOrg() != null)  
							 && (((Process)element).getUddiName() != null)) ? "Yes" : "No");
				 }else{
					 return "";
				 }
			 }else{
				 return "";
			 }	 
			
			
	  }

	  public void addListener(ILabelProviderListener ilabelproviderlistener)
	  {
	  }

	  public void dispose()
	  {
	  }

	  public boolean isLabelProperty(Object obj, String s)
	  {
	    return false;
	  }

	  public void removeListener(ILabelProviderListener ilabelproviderlistener)
	  {
	  }

	  public Image getColumnImage(Object element, int column_index)
	  {
	      return null;
	    
	  }
	}
	

	

	

	
	
}
