/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.wizards;

import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.PropertyStep;
import org.spagic.metadb.model.Step;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.registry.infomodel.ClassificationScheme;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;
import javax.xml.registry.infomodel.ServiceBinding;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.utils.SpagicLogger;

/**
 * @author Andrea Zoppello ( Engineering Ing. Informaztica )
 */
public class ServiceRegistryWizard extends Wizard {

	private org.spagic.metadb.model.Process process = null;
	private boolean publishedInRegistry = false;
	private String inRegistryName = null;
	private String inRegistryOrg  = null;
	
	private SelectProcessFromDBPage selectProcessFromDBPage;
	private ProcessNameAndOrganizationPage processNameAndOrganizationPage;
	private ClassificationPage classificationPage;
	
	private List<String> classifications =null;
	
	/** current selection of wizard. */
	private ISelection selection;

	/**
	 * Default constructor for SpagicNewProjectWizard.
	 */
	public ServiceRegistryWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/**
	 * Adding the page to the wizard.
	 */
	public final void addPages() {
		try{
			super.addPages();
	    	selectProcessFromDBPage = new SelectProcessFromDBPage(selection);
	    	
	    	processNameAndOrganizationPage = new ProcessNameAndOrganizationPage(selection);
	    	classificationPage = new ClassificationPage(selection);
	    	
	    	addPage(selectProcessFromDBPage);
	    	addPage(processNameAndOrganizationPage);
	    	addPage(classificationPage);
	     
	   	}
	   	catch(Exception x)
	   	{
	   		reportError(x);
	   	}
	   }

	
	public boolean performFinish()
	{
	   try
	   {
	      WorkspaceModifyOperation op =
	         new WorkspaceModifyOperation()
	      {
	         protected void execute(IProgressMonitor monitor)
	         {
	        	 if (!(publishedInRegistry)){
	        		 publishToRegistry();
	        	 }
	        	 classifyService();
	         }
	      };
	      getContainer().run(false,true,op);
	      MessageDialog.openInformation(getShell(), "Spagic", "Business Registry Configuration Finished");
	   }
	   catch(InvocationTargetException x)
	   {
		  reportError(x);
	      return false;
	   }
	   catch(InterruptedException x)
	   {
		  reportError(x);
	      return false;
	   }
	   return true; 
	}
	

	   
	  

	

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 * @param workbench : the workbench
	 * @param selectionParam : current selection
	 */
	public final void init(
			final IWorkbench workbench,
			final IStructuredSelection selectionParam) {
		setNeedsProgressMonitor(true);
	}

	
	
	
	
	/**
	    * Displays an error that occured during the project creation.
	    * @param x details on the error
	    */
	private void reportError(Exception x){
	      ErrorDialog.openError(getShell(),
	                            "Error",
	                            "Error in Creating New Project",
	                            makeStatus(x));
    }
	   
	   
	public static IStatus makeStatus(Exception x) {
		   		
	         return new Status(IStatus.ERROR,
	                            SpagicUIActivator.PLUGIN_ID,
	                            IStatus.ERROR,
	                            x.getMessage(), null);

	}	

	public org.spagic.metadb.model.Process getProcess() {
		return process;
	}

	public void setProcess(org.spagic.metadb.model.Process process) {
		this.process = process;
		publishedInRegistry = (process.getUddiName()) != null && (process.getUddiOrg() != null);  
	}

	public String getInRegistryName() {
		return inRegistryName;
	}

	public void setInRegistryName(String inRegistryName) {
		this.inRegistryName = inRegistryName;
	}

	public String getInRegistryOrg() {
		return inRegistryOrg;
	}

	public void setInRegistryOrg(String inRegistryOrg) {
		this.inRegistryOrg = inRegistryOrg;
	}

	public List<String> getClassifications() {
		return classifications;
	}

	public void setClassifications(List<String> classifications) {
		this.classifications = classifications;
	}

	public boolean publishToRegistry() {
		Session session = null;
		Transaction tx = null;
		IServiceRegistry serviceRegistry = null;
		boolean registryOk = false;
		boolean databaseOk = false;
		Organization uddiOrg = null;
		Service serviceInRegistry = null;
		try{
			session = SpagicUIActivator.getDefault().getSessionFactory().openSession();
			tx = session.beginTransaction();
			serviceRegistry = SpagicUIActivator.getDefault().getServiceRegistry();
			
			Process aProcess = AuditDBUtils.getProcessById(session,this.process.getIdProcess());
			
			
		    uddiOrg = serviceRegistry.getOrganizationByName(inRegistryOrg);
			
			String description = "Description for " + inRegistryName;
			serviceInRegistry = SpagicUIActivator.getDefault().getServiceRegistry().newService(inRegistryName,description );
			List<ServiceBinding> bindings = new ArrayList<ServiceBinding>();
			
			
			String uri = null;
			
			for (Step step : aProcess.getSteps()){
				if (step.isIsStart()){
					for (PropertyStep propStep : step.getPropertySteps()){
						if (propStep.getProperty().getCode().equalsIgnoreCase("Uri")){
							uri = propStep.getProperty().getValue();
							if ((uri != null) && (uri.trim().length() > 0)){
								ServiceBinding binding = serviceRegistry.newServiceBinding(inRegistryName, uri);
								bindings.add(binding);
							}
						}
					}
				}
			}
			serviceInRegistry = serviceRegistry.publishService(uddiOrg, serviceInRegistry, bindings);
			registryOk = true;
			
			aProcess.setUddiName(inRegistryName);
			aProcess.setUddiOrg(inRegistryOrg);
			this.process.setUddiName(inRegistryName);
			this.process.setUddiOrg(inRegistryOrg);
			
			AuditDBUtils.genericStore(session, aProcess);
			session.flush();
			tx.commit();
			databaseOk = true;
			
			return true;
		}catch (Throwable t) {
			if (registryOk && !databaseOk){
				// PUBLICATION OK IN REGISTRY BUT NOT IN DATABASE
				// DELETE FROM REGISTRY
				try{
					serviceRegistry.deleteService(uddiOrg, serviceInRegistry);
				}catch (Exception e) {
					SpagicLogger.error(SpagicUIActivator.PLUGIN_ID,e.getMessage(), e);
				}
			}
			
			
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, t.getMessage(), t);
			return false;
		}finally{
			if (session != null && session.isOpen()){
				session.close();
			}
			serviceRegistry.finalizeRegistry();
			
		}
	}
	
	public boolean classifyService() {
		IServiceRegistry serviceRegistry =  null;
		try{
			serviceRegistry = SpagicUIActivator.getDefault().getServiceRegistry();
			serviceRegistry.initRegistry();
			Organization org = serviceRegistry.getOrganizationByName(process.getUddiOrg());
			Service serviceInRegistry =  serviceRegistry.getServiceByOrganizationNameAndServiceName(process.getUddiOrg(), process.getUddiName());
			
			serviceRegistry.removeAllClassificationForService(serviceInRegistry);
			
			String[] taxValue = null;
			ClassificationScheme cScheme = null;
			for (String clsString : this.classifications){
				taxValue = clsString.split(";");
				cScheme = serviceRegistry.getClassificationSchemeByName(taxValue[0]);
				serviceRegistry.classifyServiceByTaxonomy(serviceInRegistry, cScheme, taxValue[1]);
			}
			return true;
		}catch(Throwable e){
			return false;
		}finally{
			serviceRegistry.finalizeRegistry();
		}
	}

	public boolean isPublishedInRegistry() {
		return publishedInRegistry;
	}

	public void setPublishedInRegistry(boolean publishedInRegistry) {
		this.publishedInRegistry = publishedInRegistry;
	}
	
}
