/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.wizards;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.spagic.ui.SpagicUIActivator;

/**
 * @author Andrea Zoppello ( Engineering Ing. Informaztica )
 */
public class SpagicNewProjectWizard extends Wizard implements INewWizard {

	
	private WizardNewProjectCreationPage namePage;
	/** current selection of wizard. */
	private ISelection selection;

	/**
	 * Default constructor for SpagicNewProjectWizard.
	 */
	public SpagicNewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/**
	 * Adding the page to the wizard.
	 */
	public final void addPages() {
		try{
		super.addPages();
	      namePage = new WizardNewProjectCreationPage("New Spagic 2.0 Project");
	      namePage.setTitle("New Spagic 2.0 Project");
	      namePage.setDescription("Create a Project With Default Spagic Project Layout");

	      addPage(namePage);
	   	}
	   	catch(Exception x)
	   	{
	   		reportError(x);
	   	}
	   }

	
	public boolean performFinish()
	{
	   try
	   {
	      WorkspaceModifyOperation op =
	         new WorkspaceModifyOperation()
	      {
	         protected void execute(IProgressMonitor monitor)
	         {
	            createProject(monitor != null ?
	                          monitor : new NullProgressMonitor());
	         }
	      };
	      getContainer().run(false,true,op);
	   }
	   catch(InvocationTargetException x)
	   {
		  reportError(x);
	      return false;
	   }
	   catch(InterruptedException x)
	   {
		  reportError(x);
	      return false;
	   }
	   return true; 
	}
	

	   protected void createProject(IProgressMonitor monitor)
	   {
	      monitor.beginTask("Creating Project",50);
	      try
	      {
	         IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
	         monitor.subTask("Creating Project Directories ");
	         IProject project = root.getProject(namePage.getProjectName());
	         IProjectDescription description = ResourcesPlugin.getWorkspace().newProjectDescription(project.getName());
	         if(!Platform.getLocation().equals(namePage.getLocationPath()))
	            description.setLocation(namePage.getLocationPath());
	         
	         project.create(description,monitor);
	         monitor.worked(10);
	         project.open(monitor);
	        
	         monitor.worked(10);
	         IPath projectPath = project.getFullPath();
	         IPath integrationProcessPath = projectPath.append("IntegrationProcess");
	         IPath bpelProjectPath = projectPath.append("Bpel");
	         IPath semanticRulesPath = projectPath.append("SemanticRules");
	         IPath syntaxRulesPath = projectPath.append("SyntaxRules");
	         IPath scriptsPath = projectPath.append("Scripts");
	         IPath mappingsPath = projectPath.append("Mappings");
	         IPath wsdlFilesPath = projectPath.append("WsdlFiles");
	         
	         IFolder integrationProcessPathFolder = root.getFolder(integrationProcessPath);
	         IFolder bpelPathFolder = root.getFolder(bpelProjectPath);
	         IFolder semanticRulesPathFolder = root.getFolder(semanticRulesPath );
	         IFolder syntaxRulesPathFolder = root.getFolder(syntaxRulesPath);
	         IFolder scriptsPathPathFolder = root.getFolder(scriptsPath);
	         IFolder mappingsPathPathFolder = root.getFolder(mappingsPath);
	         IFolder wsdlFilesPathPathFolder = root.getFolder(wsdlFilesPath);
	              
	         
	         createFolderHelper(integrationProcessPathFolder, monitor);
	         createFolderHelper(bpelPathFolder, monitor);
	         createFolderHelper(semanticRulesPathFolder,monitor);
	         createFolderHelper(syntaxRulesPathFolder,monitor);
	         createFolderHelper(scriptsPathPathFolder,monitor);
	         createFolderHelper(mappingsPathPathFolder,monitor);
	         createFolderHelper(wsdlFilesPathPathFolder,monitor);
	        
	      }
	      catch(CoreException x)
	      {
	         reportError(x);
	      }
	      finally
	      {
	         monitor.done();
	      }
	   }

	   /**
	    * Helper method: it recursively creates a folder path.
	    * @param folder
	    * @param monitor
	    * @throws CoreException
	    * @see java.io.File#mkdirs()
	    */
	   private void createFolderHelper(IFolder folder,IProgressMonitor monitor)
	      throws CoreException
	   {
	      if(!folder.exists())
	      {
	         IContainer parent = folder.getParent();
	         if(parent instanceof IFolder
	            && (!((IFolder)parent).exists()))
	            createFolderHelper((IFolder)parent,monitor);
	         folder.create(false,true,monitor);
	      }
	   }

	/**
	 * method to throw real exception.
	 * @param message : error message
	 * @throws CoreException : the wanted exception
	 */
	private void throwCoreException(final String message) throws CoreException {
		IStatus status = null;
		status = new Status(
				IStatus.ERROR, "Spagic",
				IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 * @param workbench : the workbench
	 * @param selectionParam : current selection
	 */
	public final void init(
			final IWorkbench workbench,
			final IStructuredSelection selectionParam) {
		setNeedsProgressMonitor(true);
	}

	
	
	
	
	/**
	    * Displays an error that occured during the project creation.
	    * @param x details on the error
	    */
	   private void reportError(Exception x)
	   {
	      ErrorDialog.openError(getShell(),
	                            "Error",
	                            "Error in Creating New Project",
	                            makeStatus(x));
	   }
	   
	   
	   public static IStatus makeStatus(Exception x)
	   {
		   		
	         return new Status(IStatus.ERROR,
	                            SpagicUIActivator.PLUGIN_ID,
	                            IStatus.ERROR,
	                            x.getMessage(), null);

	   }

	   

	

	
}
