/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.utils.preference;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.spagic.utils.SpagicUtilsActivator;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class SpagicPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public SpagicPreferencePage() {
		
		super(GRID);
		setPreferenceStore(SpagicUtilsActivator.getDefault().getPreferenceStore());
		setDescription("Preferences For Spagic Studio Plugin");
	
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {

		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_BASE_NAME_SPACE, "Spagic Base Name Space Prefix", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_SERVICE_MANAGER_HOST, "Spagic Service Manager Host", getFieldEditorParent()));
		
		addField(
			new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_URL, "Spagic DB URL", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER, "Spagic DB DRIVER", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_USER, "Spagic DB USER", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD, "Spagic DB PASSWORD", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_HIB_DIALECT, "Spagic DB DIALECT", getFieldEditorParent()));
		addField(
				new BooleanFieldEditor(
					SpagicPreferenceConstants.P_SPAGIC_DB_SHOW_SQL,
					"Show sql",
					getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY_INQUIRY_URL, "Service Registry Inquiry URL", getFieldEditorParent()));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PUBLISH_URL, "Service Registry Publish URL", getFieldEditorParent()));

	
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY_USER, "Service Registry User", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY_PASSWORD, "Service Registry Password", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY_DEFAULT_ORG, "Default Organization", getFieldEditorParent()));
		
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE, "Graphviz dot program Location", getFieldEditorParent()));
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER, "Graphviz temp folder", getFieldEditorParent()));
		
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SMX_GET_RESOURCES, "SMX GetResources Service ", getFieldEditorParent()));
		
		
	}

	/** (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	@Override
	protected void contributeButtons(Composite parent) {
		super.contributeButtons(parent);
	}
	
}